/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/AddSmStorageConnectionCommand.java#4 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmAddStorageConnectionRequest;
import com.netapp.smcore.plugin.contracts.SmAddStorageConnectionResponse;
import com.netapp.smcore.plugin.contracts.SmOperationMode;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.parameters.EmsAsupConfigParameters;
import com.netapp.snapcenter.cli.command.parameters.NotificationParameters;
import com.netapp.snapcenter.cli.contracts.Protocol;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.addsmstorageconnection.command.description")
public class AddSmStorageConnectionCommand extends AbstractSMSCommand {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(AddSmStorageConnectionCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_ADDSMSTORAGECONNECTION_COMMAND_NAME);

    @Parameter(names = { "-SVM" }, descriptionKey = "cli.addsmstorageconnection.command.svm.option.description", required = true)
    private String storageVM;

    @Parameter(names = { "-Protocol" }, descriptionKey = "cli.addsmstorageconnection.command.protocol.option.description", required = true)
    private Protocol protocol;

    @Parameter(names = { "-Port" }, descriptionKey = "cli.addsmstorageconnection.command.port.option.description")
    private Integer port;

    @Parameter(names = { "-Username" }, descriptionKey = "cli.addsmstorageconnection.command.username.option.description")
    private String svmUserName;

    private SmOperationMode smOperationMode = SmOperationMode.NEW;

    /*
     * TODO: Should decide on having password option exposed
     */
    @Parameter(names = { "-Password" }, descriptionKey = "cli.addsmstorageconnection.command.password.option.description", hidden = true)
    private String svmPassword;

    @Parameter(names = { "-PreferredIpAddress" }, descriptionKey = "cli.addsmstorageconnection.command.preferredipaddress.option.description")
    private String preferredIpAddress;

    @Parameter(names = { "-Timeout" }, descriptionKey = "cli.addsmstorageconnection.command.timeout.option.description")
    private int timeout = STORAGE_CONNECTION_TIMEOUT_IN_SECONDS;

    @ParametersDelegate
    private EmsAsupConfigParameters emsAsupConfigParameters = new EmsAsupConfigParameters();

	@Parameter(names = { "-IsSecondary" }, descriptionKey = "cli.addsmstorageconnection.command.secondary.option.description")
	private boolean isSecondary;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() {

        if (getSvmUserName() == null) {
            String user = CLIUtils
                    .readLineFromConsole(
                            "%s: ",
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_ADDORSETSMSTORAGECONNECTION_PROMPT_USER_SVM_USERNAME));
            setSvmUserName(user);
        }
        if (getSvmPassword() == null) {
            String pass = CLIUtils
                    .readPasswordFromConsole(
                            "%s: ",
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_ADDORSETSMSTORAGECONNECTION_PROMPT_USER_SVM_PASSWORD));
            setSvmPassword(pass);
        }

        /*
         * If port not specified then set it to standard port number based on
         * protocol
         */
        if (getPort() == null) {
            int defPort = (getProtocol() == Protocol.HTTPS) ? 443 : 80;
            setPort(defPort);
            String displayUsingPortDefault = MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_STORAGE_CONNECTION_USING_PORT_AS_DEFAULT, getPort());
            CLIUtils.writeToLogAndConsole(displayUsingPortDefault, logger,
                    LogLevel.INFO);
        }

    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmAddStorageConnectionRequest, SmAddStorageConnectionResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmAddStorageConnectionRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmAddStorageConnectionResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_ADD_STORAGE_CONNECTION_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    public String getStorageVM() {
        return storageVM;
    }

    public void setStorageVM(String storageVM) {
        this.storageVM = storageVM;
    }

    public Protocol getProtocol() {
        return protocol;
    }

    public void setProtocol(Protocol protocol) {
        this.protocol = protocol;
    }

    public Integer getPort() {
        return port;
    }

    public void setPort(Integer port) {
        this.port = port;
    }

    public void setSvmUserName(String svmUserName) {
        this.svmUserName = svmUserName;
    }

    public void setSvmPassword(String svmPassword) {
        this.svmPassword = svmPassword;
    }

    public String getSvmUserName() {
        return svmUserName;
    }

    public String getSvmPassword() {
        return svmPassword;
    }

    public String getPreferredIpAddress() {
        return preferredIpAddress;
    }

    public void setPreferredIpAddress(String preferredIpAddress) {
        this.preferredIpAddress = preferredIpAddress;
    }

    public int getTimeout() {
        return timeout;
    }

    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }

    public boolean getIsSecondary() {
		return isSecondary;
	}

	public void setIsSecondary(boolean isSecondary) {
		this.isSecondary = isSecondary;
	}

    public SmOperationMode getSmOperationMode() {
        return smOperationMode;
    }

    public void setSmOperationMode(SmOperationMode smOperationMode) {
        this.smOperationMode = smOperationMode;
    }
    
    public EmsAsupConfigParameters getEmsAsupConfigParameters() {
        return emsAsupConfigParameters;
    }

    public void setEmsAsupConfigParameters(
            EmsAsupConfigParameters emsAsupConfigParameters) {
        this.emsAsupConfigParameters = emsAsupConfigParameters;
    }

}
