/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmBackupCommand.java#4 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper.ORACLE;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackup;
import com.netapp.smcore.plugin.contracts.SmBackup;
import com.netapp.smcore.plugin.contracts.SmCatalogingStatus;
import com.netapp.smcore.plugin.contracts.SmGetBackupRequest;
import com.netapp.smcore.plugin.contracts.SmGetBackupResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmVerificatinResultEnum;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmbackup.command.description")
public class GetSmBackupCommand extends AbstractSMSCommand {

    private static final String NOT_MOUNTED = "Not Mounted";
    private static final String MOUNTED = "Mounted";

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_COMMAND_NAME);

    /*
     * TODO: How to handle if user gives conflicting backup name and backup id
     * at the same time. Should Clarify from server
     */
    @Parameter(names = { "-BackupName" }, descriptionKey = "cli.getsmbackup.command.backupName.option.description")
    private String backupName;

    @Parameter(names = { "-BackupId" }, descriptionKey = "cli.getsmbackup.command.backupId.option.description")
    private String backupId;

    @Parameter(names = { "-AppObjectName" }, descriptionKey = "cli.getsmbackup.command.appObjectName.option.description")
    private String appObjectName;

    @Parameter(names = { "-AppObjectId" }, descriptionKey = "cli.getsmbackup.command.appObjectId.option.description")
    private String appObjectId;

    @Parameter(names = { "-Secondary" }, descriptionKey = "cli.getsmbackup.command.secondary.option.description")
    private boolean secondary;

    @Parameter(names = { "-BackupType" }, descriptionKey = "cli.getsmbackup.command.backupType.option.description")
    private String backupType;

    @Parameter(names = { "-JobId" }, descriptionKey = "cli.getsmbackup.command.jobId.option.description")
    private Long jobId;

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.getsmbackup.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    @Parameter(names = { "-ListMountInfo" }, descriptionKey = "cli.getsmbackup.command.listMountInfo.option.description")
    private boolean listMountInfo;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
        switch (pluginCode) {

            case SCO:
                break;

            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        pluginCode.name()));
        }
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetBackupRequest, SmGetBackupResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmGetBackupRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmGetBackupResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_BACKUPS_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse)
            throws CLIException {

        SmGetBackupResponse smGetBackupResponse = (SmGetBackupResponse) smResponse;
        ArrayOfSmBackup arrayOfSmBackup = smGetBackupResponse.getBackups();
        if (arrayOfSmBackup == null
                || CLIUtils.isNullOrEmptyCollection(arrayOfSmBackup
                        .getSmBackup())) {
            return;
        }

        String headerBackupId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_BACKUP_ID_HEADER);
        String headerBackupName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_BACKUP_NAME_HEADER);
        String headerBackupStartTime = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_BACKUP_START_TIME_HEADER);
        String headerBackupEndTime = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_BACKUP_END_TIME_HEADER);
        String headerBackupType = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_BACKUP_TYPE_HEADER);
        String headerVerificationStatus = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_VERIFICATION_STATUS_HEADER);
        String headerCatalogingStatus = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_CATALOGING_STATUS_HEADER);
        String headerMountStatus = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_MOUNT_STATUS_HEADER);
        String headerMountPath = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_MOUNT_PATH_HEADER);
        String headerMountHost = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMBACKUP_TABLE_MOUNT_HOST_HEADER);

        String[] backupTableHeaders = { headerBackupId, headerBackupName,
                headerBackupStartTime, headerBackupEndTime, headerBackupType,
                headerVerificationStatus, headerCatalogingStatus,
                headerMountStatus, headerMountPath, headerMountHost };
        TextTableInterface backupTableFormatter = new TextTable(
                backupTableHeaders);

        /* Sorting based on backup time so that latest backup will be at top. */
        Collections.sort(arrayOfSmBackup.getSmBackup(),
                new Comparator<SmBackup>() {

                    @Override
                    public int compare(SmBackup arg1, SmBackup arg2) {
                        XMLGregorianCalendar backupTime1 = arg1.getBackupTime();
                        XMLGregorianCalendar backupTime2 = arg2.getBackupTime();                        
                        if (backupTime1 == null && backupTime2 == null) {
                            return 0;
                        } else if (backupTime1 != null && backupTime2 == null) {
                            return 1;
                        } else if (backupTime1 == null && backupTime2 != null) {
                            return -1;
                        } else {
                            return backupTime2.compare(backupTime1);
                        }
                    }
                });
        for (SmBackup smBackup : arrayOfSmBackup.getSmBackup()) {

            if (pluginCode.equals(PluginCode.SCO)
                    && StringUtils.isEmpty(smBackup.getBackupType())
                    || !smBackup.getBackupType().toLowerCase()
                            .contains(ORACLE.toLowerCase())) {
                continue;
            }

            String columnBackupId = CLIUtils.getStringValue(smBackup
                    .getBackupId());
            String columnBackupName = CLIUtils.getStringValue(smBackup
                    .getBackupName());
            String columnBackupStartTime = CLIUtils
                    .localizeDateAndGetFormattedString(smBackup.getStartime());
            String columnBackupEndTime = CLIUtils
                    .localizeDateAndGetFormattedString(smBackup.getEndtime());
            String columnBackupType = CLIUtils.getStringValue(smBackup
                    .getBackupType());
            String columnVerificationStatus = CLIUtils.getVerificationStatus(smBackup
                    .getVerifyResultSummary());
            String columnCatalgingStatus = CLIUtils.getCatalogingStatus(smBackup
                    .getCatalogBackupStatus());
            String columnMountStatus = smBackup.isIsMounted() == null
                    || smBackup.isIsMounted().booleanValue() == false ? NOT_MOUNTED
                    : MOUNTED;
            String columnMountPath = CLIUtils.getStringValue(smBackup
                    .getMountedPath());
            String columnMountHost = CLIUtils.getStringValue(smBackup
                    .getMountedHostName());

            backupTableFormatter.addRow(columnBackupId, columnBackupName,
                    columnBackupStartTime, columnBackupEndTime,
                    columnBackupType, columnVerificationStatus, columnCatalgingStatus,
                    columnMountStatus, columnMountPath, columnMountHost);
        }

        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            List<String> displayCols = new ArrayList<>();
            if (isListMountInfo()) {
                displayCols.add(headerBackupName);
                displayCols.add(headerMountStatus);
                displayCols.add(headerMountPath);
                displayCols.add(headerMountHost);
            } else {
                displayCols.add(headerBackupId);
                displayCols.add(headerBackupName);
                displayCols.add(headerBackupStartTime);
                displayCols.add(headerBackupEndTime);
                displayCols.add(headerBackupType);
                displayCols.add(headerVerificationStatus);
                displayCols.add(headerCatalogingStatus);
            }
            backupTableFormatter.displayFormattedTable(displayCols);
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(arrayOfSmBackup);
        }
    }

    public String getBackupName() {
        return backupName;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public String getBackupId() {
        return backupId;
    }

    public void setBackupId(String backupId) {
        this.backupId = backupId;
    }

    public String getAppObjectName() {
        return appObjectName;
    }

    public void setAppObjectName(String appObjectName) {
        this.appObjectName = appObjectName;
    }

    public String getAppObjectId() {
        return appObjectId;
    }

    public void setAppObjectId(String appObjectId) {
        this.appObjectId = appObjectId;
    }

    public boolean isSecondary() {
        return secondary;
    }

    public void setSecondary(boolean secondary) {
        this.secondary = secondary;
    }

    public String getBackupType() {
        return backupType;
    }

    public void setBackupType(String backupType) {
        this.backupType = backupType;
    }

    public Long getJobId() {
        return jobId;
    }

    public void setJobId(Long jobId) {
        this.jobId = jobId;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public void setListMountInfo(boolean listMountInfo) {
        this.listMountInfo = listMountInfo;
    }

    public boolean isListMountInfo() {
        return listMountInfo;
    }
}