/*
 * $Id: //depot/prod/orion/sc_main/mintaka/spl/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmBackupDetailsCommand.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper.ORACLE_DATA_BACKUP_TYPE;
import static com.netapp.snapcenter.cli.utils.CLIUtils.NEW_LINE;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackup;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackupComponentInfo;
import com.netapp.smcore.plugin.contracts.SmBackup;
import com.netapp.smcore.plugin.contracts.SmGetBackupDetailsRequest;
import com.netapp.smcore.plugin.contracts.SmGetBackupDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmKeyValue;
import com.netapp.smcore.plugin.contracts.SmKeyValueCollection;
import com.netapp.smcore.plugin.contracts.SmOracleAbstractTableSpaceConfiguration;
import com.netapp.smcore.plugin.contracts.SmOracleBackupComponentInfo;
import com.netapp.smcore.plugin.contracts.SmOracleDatabaseType;
import com.netapp.smcore.plugin.contracts.SmOracleDb;
import com.netapp.smcore.plugin.contracts.SmOracleDbEnhanced;
import com.netapp.smcore.plugin.contracts.SmOraclePluggableDatabase;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmString;
import com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandResponseValidationUtil;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.OracleDatabaseDetailsUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmbackupdetails.command.description")
public class GetSmBackupDetailsCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPDETAILS_COMMAND_NAME);

    public static final String SCO_CATALOG_METADATA_DATAFILE_TAG_KEY = "CATALOGED_DATAFILES_TAG";

    public static final String SCO_CATALOG_METADATA_CONTROLFILE_PATH_KEY = "CATALOGED_CONTROLFILE_PATH";

    public static final String SCO_CATALOG_METADATA_ARCHIVELOG_DESTINATIONS_KEY = "CATALOGED_ARCHIVELOG_DESTINATIONS";

    @Parameter(names = { "-AppObjectId" }, descriptionKey = "cli.getsmbackupdetails.command.appObjectId.option.description", required = true)
    private String appObjectId;

    //@Parameter(names = { "-BackupName" }, descriptionKey = "cli.getsmbackupdetails.command.backupName.option.description")
    private String backupName;

    @Parameter(names = { "-BackupId" }, descriptionKey = "cli.getsmbackupdetails.command.backupId.option.description", required=true)
    private String backupId;

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.getsmbackupdetails.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    @Parameter(names = { "-Secondary" }, descriptionKey = "cli.getsmbackupdetails.command.secondary.option.description")
    private boolean secondary;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetBackupDetailsRequest, SmGetBackupDetailsResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmGetBackupDetailsRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmGetBackupDetailsResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_BACKUP_DETAILS_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected void processCommandArguments() throws CLIException {

        if (!SmsCommandArgumentsValidationUtil.isValidAppObjectId(appObjectId)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_COMMAND_PARAMETER_APP_OBJECT_ID_NOT_VALID));
        }

        if (StringUtils.isEmpty(getBackupName())
                && StringUtils.isEmpty(getBackupId())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_GETSMBACKUPDETAILS_BACKUP_NAME_OR_ID_NOT_SPECIFIED));
        }

        SmsBackupValidationHelper smsBackupValidationHelper = new SmsBackupValidationHelper(getSnapCenterServerServiceHelper());

        switch (getPluginCode()) {
            case SCO:

                ProgressBar progressBar = ProgressBarUtil
                        .createAndStartProgressBar("Validating backup details ");
                try {
                    // Validate the specified backup
                    boolean validOracleBackup = smsBackupValidationHelper
                            .checkIfValidOracleBackup(backupId, backupName,
                                    appObjectId, isSecondary());
                    if (!validOracleBackup) {
                        String errorMessage = StringUtils.EMPTY;
                        if (isSecondary()) {
                            errorMessage = StringUtils.isBlank(getBackupName()) ? MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_GETSMBACKUPDETAILS_NOT_VALID_SECONDARY_ORACLE_BACKUP_ID_OR_RESOURCE_NOT_IN_BACKUP,
                                            getBackupId(), getAppObjectId())
                                    : MessageBundle
                                            .localizeMessage(
                                                    CLIMsgKey.CLI_GETSMBACKUPDETAILS_NOT_VALID_SECONDARY_ORACLE_BACKUP_NAME_OR_RESOURCE_NOT_IN_BACKUP,
                                                    getBackupName(),
                                                    getAppObjectId());
                        } else {
                            errorMessage = StringUtils.isBlank(getBackupName()) ? MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_GETSMBACKUPDETAILS_NOT_VALID_ORACLE_BACKUP_ID_OR_RESOURCE_NOT_IN_BACKUP,
                                            getBackupId(), getAppObjectId())
                                    : MessageBundle
                                            .localizeMessage(
                                                    CLIMsgKey.CLI_GETSMBACKUPDETAILS_NOT_VALID_ORACLE_BACKUP_NAME_OR_RESOURCE_NOT_IN_BACKUP,
                                                    getBackupName(),
                                                    getAppObjectId());
                        }
                        throw new CLIException(errorMessage);
                    }

                } catch (CLIException e) {
                    throw e;
                } finally {
                    ProgressBarUtil.stopProgressBar(progressBar);
                }
                break;

            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        pluginCode.name()));
        }

    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse)
            throws CLIException {
        SmsCommandResponseValidationUtil
                .checkBackupDetailsResponseInformation((SmGetBackupDetailsResponse) smResponse);

        FormatterType formatterType = getFormatterType();
        if (formatterType.equals(FormatterType.TEXT_TABLE)
                || formatterType.equals(FormatterType.TEXT)) {
            switch (getPluginCode()) {
                case SCO:
                    printOracleBackupDetails((SmGetBackupDetailsResponse)smResponse);
                    break;
                default:
                    break;
            }
        } else {
            Formatter formatter = FormatterFactory.getFormatter(formatterType);
            formatter.writeFormattedObject(smResponse);
        }
    }

    private void printOracleBackupDetails(SmGetBackupDetailsResponse smResponse)
            throws CLIException {

        ArrayOfSmBackup arrayOfSmBackup = smResponse.getBackups();

        SmBackup smBackup = arrayOfSmBackup.getSmBackup().iterator().next();

        ArrayOfSmBackupComponentInfo arrayOfSmBackupComponentInfo = smBackup
                .getBackupComponents();

        SmOracleBackupComponentInfo smOracleBackupComponentInfo = (SmOracleBackupComponentInfo) arrayOfSmBackupComponentInfo
                .getSmBackupComponentInfo().iterator().next();

        SmOracleDbEnhanced smOracleDbEnhanced = smOracleBackupComponentInfo
                .getSmOracleDatabaseInstance();

        StringBuffer oracleBackupInfo = new StringBuffer();
        oracleBackupInfo.append("ORACLE DATABASE BACKUP DETAILS");
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(String.format("Backup name: %s", smBackup.getBackupName()));
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(String.format("Backup type: %s", smBackup.getBackupType()));
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(String.format("Database name/SID: %s", ((SmOracleDb)(smOracleBackupComponentInfo.getComponentObject())).getDbSid()));
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(String.format("Database unique name: %s", smOracleDbEnhanced.getDatabaseUniqueName()));
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(String.format("Database type: %s", OracleDatabaseDetailsUtils.getOracleDatabaseTypeForDisplay(smOracleDbEnhanced.isIsRac(), smOracleDbEnhanced.getDatabaseType().equals(SmOracleDatabaseType.CDB), smOracleDbEnhanced.getDatabaseRole())));
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(String.format("Database id: %s", smOracleDbEnhanced.getDbId()));
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(String.format("Database home: %s", smOracleDbEnhanced.getOracleDetail().getOracleHome()));
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(String.format("Database version: %s", smOracleDbEnhanced.getOracleDetail().getOracleFullVersion()));
        oracleBackupInfo.append(NEW_LINE);
        oracleBackupInfo.append(String.format("Archive log mode: %s", smOracleDbEnhanced.isIsInArchiveLogMode()));
        oracleBackupInfo.append(NEW_LINE);
        if (smBackup.getBackupType().equalsIgnoreCase(ORACLE_DATA_BACKUP_TYPE)) {
            oracleBackupInfo.append(String.format("Check point SCN: %s", smOracleDbEnhanced.getCheckPointScn()));
            oracleBackupInfo.append(NEW_LINE);
            oracleBackupInfo.append(String.format("End backup SCN: %s", smOracleDbEnhanced.getEndBackupScn()));
            oracleBackupInfo.append(NEW_LINE);
        } else {
            oracleBackupInfo.append(String.format("Log start SCN: %s", smOracleDbEnhanced.getLogStartScn()));
            oracleBackupInfo.append(NEW_LINE);
            oracleBackupInfo.append(String.format("Log end SCN: %s", smOracleDbEnhanced.getLogStartScn()));
            oracleBackupInfo.append(NEW_LINE);
        }

        if (smOracleBackupComponentInfo.getCatalogBackupResult() != null
                && smOracleBackupComponentInfo.getCatalogBackupResult()
                        .getCatalogingStatus() != null) {
            oracleBackupInfo.append(String.format("Cataloging status: %s",
                    CLIUtils.getCatalogingStatus(smOracleBackupComponentInfo.getCatalogBackupResult()
                            .getCatalogingStatus())));
            oracleBackupInfo.append(NEW_LINE);

            SmKeyValueCollection catMeta = smOracleBackupComponentInfo
                    .getCatalogBackupResult().getCatalogingMetadata();
            if (catMeta != null && catMeta.getData() != null
                    && catMeta.getData().getSmKeyValue() != null
                    && !catMeta.getData().getSmKeyValue().isEmpty()) {
                List<SmKeyValue> catalogMetadata = catMeta.getData()
                        .getSmKeyValue();

                String tag = null;
                String catalogedControlFilePath = null;
                Set<String> catalogedArchiveLocations = new HashSet<>();

                for (SmKeyValue kv : catalogMetadata) {
                    if (kv.getKey().equalsIgnoreCase(
                            SCO_CATALOG_METADATA_DATAFILE_TAG_KEY)) {
                        tag = kv.getValue();
                    } else if (kv.getKey().equalsIgnoreCase(
                            SCO_CATALOG_METADATA_CONTROLFILE_PATH_KEY)) {
                        catalogedControlFilePath = kv.getValue();
                    } else if (kv.getKey().equalsIgnoreCase(
                            SCO_CATALOG_METADATA_ARCHIVELOG_DESTINATIONS_KEY)) {
                        if (kv.getValues() != null
                                && !kv.getValues().getSmString().isEmpty()) {
                            for (SmString loc : kv.getValues().getSmString()) {
                                catalogedArchiveLocations.add(loc.getValue());
                            }
                        }
                    }
                }

                if (StringUtils.isNotBlank(tag)) {
                    oracleBackupInfo.append(String.format(
                            "Cataloged datafiles tag name: %s",
                            tag));
                    oracleBackupInfo.append(NEW_LINE);
                }

                if (StringUtils.isNotBlank(catalogedControlFilePath)) {
                    oracleBackupInfo.append(String.format(
                            "Cataloged controlfile path: %s",
                            catalogedControlFilePath));
                    oracleBackupInfo.append(NEW_LINE);
                }

                if (catalogedArchiveLocations != null
                        && !catalogedArchiveLocations.isEmpty()) {
                    oracleBackupInfo.append(String.format(
                            "Cataloged archivelog locations: %s",
                            catalogedArchiveLocations.toString()));
                    oracleBackupInfo.append(NEW_LINE);
                }
            }
        }

        oracleBackupInfo.append(NEW_LINE);

        // Tablespaces
        if (smOracleDbEnhanced.getTablespaces() != null
                && !smOracleDbEnhanced.getTablespaces()
                        .getSmOracleAbstractTableSpaceConfiguration().isEmpty()) {
            int tbspCount = 1;
            List<SmOracleAbstractTableSpaceConfiguration> tablespaces = smOracleDbEnhanced
                    .getTablespaces()
                    .getSmOracleAbstractTableSpaceConfiguration();
            oracleBackupInfo.append("TABLESPACES");
            oracleBackupInfo.append(NEW_LINE);
            oracleBackupInfo.append(NEW_LINE);
            for (SmOracleAbstractTableSpaceConfiguration tbsp : tablespaces) {
                oracleBackupInfo.append(String.format("%8d. %s", tbspCount++, tbsp.getName()));
                oracleBackupInfo.append(NEW_LINE);
            }
            oracleBackupInfo.append(NEW_LINE);
        }

        // Pluggable databases
        if (smOracleDbEnhanced.getDatabaseType().equals(
                SmOracleDatabaseType.CDB)) {
            if (smOracleDbEnhanced.getPluggableDatabases() != null
                    && !smOracleDbEnhanced.getPluggableDatabases()
                            .getSmOraclePluggableDatabase().isEmpty()) {
                int pdbCount = 1;
                List<SmOraclePluggableDatabase> pdbs = smOracleDbEnhanced
                        .getPluggableDatabases().getSmOraclePluggableDatabase();
                oracleBackupInfo.append("PLUGGABLE DATABASES");
                oracleBackupInfo.append(NEW_LINE);
                oracleBackupInfo.append(NEW_LINE);
                for (SmOraclePluggableDatabase pdb : pdbs) {
                    oracleBackupInfo.append(String.format("%6d. PDB Name: %s",
                            pdbCount++, pdb.getPdbName()));
                    oracleBackupInfo.append(NEW_LINE);
                    if (pdb.getTableSpaces() != null
                            && !pdb.getTableSpaces()
                                    .getSmOracleAbstractTableSpaceConfiguration()
                                    .isEmpty()) {
                        oracleBackupInfo.append("\tPDB tablespaces:");
                        oracleBackupInfo.append(NEW_LINE);
                        int pdbTbspCount = 1;
                        for (SmOracleAbstractTableSpaceConfiguration pdbTbsp : pdb
                                .getTableSpaces()
                                .getSmOracleAbstractTableSpaceConfiguration()) {
                            oracleBackupInfo.append(String.format("\t%6d. %s",
                                    pdbTbspCount++, pdbTbsp.getName()));
                            oracleBackupInfo.append(NEW_LINE);
                        }
                    }
                    oracleBackupInfo.append(NEW_LINE);
                }
                oracleBackupInfo.append(NEW_LINE);
            }
        }

        CLIUtils.writeToConsole(oracleBackupInfo.toString());
    }

    public String getAppObjectId() {
        return appObjectId;
    }

    public void setAppObjectId(String appObjectId) {
        this.appObjectId = appObjectId;
    }

    public String getBackupName() {
        return backupName;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public String getBackupId() {
        return backupId;
    }

    public void setBackupId(String backupId) {
        this.backupId = backupId;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public boolean isSecondary() {
        return secondary;
    }

    public void setSecondary(boolean secondary) {
        this.secondary = secondary;
    }
}
