/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmCloneCommand.java#4 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmClone;
import com.netapp.smcore.plugin.contracts.SmClone;
import com.netapp.smcore.plugin.contracts.SmCloneObjectMapping;
import com.netapp.smcore.plugin.contracts.SmGetCloneRequest;
import com.netapp.smcore.plugin.contracts.SmGetCloneResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmclone.command.description")
public class GetSmCloneCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMCLONE_COMMAND_NAME);

    @Parameter(names = { "-CloneName" }, descriptionKey = "cli.getsmclone.command.cloneName.option.description")
    private String cloneName;

    @Parameter(names = { "-CloneId" }, descriptionKey = "cli.getsmclone.command.cloneId.option.description")
    private String cloneId;

    @Parameter(names = { "-BackupName" }, descriptionKey = "cli.getsmclone.command.backupName.option.description")
    private String backupName;

    @Parameter(names = { "-BackupId" }, descriptionKey = "cli.getsmclone.command.backupId.option.description")
    private String backupId;

    @Parameter(names = { "-AppObjectName" }, descriptionKey = "cli.getsmclone.command.appObjectName.option.description")
    private String appObjectName;

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.getsmclone.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetCloneRequest, SmGetCloneResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmGetCloneRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmGetCloneResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_CLONE_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse)
            throws CLIException {

        SmGetCloneResponse smGetCloneResponse = (SmGetCloneResponse) smResponse;
        ArrayOfSmClone arrayOfSmClone = smGetCloneResponse.getClones();
        if (arrayOfSmClone == null
                || CLIUtils
                        .isNullOrEmptyCollection(arrayOfSmClone.getSmClone())) {
            return;
        }

        String headerCloneId = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMCLONE_TABLE_CLONE_ID_HEADER);
        String headerCloneName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMCLONE_TABLE_CLONE_NAME_HEADER);
        String headerSourceDb = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMCLONE_TABLE_SOURCE_DB_HEADER);
        String headerCloneDb = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMCLONE_TABLE_CLONE_DB_HEADER);
        String headerCloneHost = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMCLONE_TABLE_CLONE_HOST_HEADER);

        String[] backupTableHeaders = { headerCloneId, headerCloneName,
                headerSourceDb, headerCloneDb, headerCloneHost };
        TextTableInterface backupTableFormatter = new TextTable(
                backupTableHeaders);

        for (SmClone smClone : arrayOfSmClone.getSmClone()) {

            String columnCloneId = CLIUtils
                    .getStringValue(smClone.getCloneID());
            String columnCloneName = CLIUtils.getStringValue(smClone
                    .getCloneName());
            SmCloneObjectMapping smCloneObjectMapping = smClone
                    .getCloneObjects().getSmCloneObjectMapping().iterator()
                    .next();
            String columnSourceDb = CLIUtils
                    .getStringValue(smCloneObjectMapping.getSourceObject()
                            .getName());
            String columnCloneDb = CLIUtils.getStringValue(smCloneObjectMapping
                    .getCloneObject().getName());
            String columnCloneHost = CLIUtils.getStringValue(smClone
                    .getSmCloneHost().getHostName());
            backupTableFormatter.addRow(columnCloneId, columnCloneName,
                    columnSourceDb, columnCloneDb, columnCloneHost);
        }

        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            backupTableFormatter.displayFormattedTable();
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(arrayOfSmClone);
        }
    }

    public String getCloneName() {
        return cloneName;
    }

    public void setCloneName(String cloneName) {
        this.cloneName = cloneName;
    }

    public String getCloneId() {
        return cloneId;
    }

    public void setCloneId(String cloneId) {
        this.cloneId = cloneId;
    }

    public String getBackupName() {
        return backupName;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public String getBackupId() {
        return backupId;
    }

    public void setBackupId(String backupId) {
        this.backupId = backupId;
    }

    public String getAppObjectName() {
        return appObjectName;
    }

    public void setAppObjectName(String appObjectName) {
        this.appObjectName = appObjectName;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }
}
