/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmHostCommand.java#4 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmGetHostRequest;
import com.netapp.smcore.plugin.contracts.SmGetHostResponse;
import com.netapp.smcore.plugin.contracts.SmHost;
import com.netapp.smcore.plugin.contracts.SmHostInfo;
import com.netapp.smcore.plugin.contracts.SmHostStatus;
import com.netapp.smcore.plugin.contracts.SmPluginInfo;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmhost.command.description")
public class GetSmHostCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMHOST_COMMAND_NAME);

    @Parameter(names = { "-HostNames" }, descriptionKey = "cli.getsmhost.command.hostNames.option.description", splitter = CommaParameterSplitter.class)
    private List<String> hostNames = new ArrayList<String>();

    @Parameter(names = { "-ListPluginInfo" }, descriptionKey = "cli.getsmhost.command.listPluginInfo.option.description")
    private boolean includeHostPluginInfo;

    // To get smhost this must be true in request
    private boolean displayAll = true;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetHostRequest, SmGetHostResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmGetHostRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmGetHostResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_HOSTS_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse) throws CLIException {
        SmGetHostResponse smGetHostResponse = (SmGetHostResponse) smResponse;

        SmHostInfo smHostInfo = smGetHostResponse.getHostInfo();

        if (smHostInfo == null
                || smHostInfo.getHosts() == null
                || CLIUtils.isNullOrEmptyCollection(smHostInfo.getHosts()
                        .getSmHost())) {
            return;
        }

        String headerHostName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMHOST_TABLE_HOST_NAME_HEADER);
        String headerHostStatus = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMHOST_TABLE_HOST_STATUS_HEADER);
        String headerOS = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMHOST_TABLE_OS_HEADER);
        String headerPort = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMHOST_TABLE_PORT_HEADER);

        String headerPluginsName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMHOST_TABLE_PLUGINS_NAME_HEADER);
        String headerPluginsCode = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMHOST_TABLE_PLUGINS_CODE_HEADER);
        String headerPluginsVersion = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMHOST_TABLE_PLUGINS_VERSION_HEADER);

        String[] hostsTableHeaders = { headerHostName, headerHostStatus,
                                       headerOS, headerPort, headerPluginsName,
                                       headerPluginsCode, headerPluginsVersion };

        TextTableInterface hostsTable = new TextTable(hostsTableHeaders);

        for (SmHost smHost : smHostInfo.getHosts().getSmHost()) {

            String columnHostName = CLIUtils.getStringValue(smHost
                    .getHostName());
            String columnHostStatus = getHostStatus(smHost.getHostStatus());
            String columnOS = "";
            if (smHost.getOsInfo() != null) {
                columnOS = CLIUtils.getEnumName(smHost.getOsInfo()
                        .getOperatingSystemType());
            }
            String columnPort = CLIUtils.getStringValue(smHost.getPort());

            String columnPluginsName = "";
            String columnPluginsCode = "";
            String columnPluginsVersion = "";

            if (smHost.getHostPluginInfos() != null
                    && smHost.getHostPluginInfos().getSmPluginInfo() != null) {
                List<String> pluginsNameList = new ArrayList<>();
                List<String> pluginsCodeList = new ArrayList<>();
                List<String> pluginsVersionList = new ArrayList<>();
                for (SmPluginInfo smPluginInfo : smHost.getHostPluginInfos()
                        .getSmPluginInfo()) {
                    pluginsNameList.add(CLIUtils.getStringValue(smPluginInfo
                            .getPluginName()));
                    pluginsCodeList.add(CLIUtils.getEnumName(smPluginInfo
                            .getPluginCode()));
                    pluginsVersionList.add(CLIUtils.getStringValue(smPluginInfo
                            .getPluginVersion()));
                }
                columnPluginsName = StringUtils.substringBetween(
                        pluginsNameList.toString(), "[", "]");
                columnPluginsCode = StringUtils.substringBetween(
                        pluginsCodeList.toString(), "[", "]");
                columnPluginsVersion = StringUtils.substringBetween(
                        pluginsVersionList.toString(), "[", "]");
            }

            hostsTable.addRow(columnHostName, columnHostStatus, columnOS,
                    columnPort, columnPluginsName, columnPluginsCode,
                    columnPluginsVersion);
        }

        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            if (isIncludeHostPluginInfo()) {
                hostsTable.displayFormattedTable(headerHostName,
                        headerPluginsName, headerPluginsCode, headerPluginsVersion);
            } else {
                hostsTable.displayFormattedTable(headerHostName,
                        headerHostStatus, headerOS, headerPort);
            }
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(smHostInfo.getHosts());
        }
    }

    private String getHostStatus(SmHostStatus smHostStatus) {
        // Refer the SmHostStatus Enum
        if (smHostStatus != null) {
            return smHostStatus.value().replace("eHost", "").trim();
        } else {
            return "";
        }
    }

    public void setHostNames(List<String> hostNames) {
        this.hostNames = hostNames;
    }

    public List<String> getHostNames() {
        return hostNames;
    }

    public void setIncludeHostPluginInfo(boolean includeHostPluginInfo) {
        this.includeHostPluginInfo = includeHostPluginInfo;
    }

    public boolean isIncludeHostPluginInfo() {
        return includeHostPluginInfo;
    }

    public boolean isDisplayAll() {
        return displayAll;
    }

    public void setDisplayAll(boolean displayAll) {
        this.displayAll = displayAll;
    }
}
