/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmSMTPServerCommand.java#4 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmGetHostResponse;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerRequest;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.smcore.plugin.contracts.SmHost;
import com.netapp.smcore.plugin.contracts.SmHostInfo;
import com.netapp.smcore.plugin.contracts.SmPluginInfo;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.HttpMethod;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmsmtpserver.command.description")
public class GetSmSMTPServerCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMSMTPSERVER_COMMAND_NAME);
    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(GetSmSMTPServerCommand.class);

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetSMTPServerRequest, SmGetSMTPServerResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmGetSMTPServerRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetSMTPServerResponse.class);
        smsRestInvocationContext.setHttpMethod(HttpMethod.GET);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_SMTP_SERVER_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse)
            throws CLIException {
        SmGetSMTPServerResponse smGetSMTPServerResponse = (SmGetSMTPServerResponse) smResponse;

        String headerSmtpServer = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSMTPSERVER_TABLE_SMTP_SERVER_HEADER);
        String headerEmailTo = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSMTPSERVER_TABLE_EMAIL_TO_HEADER);
        String headerEmailFrom = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSMTPSERVER_TABLE_EMAIL_FROM_HEADER);

        String[] emailNotificationsTableHeaders = { headerSmtpServer,
                headerEmailTo, headerEmailFrom };

        TextTableInterface emailNotificationTable = new TextTable(
                emailNotificationsTableHeaders);
   
        String columnSMTPEmailFrom = smGetSMTPServerResponse.getSMTPEmailFrom();
        String columnSMTPEmailTo = smGetSMTPServerResponse.getSMTPEmailTo();
        String columnSMTPEmailServer = smGetSMTPServerResponse.getSMTPServer();

        emailNotificationTable.addRow(columnSMTPEmailServer,
                columnSMTPEmailFrom, columnSMTPEmailTo);

        emailNotificationTable.displayFormattedTable(headerSmtpServer,
                headerEmailFrom, headerEmailTo);
    }

}
