/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmSecondaryDetailsCommand.java#4 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.ORACLE_DATABASE;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmArchivedDetail;
import com.netapp.smcore.plugin.contracts.ArrayOfSnapshotLocator;
import com.netapp.smcore.plugin.contracts.SmArchivedDetail;
import com.netapp.smcore.plugin.contracts.SmGetArchivedDetailsRequest;
import com.netapp.smcore.plugin.contracts.SmGetArchivedDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SnapshotLocator;
import com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmsecondarydetails.command.description")
public class GetSmSecondaryDetailsCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMSECONDARYDETAILS_COMMAND_NAME);

    @Parameter(names = { "-BackupId" }, descriptionKey = "cli.getsmsecondarydetails.command.backupId.option.description")
    private String backupId;

    @Parameter(names = { "-AppObjectId" }, descriptionKey = "cli.getsmsecondarydetails.command.appObjectId.option.description", required = true)
    private String appObjectId;

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.getsmsecondarydetails.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    private String appObjectType;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
        SmsBackupValidationHelper smsBackupValidationHelper = new SmsBackupValidationHelper(
                getSnapCenterServerServiceHelper());
        switch (pluginCode) {

            case SCO:

                setAppObjectType(ORACLE_DATABASE);

                if (StringUtils.isNotBlank(getBackupId())) {

                    ProgressBar progressBar = ProgressBarUtil
                            .createAndStartProgressBar("Validating backup details ");

                    // Validate the specified backup
                    boolean isValidOracleBackup = smsBackupValidationHelper
                            .checkIfValidOracleBackup(backupId, null,
                                    appObjectId, true);

                    ProgressBarUtil.stopProgressBar(progressBar);

                    if (!isValidOracleBackup) {
                        throw new CLIException(
                                MessageBundle
                                        .localizeMessage(
                                                CLIMsgKey.CLI_GETSMSECONDARYDETAILS_NOT_VALID_ORACLE_SECONDARY_BACKUP_ID_OR_RESOURCE_NOT_IN_BACKUP,
                                                getBackupId(), getAppObjectId()));
                    }
                }

                break;

            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        pluginCode.name()));
        }
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetArchivedDetailsRequest, SmGetArchivedDetailsResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmGetArchivedDetailsRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetArchivedDetailsResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_ARCHIVED_DETAILS_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse)
            throws CLIException {

        SmGetArchivedDetailsResponse smGetArchivedDetailsResponse = (SmGetArchivedDetailsResponse) smResponse;
        ArrayOfSmArchivedDetail arrayOfSmArchivedDetail = smGetArchivedDetailsResponse
                .getArchivedDetails();
        if (arrayOfSmArchivedDetail == null
                || CLIUtils.isNullOrEmptyCollection(arrayOfSmArchivedDetail
                        .getSmArchivedDetail())) {
            return;
        }

        String headerPrimary = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSECONDARYDETAILS_TABLE_PRIMARY_HEADER);
        String headerSecondary = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSECONDARYDETAILS_TABLE_SECONDARY_HEADER);
        String headerRelationshipType = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSECONDARYDETAILS_TABLE_RELATIONSHIPTYPE_HEADER);

        
        String[] secondaryTableHeaders = { headerPrimary, headerSecondary, headerRelationshipType };
        TextTableInterface secondaryTableFormatter = new TextTable(
                secondaryTableHeaders);

        for (SmArchivedDetail smArchivedDetail : arrayOfSmArchivedDetail
                .getSmArchivedDetail()) {

            SnapshotLocator primarySnapshotLocator = smArchivedDetail
                    .getPrimary();
            ArrayOfSnapshotLocator arrayOfSecondarySnapshotLocator = smArchivedDetail
                    .getSecondaries();

            String primarySVM = CLIUtils.getStringValue(primarySnapshotLocator
                    .getStorageSystemName());
            String primaryVolume = CLIUtils
                    .getStringValue(primarySnapshotLocator
                            .getStorageVolumeName());
            String columnPrimary = String.format("%s:%s", primarySVM,
                    primaryVolume);

            for (SnapshotLocator secondarySL : arrayOfSecondarySnapshotLocator
                    .getSnapshotLocator()) {
                String secondarySVM = CLIUtils.getStringValue(secondarySL
                        .getStorageSystemName());
                String secondaryVolume = CLIUtils.getStringValue(secondarySL
                        .getStorageVolumeName());
                String relationshipType = CLIUtils.getStringValue(secondarySL.getRelationshipType().toString());
                String columnSecondary = String.format("%s:%s", secondarySVM,
                        secondaryVolume);

                secondaryTableFormatter.addRow(columnPrimary, columnSecondary, relationshipType);

            }
        }

        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            secondaryTableFormatter.displayFormattedTable();
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(arrayOfSmArchivedDetail);
        }

    }

    public String getBackupId() {
        return backupId;
    }

    public void setBackupId(String backupId) {
        this.backupId = backupId;
    }

    public String getAppObjectId() {
        return appObjectId;
    }

    public void setAppObjectId(String appObjectId) {
        this.appObjectId = appObjectId;
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public String getAppObjectType() {
        return appObjectType;
    }

    public void setAppObjectType(String appObjectType) {
        this.appObjectType = appObjectType;
    }
}