/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/GetSmStorageConnectionCommand.java#4 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmStorageContext;
import com.netapp.smcore.plugin.contracts.SDStorageSystemId;
import com.netapp.smcore.plugin.contracts.SmGetStorageConnectionRequest;
import com.netapp.smcore.plugin.contracts.SmGetStorageConnectionResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmStorageContext;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.Formatter;
import com.netapp.snapcenter.cli.formatter.FormatterFactory;
import com.netapp.snapcenter.cli.formatter.FormatterType;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsmstorageconnection.command.description")
public class GetSmStorageConnectionCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_COMMAND_NAME);

    @Parameter(names = { "-SVM" }, descriptionKey = "cli.getsmstorageconnection.command.svm.option.description")
    private String storageVM;

    private boolean displayAll = true;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmGetStorageConnectionRequest, SmGetStorageConnectionResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmGetStorageConnectionRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmGetStorageConnectionResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_GET_STORAGE_CONNECTION_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void printCommandResponseObject(T smResponse) throws CLIException {
        SmGetStorageConnectionResponse smGetStorageConnectionResponse = (SmGetStorageConnectionResponse) smResponse;
        ArrayOfSmStorageContext arrayOfSmStorageContext = smGetStorageConnectionResponse
                .getStorageContext();
        if (arrayOfSmStorageContext == null
                || CLIUtils.isNullOrEmptyCollection(arrayOfSmStorageContext
                        .getSmStorageContext())) {
            return;
        }
        String headerSvmName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_TABLE_SVM_NAME_HEADER);
        String headerSvmProtocol = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_TABLE_SVM_PROTOCOL_HEADER);
        String headerSvmPort = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_TABLE_SVM_PORT_HEADER);
        String headerSvmUsername = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_TABLE_SVM_USERNAME_HEADER);
        String headerEnableAsupOnFailure = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_TABLE_ENABLE_ASUP_ON_FAILURE_HEADER);
        String headerEnableSysLog = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_TABLE_ENABLE_SYS_LOG_HEADER);
        String headerTimeOut = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_TABLE_TIMEOUT_HEADER);
        String headerPreferredIPAddress  = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_TABLE_PREFERRED_IP_ADDRESS_HEADER);
        String headerIsSecondary = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMSTORAGECONNECTION_TABLE_ISSECONDARY_HEADER);

        String[] storageTableHeaders = { headerSvmName,
                                         headerSvmProtocol,
                                         headerSvmPort,
                                         headerSvmUsername,
                                         headerTimeOut,
                                         headerPreferredIPAddress,
                                         headerEnableAsupOnFailure,
                                         headerEnableSysLog,
                                         headerIsSecondary};

        TextTableInterface storageTableFormatter = new TextTable(
                storageTableHeaders);

        for (SmStorageContext smStorageContext : arrayOfSmStorageContext
                .getSmStorageContext()) {

            SDStorageSystemId sdStorageSystemId = smStorageContext
                    .getStorageConnection();
            if (sdStorageSystemId == null) {
                continue;
            }

			String columnSvmName = CLIUtils.getStringValue(sdStorageSystemId
					.getName());
			String columnSvmProtocol = CLIUtils.getEnumName(sdStorageSystemId
					.getTransportType());
			String columnSvmPort = sdStorageSystemId.getPort() == null ? ""
					: sdStorageSystemId.getPort().toString();
			String columnSvmUserName = CLIUtils
					.getStringValue(sdStorageSystemId.getUserName());
			String columTimeOut = CLIUtils.getStringValue(sdStorageSystemId
					.getTimeout().toString());
			String columPreferredIPAddress = CLIUtils
					.getStringValue(sdStorageSystemId.getPreferredIpAddress());
			String columEnableAsupOnFailure = String.valueOf(smStorageContext
					.isEnableAsupOnFailure());
			String columEnableSyslog = String.valueOf(smStorageContext
					.isEnableSysLog());
			String isSecondary = String.valueOf(smStorageContext
					.getStorageConnection().isIsSecondary());

			storageTableFormatter.addRow(columnSvmName, columnSvmProtocol,
					columnSvmPort, columnSvmUserName, columTimeOut,
					columPreferredIPAddress, columEnableAsupOnFailure,
					columEnableSyslog, isSecondary);
        }
        if (getFormatterType() == FormatterType.TEXT_TABLE) {
            storageTableFormatter.displayFormattedTable();
        } else {
            Formatter formatter = FormatterFactory
                    .getFormatter(getFormatterType());
            formatter.writeFormattedObject(arrayOfSmStorageContext);
        }
    }

    public void setStorageVM(String storageVM) {
        this.storageVM = storageVM;
    }

    public String getStorageVM() {
        return storageVM;
    }

    public void setDisplayAll(boolean displayAll) {
        this.displayAll = displayAll;
    }

    public boolean isDisplayAll() {
        return displayAll;
    }
}
