/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/ModifySmProtectResourceCommand.java#4 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.ORACLE_DATABASE;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_HOST;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_NAMES;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_TYPE;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmBackupPolicyVerificationSettings;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourcesResponse;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsResponse;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.smcore.plugin.contracts.SmPolicy;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.smcore.plugin.contracts.SmSchedulerType;
import com.netapp.snapcenter.cli.command.parameters.BackupResourceGroupParameters;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupScheduleParameters;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupVerificationParameters;
import com.netapp.snapcenter.cli.command.parameters.SMOBackupDataSetParameters;
import com.netapp.snapcenter.cli.command.parameters.VerificationResourceGroupParameters;
import com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToSchedulesMapConverter;
import com.netapp.snapcenter.cli.command.parser.NoParameterSplitter;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.helper.ResourceProtectionGroupHelper;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.setsmprotectresource.command.description")
public class ModifySmProtectResourceCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETSMPROTECTRESOURCE_COMMAND_NAME);

    @Parameter(names = { "-PluginCode" }, descriptionKey = "cli.setsmprotectresource.command.pluginCode.option.description")
    private PluginCode pluginCode = PluginCode.SCO;

    @Parameter(names = { "-Description" }, descriptionKey = "cli.setsmprotectresource.command.description.option.description")
    private String description;

    @ParametersDelegate
    private BackupResourceGroupParameters backupResourceGroupParameters = new BackupResourceGroupParameters();

    @ParametersDelegate
    private SMOBackupDataSetParameters smoBackupDataSetParameters = new SMOBackupDataSetParameters();

    @ParametersDelegate
    private VerificationResourceGroupParameters verificationResourceGroupParameters = new VerificationResourceGroupParameters();

    // The resource group name will be retrieved from the server.
    private String resourceGroupName;

    @Parameter(names = { "-Resource" }, descriptionKey = "cli.setsmprotectresource.command.resources.option.description", converter = CustomStringToHostResourcesMapConverter.class, splitter = NoParameterSplitter.class, required = true)
    private Map<String, String> resource;

    @Parameter(names = { "-Schedules" }, descriptionKey = "cli.setsmprotectresource.command.schedules.option.description", converter = CustomStringToSchedulesMapConverter.class, splitter = NoParameterSplitter.class)
    private List<Map<String, String>> schedules = new ArrayList<Map<String, String>>();

    private ExtendedGenericLogger logger;

    private List<SmPolicy> resourceGroupPolicyList;

    private Map<String, List<SmSchedulerType>> policyVsScheduleTypeMap;

    private List<ResourceGroupScheduleParameters> scheduleParametersList;

    private ProgressBar progressBar;

    private ModifySmResourceGroupCommand modifySmResourceGroupCommand;

    public ModifySmProtectResourceCommand() {
        modifySmResourceGroupCommand = new ModifySmResourceGroupCommand();
        resourceGroupPolicyList = new ArrayList<SmPolicy>();
        policyVsScheduleTypeMap = new HashMap<String, List<SmSchedulerType>>();
        scheduleParametersList = new ArrayList<ResourceGroupScheduleParameters>();
        logger = LoggerFactory
                .getExtendedClassAwareLogger(ModifySmProtectResourceCommand.class);
    }

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {

        super.initialize();

        /*
         * Process Command arguments to perform validation, setting defaults
         * etc..
         */
        processCommandArguments();

        // Delegate the validation and execution to
        // modifySmResourceGroupCommand,
        // since it is doing the same steps
        modifySmResourceGroupCommand.setPluginCode(pluginCode);
        modifySmResourceGroupCommand.setResourceGroupName(resourceGroupName);
        modifySmResourceGroupCommand.setDescription(description);
        modifySmResourceGroupCommand
                .setBackupResourceGroupParameters(backupResourceGroupParameters);
        modifySmResourceGroupCommand
                .setSmoBackupDataSetParameters(smoBackupDataSetParameters);
        modifySmResourceGroupCommand.setResources(Collections
                .singletonList(resource));
        modifySmResourceGroupCommand.setSchedules(schedules);
        modifySmResourceGroupCommand.setSnapCenterServerServiceHelper(super
                .getSnapCenterServerServiceHelper());
        modifySmResourceGroupCommand
                .setVerificationResourceGroupParameters(verificationResourceGroupParameters);

        modifySmResourceGroupCommand.processCommandArguments();

        // Command to request mapping
        SmRequest smRequest = (SmRequest) modifySmResourceGroupCommand
                .getCommandMapper().map(modifySmResourceGroupCommand);

        SmResponse smResponse = new SmResponse();

        try {
            // Start Progress Bar
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar(MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_CONTACTING_SNAPCENTER_SERVER));
            // Invoke Server rest end point
            SMSRestInvocationContext<?, ?> commandRestInvocationContext = getCommandSpecificSMSRestInvocationContext(smRequest);
            smResponse = (SmResponse) super.getSnapCenterServerServiceHelper()
                    .makeRestCallToSnapCenterServer(
                            commandRestInvocationContext);
        } catch (CLIException e) {
            throw e;
        } finally {
            // Stop Progress Bar
            ProgressBarUtil.stopProgressBar(progressBar);
        }

        // Tasks like printing response etc.. are done here.
        super.processCommandResponse(smResponse);

    }

    protected void processCommandArguments() throws CLIException {

        SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
                super.getSnapCenterServerServiceHelper());

        switch (pluginCode) {
            case SCO:
                break;
            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        pluginCode.name()));
        }

        try {
            progressBar = ProgressBarUtil
                    .createAndStartProgressBar(MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_VALIDATING_RESOURCE_ALREADY_PROTECTED));

            String resourceNames = resource.get(RESOURCE_NAMES);
            String hostName = resource.get(RESOURCE_HOST);
            String resourceType = resource.get(RESOURCE_TYPE);

            if (pluginCode.equals(PluginCode.SCO)) {
                if (resourceType == null
                        || !resourceType.equalsIgnoreCase(ORACLE_DATABASE)) {
                    throw new IllegalArgumentException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_FOR_SMO_RESOURCE_TYPE_SHOULD_BE_ORACLE_DATABASE,
                                            resourceType));

                }
            }

            SmDiscoverResourcesResponse smDiscoverResourcesResponse = smsCommandRequestValidationHelper
                    .getDiscoverResourceDetailsFromServer(
                            SmPluginCode.fromValue(pluginCode.name()),
                            hostName, true);
            // Since only one resource pass the first list element
            String smObjectGuid = ResourceProtectionGroupHelper
                    .getSmObjectGuidFromResourceName(
                            smDiscoverResourcesResponse, resourceNames);

            // Verify whether resource is already protected by contacting CSM
            // server. If not protected throw error.
            SmGetProtectionGroupsResponse smGetProtectionGroupsResponse = smsCommandRequestValidationHelper
                    .getInternalProtectionGroupDetailsFromServer(smObjectGuid);
            String internalResourceName = ResourceProtectionGroupHelper
                    .getInternalResourceGroupNameFromProtectionGroup(
                            smGetProtectionGroupsResponse, smObjectGuid);

            setResourceGroupName(internalResourceName);

			SmGetSMTPServerResponse smGetSMTPServerResponse = smsCommandRequestValidationHelper
					.getSmSMTPServerDetailsFromServer();

			SmsCommandArgumentsValidationUtil
					.validateAndSetNotificationParameters(
							backupResourceGroupParameters
									.getNotificationParameters(),
							smGetSMTPServerResponse,COMMANDNAME);

        } catch (Exception exception) {
            throw exception;
        } finally {
            ProgressBarUtil.stopProgressBar(progressBar);

        }

    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        return modifySmResourceGroupCommand
                .getCommandSpecificSMSRestInvocationContext(cmdRequest);
    }

    public PluginCode getPluginCode() {
        return pluginCode;
    }

    public void setPluginCode(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public BackupResourceGroupParameters getBackupResourceGroupParameters() {
        return backupResourceGroupParameters;
    }

    public void setBackupResourceGroupParameters(
            BackupResourceGroupParameters backupResourceGroupParameters) {
        this.backupResourceGroupParameters = backupResourceGroupParameters;
    }

    public Map<String, String> getResource() {
        return resource;
    }

    public void setResources(Map<String, String> resource) {
        this.resource = resource;
    }

    public List<Map<String, String>> getSchedules() {
        return schedules;
    }

    public void setSchedules(List<Map<String, String>> schedules) {
        this.schedules = schedules;
    }

    public List<ResourceGroupScheduleParameters> getScheduleParametersList() {
        return scheduleParametersList;
    }

    public void setScheduleParametersList(
            List<ResourceGroupScheduleParameters> scheduleList) {
        this.scheduleParametersList = scheduleList;
    }

    public VerificationResourceGroupParameters getVerificationResourceGroupParameters() {
        return verificationResourceGroupParameters;
    }

    public void setVerificationResourceGroupParameters(
            VerificationResourceGroupParameters verificationResourceGroupParameters) {
        this.verificationResourceGroupParameters = verificationResourceGroupParameters;
    }

    public List<SmPolicy> getResourceGroupPolicyList() {
        return resourceGroupPolicyList;
    }

    public void setResourceGroupPolicyList(
            List<SmPolicy> resourceGroupPolicyList) {
        this.resourceGroupPolicyList = resourceGroupPolicyList;
    }

    public void setSmoBackupDataSetParameters(
            SMOBackupDataSetParameters smoBackupDataSetParameters) {
        this.smoBackupDataSetParameters = smoBackupDataSetParameters;
    }

    public SMOBackupDataSetParameters getSmoBackupDataSetParameters() {
        return smoBackupDataSetParameters;
    }

    public String getResourceGroupName() {
        return resourceGroupName;
    }

    public void setResourceGroupName(String resourceGroupName) {
        this.resourceGroupName = resourceGroupName;
    }

}
