/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/NewSmBackupCommand.java#4 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.ORACLE_DATABASE;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_HOST;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_NAMES;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_TYPE;

import java.util.Map;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmCreateBackupRequest;
import com.netapp.smcore.plugin.contracts.SmCreateBackupResponse;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourcesResponse;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsResponse;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter;
import com.netapp.snapcenter.cli.command.parser.NoParameterSplitter;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.helper.ResourceProtectionGroupHelper;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;

import org.apache.commons.lang3.StringUtils;

import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.spl.common.utils.HostInfoUtil;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.newsmbackup.command.description")
public class NewSmBackupCommand extends AbstractSMSCommand {

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(NewSmBackupCommand.class);

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_NEWSMBACKUP_COMMAND_NAME);

    @Parameter(names = { "-ResourceGroupName" }, descriptionKey = "cli.newsmbackup.command.resourcegroup.option.description")
    private String resourceGroup = new String();

    @Parameter(names = { "-Resource" }, descriptionKey = "cli.addsmprotectresource.command.resources.option.description", converter = CustomStringToHostResourcesMapConverter.class, splitter = NoParameterSplitter.class)
    private Map<String, String> resource;

    @Parameter(names = { "-Policy" }, descriptionKey = "cli.newsmbackup.command.policy.option.description", required = true)
    private String policyName;

    @Parameter(names = { "-WaitForCompletion" }, descriptionKey = "cli.newsmbackup.command.waitForCompletion.option.description")
    private boolean waitForCompletion;
    
    @Parameter(names = { "-VerifyOnSecondary" }, descriptionKey = "cli.newsmbackup.command.verifyOnSecondary.option.description")
    private boolean verifyOnSecondary;

    @Parameter(names = { "-EnableVerification" }, descriptionKey = "cli.newsmbackup.command.enableVerification.option.description")
    private boolean enableVerification;

    private ProgressBar progressBar;

    private String protectoinGroupName;

    private PluginCode pluginCode;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }
 
    @Override
    protected void processCommandArguments() throws CLIException {
        SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
                getSnapCenterServerServiceHelper());

        if (StringUtils.isBlank(resourceGroup) && CLIUtils.isEmpty(resource)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_NEWSMBACKUP_RESOURCE_OR_RESOURCEGROUP_NOT_PROVIDED));
        } else if (StringUtils.isNotBlank(resourceGroup)
                && !CLIUtils.isEmpty(resource)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_NEWSMBACKUP_RESOURCE_AND_RESOURCEGROUP_BOTH_PROVIDED));
        }

        if (!CLIUtils.isEmpty(resource)) {

            // When resource is provided get the protection group name from the
            // server
            try {
                progressBar = ProgressBarUtil
                        .createAndStartProgressBar(MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_VALIDATING_RESOURCE_ALREADY_PROTECTED));

                String resourceNames = resource.get(RESOURCE_NAMES);
                String hostName = resource.get(RESOURCE_HOST);
                String resourceType = resource.get(RESOURCE_TYPE);

                setPluginCode(resourceType);

                SmDiscoverResourcesResponse smDiscoverResourcesResponse = smsCommandRequestValidationHelper
                        .getDiscoverResourceDetailsFromServer(
                                SmPluginCode.fromValue(pluginCode.name()),
                                hostName, true);

                String smObjectGuid = ResourceProtectionGroupHelper
                        .getSmObjectGuidFromResourceName(
                                smDiscoverResourcesResponse, resourceNames);

                // Verify whether resource is already protected by contacting
                // CSM server. If not protected throw error.
                SmGetProtectionGroupsResponse smGetProtectionGroupsResponse = smsCommandRequestValidationHelper
                        .getInternalProtectionGroupDetailsFromServer(smObjectGuid);
                if (!ResourceProtectionGroupHelper
                        .isResourceAlreadyProtected(smGetProtectionGroupsResponse)) {
                    throw new CLIException(
                            MessageBundle
                                    .localizeMessage(
                                            CLIMsgKey.CLI_SMPROTECTRESOURCE_RESOURCE_NOT_PROTECTED,
                                            resourceNames));
                }
                protectoinGroupName = ResourceProtectionGroupHelper
                        .getInternalResourceGroupNameFromProtectionGroup(
                                smGetProtectionGroupsResponse, smObjectGuid);

            } catch (Exception exception) {
                throw exception;
            } finally {
                ProgressBarUtil.stopProgressBar(progressBar);

            }
        } else {
            protectoinGroupName = resourceGroup;
        }

    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmCreateBackupRequest, SmCreateBackupResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmCreateBackupRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmCreateBackupResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_CREATE_BACKUP_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void pollForJobStatus(SmResponse smResponse) throws CLIException {
        doPollForJobStatus(smResponse, isWaitForCompletion());
    }

    private void setPluginCode(String resourceType) throws CLIException {

        switch (resourceType) {
            case ORACLE_DATABASE:
                pluginCode = PluginCode.SCO;
                break;
            default:
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_FOR_SMO_RESOURCE_TYPE_SHOULD_BE_ORACLE_DATABASE,
                                        resourceType));
        }

    }

    public Map<String, String> getResource() {
        return resource;
    }

    public void setResource(Map<String, String> resource) {
        this.resource = resource;
    }

    public String getPolicyName() {
        return policyName;
    }

    public void setPolicyName(String policyName) {
        this.policyName = policyName;
    }

    public String getResourceGroup() {
        return resourceGroup;
    }

    public void setResourceGroup(String resourceGroup) {
        this.resourceGroup = resourceGroup;
    }

    public void setWaitForCompletion(boolean waitForCompletion) {
        this.waitForCompletion = waitForCompletion;
    }

    public boolean isWaitForCompletion() {
        return waitForCompletion;
    }

    public String getProtectoinGroupName() {
        return protectoinGroupName;
    }

    public void setProtectoinGroupName(String protectoinGruopName) {
        this.protectoinGroupName = protectoinGruopName;
    }
    
    public boolean isVerifyOnSecondary() {
        return verifyOnSecondary;
    }

    public void setVerifyOnSecondary(boolean verifyOnSecondary) {
        this.verifyOnSecondary = verifyOnSecondary;
    }

    public boolean isEnableVerification() {
        return enableVerification;
    }

    public void setEnableVerification(boolean enableVerification) {
        this.enableVerification = enableVerification;
    }


}
