/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/NewSmCloneCommand.java#4 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.io.File;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import com.netapp.snapcenter.cli.command.parameters.NotificationParameters;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.beust.jcommander.ParametersDelegate;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmCreateCloneRequest;
import com.netapp.smcore.plugin.contracts.SmCreateCloneResponse;
import com.netapp.smcore.plugin.contracts.SmGetHostResponse;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.snapcenter.cli.clone.contracts.OracleCloneSpecification;
import com.netapp.snapcenter.cli.command.parameters.PrePostScriptParameters;
import com.netapp.snapcenter.cli.command.parameters.SMOCloneParameters;
import com.netapp.snapcenter.cli.command.validation.CloneCommandParametersValidationUtil;
import com.netapp.snapcenter.cli.command.validation.OracleCloneSpecificationValidationUtil;
import com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper;
import com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.spl.common.utils.HostInfoUtil;
import com.netapp.snapcenter.cli.command.validation.SmsCommandRequestValidationHelper;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.newsmclone.command.description")
public class NewSmCloneCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(NewSmCloneCommand.class);

    @Parameter(names = { "-AppPluginCode" }, descriptionKey = "cli.newsmclone.command.appPluginCode.option.description")
    private PluginCode appPluginCode = PluginCode.SCO;

    @Parameter(names = { "-CloneToHost" }, descriptionKey = "cli.newsmclone.command.cloneToHost.option.description")
    private String cloneToHost;

    @Parameter(names = { "-WaitForCompletion" }, descriptionKey = "cli.newsmclone.command.waitForCompletion.option.description")
    private boolean waitForCompletion;
    
    @Parameter(names = { "-BackupName" }, descriptionKey = "cli.newsmclone.command.backupName.option.description")
    private String backupName;
    
    @Parameter(names = { "-CloneDatabaseSID" }, descriptionKey = "cli.newsmclone.command.cloneDatabaseSid.option.description")
    private String cloneDatabaseSid;

    @ParametersDelegate
    SMOCloneParameters smoCloneParameters = new SMOCloneParameters();

    @ParametersDelegate
    private PrePostScriptParameters prePostScriptParameters = new PrePostScriptParameters();

    @ParametersDelegate
    private NotificationParameters notificationParameters = new NotificationParameters();
    
    private String backupId;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmCreateCloneRequest, SmCreateCloneResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmCreateCloneRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmCreateCloneResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_CREATE_CLONE_SERVICE_PATH);
        return smsRestInvocationContext;
    }

	@Override
	protected void processCommandArguments() throws CLIException {

		switch (appPluginCode) {
		case SCO:
			if (!StringUtils.isBlank(cloneToHost)) {
				SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
						super.getSnapCenterServerServiceHelper());
				SmGetHostResponse smGetHostResponse = smsCommandRequestValidationHelper.getSmHost(cloneToHost);
				
				if (smGetHostResponse.getHostInfo().getHosts().getSmHost()
						.isEmpty()) {
					String error = MessageBundle.localizeMessage(
							CLIMsgKey.CLI_PARAMETER_SC_OPERATION_INVALID_HOST,
							cloneToHost);
					throw new CLIException(error);
				}
			}
			CloneCommandParametersValidationUtil
					.validateAndSetSMOCloneParameters(smoCloneParameters);
			SmsBackupValidationHelper smsBackupValidationHelper = new SmsBackupValidationHelper(
					getSnapCenterServerServiceHelper());
			String cloneSpecfile = smoCloneParameters
					.getOracleCloneSpecificationFile();
			try {
				JAXBContext jaxbContext = JAXBContext
						.newInstance(OracleCloneSpecification.class);
				Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
				OracleCloneSpecification oracleCloneSpecification = (OracleCloneSpecification) unmarshaller
						.unmarshal(new File(cloneSpecfile));
				if (oracleCloneSpecification != null
						&& !StringUtils.isBlank(cloneDatabaseSid)) {
					CLIUtils.writeToLogAndConsole(
							MessageBundle
									.localizeMessage(
											CLIMsgKey.CLI_NEWSMCLONE_COMMAND_SETTING_SPECIFIED_CLONE_DATABASE_SID,
											cloneDatabaseSid), logger,
							LogLevel.INFO);
					oracleCloneSpecification
							.setCloneDatabaseSid(cloneDatabaseSid);
				}
				OracleCloneSpecificationValidationUtil
						.validateCloneSpec(oracleCloneSpecification, COMMANDNAME);
				// Set Backup Name
				if (StringUtils.isBlank(backupName)) {
					setBackupName(oracleCloneSpecification.getBackupname());
				} else {
					CLIUtils.writeToLogAndConsole(
							"Triggering clone from backup " + backupName,
							logger, LogLevel.INFO);
					setBackupId(smsBackupValidationHelper
							.fetchBackupIdIfItIsValidOracleDataBackup(
									backupName,
									oracleCloneSpecification.getAppobjectid(),
									false));
					if (StringUtils.isBlank(getBackupId())) {
						throw new CLIException(
								MessageBundle
										.localizeMessage(
												CLIMsgKey.CLI_ORACLE_CLONESPEC_CREATION_NOT_VALID_BACKUP_NAME_OR_RESOURCE_NOT_IN_BACKUP,
												getBackupName(),
												oracleCloneSpecification
														.getAppobjectid()));
					}
					oracleCloneSpecification.setBackupname(backupName);
				}

				smoCloneParameters
						.setOracleCloneSpecification(oracleCloneSpecification);
			} catch (JAXBException e) {
				throw new CLIException(
						MessageBundle
								.localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_UNABLE_TO_PARSE_CLONE_SPEC_FILE),
						e);
			}
			break;

		default:
			throw new CLIException(MessageBundle.localizeMessage(
					CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
					appPluginCode.name()));
		}

		SmsCommandArgumentsValidationUtil
				.processAndValidatePrePostScriptParameters(
						getPrePostScriptParameters(), getAppPluginCode());

		SmsCommandRequestValidationHelper smsCommandRequestValidationHelper = new SmsCommandRequestValidationHelper(
				super.getSnapCenterServerServiceHelper());
		SmGetSMTPServerResponse smGetSMTPServerResponse = smsCommandRequestValidationHelper
				.getSmSMTPServerDetailsFromServer();

		SmsCommandArgumentsValidationUtil.validateAndSetNotificationParameters(
				notificationParameters, smGetSMTPServerResponse, COMMANDNAME);

		if (StringUtils.isBlank(getCloneToHost())) {
			setCloneToHost(HostInfoUtil.getCanonicalHostName());
			String usingDefaultHost = MessageBundle.localizeMessage(
					CLIMsgKey.CLI_NEWSMCLONE_USING_LOCALHOST_AS_DEFAULT,
					getCloneToHost());
			CLIUtils.writeToLogAndConsole(usingDefaultHost, logger,
					LogLevel.INFO);
		}

	}

    @Override
    protected <T> void pollForJobStatus(SmResponse smResponse)
            throws CLIException {
        doPollForJobStatus(smResponse, isWaitForCompletion());
    }

    public PluginCode getAppPluginCode() {
        return appPluginCode;
    }

    public void setAppPluginCode(PluginCode appPluginCode) {
        this.appPluginCode = appPluginCode;
    }

    public String getCloneToHost() {
        return cloneToHost;
    }

    public void setCloneToHost(String cloneToHost) {
        this.cloneToHost = cloneToHost;
    }

    public boolean isWaitForCompletion() {
        return waitForCompletion;
    }

    public void setWaitForCompletion(boolean waitForCompletion) {
        this.waitForCompletion = waitForCompletion;
    }

    public SMOCloneParameters getSmoCloneParameters() {
        return smoCloneParameters;
    }

    public void setSmoCloneParameters(SMOCloneParameters smoCloneParameters) {
        this.smoCloneParameters = smoCloneParameters;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public String getBackupName() {
        return backupName;
    }
    
    public void setBackupId(String backupId){
    	this.backupId = backupId;
    }
    
    public String getBackupId() {
        return backupId;
    }

    public PrePostScriptParameters getPrePostScriptParameters() {
        return prePostScriptParameters;
    }

    public void setPrePostScriptParameters(
            PrePostScriptParameters prePostScriptParameters) {
        this.prePostScriptParameters = prePostScriptParameters;
    }

    public NotificationParameters getNotificationParameters() {
        return notificationParameters;
    }

    public void setNotificationParameters(NotificationParameters notificationParameters) {
        this.notificationParameters = notificationParameters;
    }

}
