/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/RemoveSmCloneCommand.java#4 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SMResult;
import com.netapp.smcore.plugin.contracts.SmDeleteCloneRequest;
import com.netapp.smcore.plugin.contracts.SmDeleteCloneResponse;
import com.netapp.smcore.plugin.contracts.SmGetCloneNameResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmResponse;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.command.parser.NoParameterSplitter;
import com.netapp.snapcenter.cli.command.validation.InvokeSmCloneResourceValidationUtil;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.ProgressBar;
import com.netapp.snapcenter.cli.utils.ProgressBarUtil;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.removesmclone.command.description")
public class RemoveSmCloneCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMCLONE_COMMAND_NAME);

    @Parameter(names = { "-CloneName" }, descriptionKey = "cli.removesmclone.command.cloneName.option.description", splitter = CommaParameterSplitter.class)
    private List<String> cloneNames = new ArrayList<>();

    @Parameter(names = { "-CloneAppObjectId" }, descriptionKey = "cli.removesmclone.command.cloneAppObjectId.option.description")
    private String cloneAppObjectId;

    @Parameter(names = { "-AppPluginCode" }, descriptionKey = "cli.removesmclone.command.appPluginCode.option.description")
    private PluginCode appPluginCode = PluginCode.SCO;

    @Parameter(names = { "-WaitForCompletion" }, descriptionKey = "cli.removesmclone.command.waitForCompletion.option.description")
    private boolean waitForCompletion;

	@Parameter(names = { "-Force" }, descriptionKey = "cli.removesmclone.command.force.option.description")
	private boolean force;
    
 
	@Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
	protected void processCommandArguments() throws CLIException {

		ProgressBar progressBar = ProgressBarUtil
				.createAndStartProgressBar((MessageBundle
						.localizeMessage(CLIMsgKey.CLI_PROGRESSBAR_CONTACTING_SNAPCENTER_SERVER)));

		switch (appPluginCode) {
			case SCO:
				break;
			default:
				throw new CLIException(MessageBundle.localizeMessage(
						CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
						appPluginCode.name()));
		}

		// Only one clone name user can specify
		if (getCloneNames().size() > 1) {
			throw new CLIException(
					MessageBundle
							.localizeMessage(CLIMsgKey.CLI_REMOVESMCLONE_MULTIPLE_CLONE_NAMES_SPECIFIED_ERROR));
		}

		if (StringUtils.isEmpty(cloneAppObjectId)
				&& (getCloneNames().isEmpty())) {
			String errorMsg = MessageBundle
					.localizeMessage(CLIMsgKey.CLI_COMMAND_REMOVESMCLONE_ID_NAME_MISSING_ERROR);
			throw new CLIException(errorMsg);
		}

		InvokeSmCloneResourceValidationUtil invokeSmCloneResourceValidationUtil = new InvokeSmCloneResourceValidationUtil(
				super.getSnapCenterServerServiceHelper());

		if (StringUtils.isNotBlank(cloneAppObjectId)) {
			SmGetCloneNameResponse smGetCloneNameResponse = invokeSmCloneResourceValidationUtil
					.getCloneNameFromAppObjectId(cloneAppObjectId);
			cloneNames.add((smGetCloneNameResponse.getCloneName()));

		} else {
			invokeSmCloneResourceValidationUtil
					.validateCloneName(getCloneNames().iterator().next());
		}

		ProgressBarUtil.stopProgressBar(progressBar);

		CLIUtils.promptUserToContinue(MessageBundle
				.localizeMessage(CLIMsgKey.CLI_REMOVESMCLONE_PROMPT_USER_REMOVE_CLONE));
	}

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmDeleteCloneRequest, SmDeleteCloneResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext.setRequest((SmDeleteCloneRequest) cmdRequest);
        smsRestInvocationContext.setResponseClass(SmDeleteCloneResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_DELETE_CLONE_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    @Override
    protected <T> void pollForJobStatus(SmResponse smResponse)
            throws CLIException {
        doPollForJobStatus(smResponse, isWaitForCompletion());
    }

    public void setCloneNames(List<String> cloneNames) {
        this.cloneNames = cloneNames;
    }

    public List<String> getCloneNames() {
        return cloneNames;
    }

    public void setAppPluginCode(PluginCode appPluginCode) {
        this.appPluginCode = appPluginCode;
    }

    public PluginCode getAppPluginCode() {
        return appPluginCode;
    }

    public void setWaitForCompletion(boolean waitForCompletion) {
        this.waitForCompletion = waitForCompletion;
    }

    public boolean isWaitForCompletion() {
        return waitForCompletion;
    }
    
    public boolean isForce() {
 		return force;
 	}

 	public void setForce(boolean force) {
 		this.force = force;
 	}
}