/*
 * $Id: //depot/prod/orion/sc_main/mintaka/spl/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/RemoveSmHostCommand.java#1 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import java.util.ArrayList;
import java.util.List;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmUnRegisterHostsRequest;
import com.netapp.smcore.plugin.contracts.SmUnRegisterHostsResponse;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.removesmhost.command.description")
public class RemoveSmHostCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESMHOST_COMMAND_NAME);

    /*
     * TODO: Should not mandate. If not specified then default to localhost and
     * ask prompt user for confirmation
     */
    @Parameter(names = { "-HostNames" }, descriptionKey = "cli.removesmhost.command.hostNames.option.description", required = true)
    private List<String> hostNames = new ArrayList<String>();

    @Parameter(names = { "-Force" }, descriptionKey = "cli.removesmhost.command.force.option.description")
    private boolean force;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
        CLIUtils.promptUserToContinue(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_REMOVESMHOST_PROMPT_USER_REMOVE_HOST));
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmUnRegisterHostsRequest, SmUnRegisterHostsResponse> smsRestInvocationContext = new SMSRestInvocationContext<SmUnRegisterHostsRequest, SmUnRegisterHostsResponse>();
        smsRestInvocationContext
                .setRequest((SmUnRegisterHostsRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmUnRegisterHostsResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_UNREGISTER_HOSTS_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    public void setHostNames(List<String> hostNames) {
        this.hostNames = hostNames;
    }

    public List<String> getHostNames() {
        return hostNames;
    }

    public boolean isForce() {
        return force;
    }

    public void setForce(boolean force) {
        this.force = force;
    }

}
