/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/RenameSmBackupCommand.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmUpdateBackupRequest;
import com.netapp.smcore.plugin.contracts.SmUpdateBackupResponse;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.renamesmbackup.command.description")
public class RenameSmBackupCommand extends AbstractSMSCommand {

    
	private static final String VALID_NAMES_REGEX = "^[a-zA-Z0-9._-]+$";
    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_RENAMESMBACKUP_COMMAND_NAME);
    
    @Parameter(names = { "-BackupName" }, descriptionKey = "cli.renamesmbackup.command.backupName.option.description", required = true)
    private String backupName;
    @Parameter(names = { "-NewBackupName" }, descriptionKey = "cli.renamesmbackup.command.newBackupName.option.description", required = true)
    private String newBackupName;
    @Parameter(names = { "-PluginType" }, descriptionKey = "cli.renamesmbackup.command.pluginType.option.description")
    private PluginCode pluginType = PluginCode.SCO;;
    
    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {
    	
    	
        
        if (StringUtils.isBlank(backupName)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_RENAMESMBACKUP_BACKUP_NAME_EMPTY));
        }
        if (StringUtils.isBlank(newBackupName)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_RENAMESMBACKUP_NEW_BACKUP_NAME_EMPTY));
        }
        
        if (!newBackupName.matches(VALID_NAMES_REGEX)) {
        	throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_RENAMESMBACKUP_INVALID_BACKUP_NAME));
        }
        
        switch (pluginType) {

            case SCO:
                break;

            default:
                throw new CLIException(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_COMMAND_NOT_SUPPORTED_FOR_PLUGIN,
                        pluginType.name()));
        }

        
    }
    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext <SmUpdateBackupRequest, SmUpdateBackupResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        SmUpdateBackupRequest request = (SmUpdateBackupRequest) cmdRequest;
        request.setRenameSnapShot(true);
        smsRestInvocationContext.setRequest(request);
        smsRestInvocationContext.setResponseClass(SmUpdateBackupResponse.class);
        smsRestInvocationContext.setServiceUrlPath(SMSRestConfiguration.SMS_RENAME_BACKUP_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    public String getBackupName() {
        return backupName;
    }

    public void setBackupName(String backupName) {
        this.backupName = backupName;
    }

    public String getNewBackupName() {
        return newBackupName;
    }

    public void setNewBackupName(String newBackupName) {
        this.newBackupName = newBackupName;
    }

    public PluginCode getPluginType() {
        return pluginType;
    }

    public void setPluginType(PluginCode pluginType) {
        this.pluginType = pluginType;
    }

    


}
