/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/SetSmRunAsCommand.java#3 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmModifyRunAsAccountRequest;
import com.netapp.smcore.plugin.contracts.SmModifyRunAsAccountResponse;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.snapcenter.cli.command.parser.AuthModeCoverter;
import com.netapp.snapcenter.cli.contracts.AuthMode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.setsmrunas.command.description")
public class SetSmRunAsCommand extends AbstractSMSCommand {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETSMRUNAS_COMMAND_NAME);

    @Parameter(names = { "-Name" }, descriptionKey = "cli.setsmrunas.command.name.option.description", required = true)
    private String runAsName;

    @Parameter(names = { "-CredentialType" }, descriptionKey = "cli.setsmrunas.command.authMode.option.description", required = true, converter = AuthModeCoverter.class)
    private AuthMode authMode;

    @Parameter(names = { "-Username" }, descriptionKey = "cli.setsmrunas.command.username.option.description")
    private String runAsUserName;

    /*
     * TODO: Should decide on having password option exposed
     */
    @Parameter(names = { "-Password" }, descriptionKey = "cli.setsmrunas.command.password.option.description", hidden = true)
    private String runAsPassword;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    protected void processCommandArguments() throws CLIException {

        if (StringUtils.isEmpty(getRunAsName())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_SETSMRUNAS_RUNAS_NAME_CANNOT_BE_EMPTY));
        }

        if (getRunAsUserName() == null) {
            String username = CLIUtils
                    .readLineFromConsole(
                            "%s: ",
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_SETSMRUNAS_PROMPT_USER_RUNAS_USERNAME));
            setRunAsUserName(username);
        }
        if (getRunAsPassword() == null) {
            String password = CLIUtils
                    .readPasswordFromConsole(
                            "%s: ",
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_SETSMRUNAS_PROMPT_USER_RUNAS_PASSWORD));
            setRunAsPassword(password);
        }
    }

    @Override
    protected <T extends SmRequest> SMSRestInvocationContext<?, ?> getCommandSpecificSMSRestInvocationContext(
            T cmdRequest) {
        SMSRestInvocationContext<SmModifyRunAsAccountRequest, SmModifyRunAsAccountResponse> smsRestInvocationContext = new SMSRestInvocationContext<>();
        smsRestInvocationContext
                .setRequest((SmModifyRunAsAccountRequest) cmdRequest);
        smsRestInvocationContext
                .setResponseClass(SmModifyRunAsAccountResponse.class);
        smsRestInvocationContext
                .setServiceUrlPath(SMSRestConfiguration.SMS_RBAC_MODIFY_RUN_AS_ACCOUNT_SERVICE_PATH);
        return smsRestInvocationContext;
    }

    public String getRunAsName() {
        return runAsName;
    }

    public void setRunAsName(String runAsName) {
        this.runAsName = runAsName;
    }

    public AuthMode getAuthMode() {
        return authMode;
    }

    public void setAuthMode(AuthMode authMode) {
        this.authMode = authMode;
    }

    public String getRunAsUserName() {
        return runAsUserName;
    }

    public void setRunAsUserName(String runAsUserName) {
        this.runAsUserName = runAsUserName;
    }

    public String getRunAsPassword() {
        return runAsPassword;
    }

    public void setRunAsPassword(String runAsPassword) {
        this.runAsPassword = runAsPassword;
    }

}
