/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/mapper/CommandClassMapBuilder.java#1 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.mapper;

import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSDStorageSystemId;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackup;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackupInfo;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackupPolicyVerificationSettings;
import com.netapp.smcore.plugin.contracts.ArrayOfSmHostResourceMapping;
import com.netapp.smcore.plugin.contracts.ArrayOfSmKeyValue;
import com.netapp.smcore.plugin.contracts.ArrayOfSmPolicy;
import com.netapp.smcore.plugin.contracts.ArrayOfSmString;
import com.netapp.smcore.plugin.contracts.ConfigSettingsType;
import com.netapp.smcore.plugin.contracts.SDStorageSystemId;
import com.netapp.smcore.plugin.contracts.SmAddStorageConnectionRequest;
import com.netapp.smcore.plugin.contracts.SmAuthMode;
import com.netapp.smcore.plugin.contracts.SmBackup;
import com.netapp.smcore.plugin.contracts.SmBackupInfo;
import com.netapp.smcore.plugin.contracts.SmBackupPolicyVerificationSettings;
import com.netapp.smcore.plugin.contracts.SmBackupVerificationType;
import com.netapp.smcore.plugin.contracts.SmCancelJobRequest;
import com.netapp.smcore.plugin.contracts.SmCatalogBackupRequest;
import com.netapp.smcore.plugin.contracts.SmCloneConfiguration;
import com.netapp.smcore.plugin.contracts.SmCloneConfigurationApplication;
import com.netapp.smcore.plugin.contracts.SmConfigurationType;
import com.netapp.smcore.plugin.contracts.SmCopyPolicyRequest;
import com.netapp.smcore.plugin.contracts.SmCreateBackupReportRequest;
import com.netapp.smcore.plugin.contracts.SmCreateBackupRequest;
import com.netapp.smcore.plugin.contracts.SmCreateCloneRequest;
import com.netapp.smcore.plugin.contracts.SmCreateJobSummaryReportRequest;
import com.netapp.smcore.plugin.contracts.SmCreatePolicyRequest;
import com.netapp.smcore.plugin.contracts.SmCreateProtectionGroupRequest;
import com.netapp.smcore.plugin.contracts.SmCreateRunAsAccountRequest;
import com.netapp.smcore.plugin.contracts.SmDeleteBackupsRequest;
import com.netapp.smcore.plugin.contracts.SmDeleteCloneRequest;
import com.netapp.smcore.plugin.contracts.SmDeletePolicyRequest;
import com.netapp.smcore.plugin.contracts.SmDeleteProtectionGroupRequest;
import com.netapp.smcore.plugin.contracts.SmDeleteRunAsAccountRequest;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourcesRequest;
import com.netapp.smcore.plugin.contracts.SmGetArchivedDetailsRequest;
import com.netapp.smcore.plugin.contracts.SmGetBackupDetailsRequest;
import com.netapp.smcore.plugin.contracts.SmGetBackupRequest;
import com.netapp.smcore.plugin.contracts.SmGetCloneRequest;
import com.netapp.smcore.plugin.contracts.SmGetConfigSettingsRequest;
import com.netapp.smcore.plugin.contracts.SmGetHostRequest;
import com.netapp.smcore.plugin.contracts.SmGetPolicyRequest;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsRequest;
import com.netapp.smcore.plugin.contracts.SmGetRunAsAccountRequest;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerRequest;
import com.netapp.smcore.plugin.contracts.SmGetStorageConnectionRequest;
import com.netapp.smcore.plugin.contracts.SmJob;
import com.netapp.smcore.plugin.contracts.SmKeyValue;
import com.netapp.smcore.plugin.contracts.SmKeyValueCollection;
import com.netapp.smcore.plugin.contracts.SmLoginRequest;
import com.netapp.smcore.plugin.contracts.SmModifyPolicyRequest;
import com.netapp.smcore.plugin.contracts.SmModifyProtectionGroupRequest;
import com.netapp.smcore.plugin.contracts.SmModifyRunAsAccountRequest;
import com.netapp.smcore.plugin.contracts.SmOracleCloneConfiguration;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.smcore.plugin.contracts.SmRecoveryRequest;
import com.netapp.smcore.plugin.contracts.SmRegisterHostsRequest;
import com.netapp.smcore.plugin.contracts.SmRemoveStorageConnectionRequest;
import com.netapp.smcore.plugin.contracts.SmSchedulerType;
import com.netapp.smcore.plugin.contracts.SmSetConfigSettingsRequest;
import com.netapp.smcore.plugin.contracts.SmString;
import com.netapp.smcore.plugin.contracts.SmUnRegisterHostsRequest;
import com.netapp.smcore.plugin.contracts.SmUncatalogBackupRequest;
import com.netapp.smcore.plugin.contracts.SmUpdateBackupRequest;
import com.netapp.smcore.plugin.contracts.SmVerificationPolicy;
import com.netapp.snapcenter.cli.command.AddSmHostCommand;
import com.netapp.snapcenter.cli.command.AddSmPolicyCommand;
import com.netapp.snapcenter.cli.command.AddSmResourceGroupCommand;
import com.netapp.snapcenter.cli.command.AddSmRunAsCommand;
import com.netapp.snapcenter.cli.command.AddSmStorageConnectionCommand;
import com.netapp.snapcenter.cli.command.CancelSmJobCommand;
import com.netapp.snapcenter.cli.command.CatalogSmBackupWithOracleRMANCommand;
import com.netapp.snapcenter.cli.command.CopySmPolicyCommand;
import com.netapp.snapcenter.cli.command.CreateSmOracleCloneSpecificationCommand;
import com.netapp.snapcenter.cli.command.GetSmBackupCommand;
import com.netapp.snapcenter.cli.command.GetSmBackupDetailsCommand;
import com.netapp.snapcenter.cli.command.GetSmBackupReportCommand;
import com.netapp.snapcenter.cli.command.GetSmCloneCommand;
import com.netapp.snapcenter.cli.command.GetSmConfigSettingsCommand;
import com.netapp.snapcenter.cli.command.GetSmHostCommand;
import com.netapp.snapcenter.cli.command.GetSmJobSummaryReportCommand;
import com.netapp.snapcenter.cli.command.GetSmPolicyCommand;
import com.netapp.snapcenter.cli.command.GetSmResourceGroupCommand;
import com.netapp.snapcenter.cli.command.GetSmResourcesCommand;
import com.netapp.snapcenter.cli.command.GetSmRunAsCommand;
import com.netapp.snapcenter.cli.command.GetSmSMTPServerCommand;
import com.netapp.snapcenter.cli.command.GetSmSecondaryDetailsCommand;
import com.netapp.snapcenter.cli.command.GetSmStorageConnectionCommand;
import com.netapp.snapcenter.cli.command.ModifySmPolicyCommand;
import com.netapp.snapcenter.cli.command.ModifySmResourceGroupCommand;
import com.netapp.snapcenter.cli.command.MountSmBackupCommand;
import com.netapp.snapcenter.cli.command.NewSmBackupCommand;
import com.netapp.snapcenter.cli.command.NewSmCloneCommand;
import com.netapp.snapcenter.cli.command.OpenSmConnectionCommand;
import com.netapp.snapcenter.cli.command.OpenSmConnectionCommand;
import com.netapp.snapcenter.cli.command.RefreshSmCloneCommand;
import com.netapp.snapcenter.cli.command.RemoveSmBackupCommand;
import com.netapp.snapcenter.cli.command.RemoveSmCloneCommand;
import com.netapp.snapcenter.cli.command.RemoveSmDataSetCommand;
import com.netapp.snapcenter.cli.command.RemoveSmHostCommand;
import com.netapp.snapcenter.cli.command.RemoveSmPolicyCommand;
import com.netapp.snapcenter.cli.command.RemoveSmProtectResourceCommand;
import com.netapp.snapcenter.cli.command.RemoveSmRunAsCommand;
import com.netapp.snapcenter.cli.command.RemoveSmStorageConnectionCommand;
import com.netapp.snapcenter.cli.command.RenameSmBackupCommand;
import com.netapp.snapcenter.cli.command.RestoreSmBackupCommand;
import com.netapp.snapcenter.cli.command.SetSmConfigSettingsCommand;
import com.netapp.snapcenter.cli.command.SetSmRunAsCommand;
import com.netapp.snapcenter.cli.command.SetSmSMTPServerCommand;
import com.netapp.snapcenter.cli.command.SetSmStorageConnectionCommand;
import com.netapp.snapcenter.cli.command.UncatalogSmBackupWithOracleRMANCommand;
import com.netapp.snapcenter.cli.command.UnmountSmBackupCommand;
import com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.snapcenter.cli.utils.SmoCloneCliUtil;

import ma.glasnost.orika.CustomMapper;
import ma.glasnost.orika.MapperFactory;
import ma.glasnost.orika.MappingContext;
import ma.glasnost.orika.metadata.ClassMapBuilder;

import java.util.HashMap;
import java.util.Map;

public class CommandClassMapBuilder {

	private CommandClassMapBuilder() {

	}

	private static final String SMSNAPDRIVERESOURCE = "SmSnapDriveResource";
	private static final String ORACLEDATABASE = "Oracle Database";
	private static final String VERIFICATION_POLICY_NAME_SUFFIX = "_Verification";
	private static final int DEFERRED_VERIFICATION_BACKUP_COUNT = 1;
	private static ExtendedGenericLogger logger = LoggerFactory
			.getExtendedClassAwareLogger(CommandClassMapBuilder.class);

	public static Map<String, Class<?>> createCommandToRequestTypeMap() {
		Map<String, Class<?>> commandToRequestType = new HashMap<>();

		commandToRequestType.put(OpenSmConnectionCommand.class.getSimpleName(),
				SmLoginRequest.class);

		commandToRequestType.put(AddSmHostCommand.class.getSimpleName(),
				SmRegisterHostsRequest.class);

		commandToRequestType.put(RemoveSmHostCommand.class.getSimpleName(),
				SmUnRegisterHostsRequest.class);

		commandToRequestType.put(GetSmHostCommand.class.getSimpleName(),
				SmGetHostRequest.class);

		commandToRequestType.put(GetSmResourcesCommand.class.getSimpleName(),
				SmDiscoverResourcesRequest.class);

		commandToRequestType.put(
				AddSmResourceGroupCommand.class.getSimpleName(),
				SmCreateProtectionGroupRequest.class);

		commandToRequestType.put(
				GetSmResourceGroupCommand.class.getSimpleName(),
				SmGetProtectionGroupsRequest.class);

		commandToRequestType.put(
				ModifySmResourceGroupCommand.class.getSimpleName(),
				SmModifyProtectionGroupRequest.class);

		commandToRequestType.put(AddSmPolicyCommand.class.getSimpleName(),
				SmCreatePolicyRequest.class);

		commandToRequestType.put(GetSmPolicyCommand.class.getSimpleName(),
				SmGetPolicyRequest.class);

		commandToRequestType.put(ModifySmPolicyCommand.class.getSimpleName(),
				SmModifyPolicyRequest.class);

		commandToRequestType.put(NewSmBackupCommand.class.getSimpleName(),
				SmCreateBackupRequest.class);

		commandToRequestType.put(GetSmBackupCommand.class.getSimpleName(),
				SmGetBackupRequest.class);

		commandToRequestType.put(
				GetSmSecondaryDetailsCommand.class.getSimpleName(),
				SmGetArchivedDetailsRequest.class);

		commandToRequestType.put(
				GetSmBackupDetailsCommand.class.getSimpleName(),
				SmGetBackupDetailsRequest.class);

		commandToRequestType.put(RemoveSmBackupCommand.class.getSimpleName(),
				SmDeleteBackupsRequest.class);

		commandToRequestType.put(
				GetSmJobSummaryReportCommand.class.getSimpleName(),
				SmCreateJobSummaryReportRequest.class);

		commandToRequestType.put(
				GetSmBackupReportCommand.class.getSimpleName(),
				SmCreateBackupReportRequest.class);

		commandToRequestType.put(RemoveSmDataSetCommand.class.getSimpleName(),
				SmDeleteProtectionGroupRequest.class);

		commandToRequestType.put(
				RemoveSmProtectResourceCommand.class.getSimpleName(),
				SmDeleteProtectionGroupRequest.class);

		commandToRequestType.put(RemoveSmPolicyCommand.class.getSimpleName(),
				SmDeletePolicyRequest.class);

		commandToRequestType.put(CopySmPolicyCommand.class.getSimpleName(),
				SmCopyPolicyRequest.class);

		commandToRequestType.put(RenameSmBackupCommand.class.getSimpleName(),
				SmUpdateBackupRequest.class);

		commandToRequestType.put(
				SetSmStorageConnectionCommand.class.getSimpleName(),
				SmAddStorageConnectionRequest.class);

		commandToRequestType.put(
				AddSmStorageConnectionCommand.class.getSimpleName(),
				SmAddStorageConnectionRequest.class);

		commandToRequestType.put(
				GetSmStorageConnectionCommand.class.getSimpleName(),
				SmGetStorageConnectionRequest.class);

		commandToRequestType.put(
				RemoveSmStorageConnectionCommand.class.getSimpleName(),
				SmRemoveStorageConnectionRequest.class);

		commandToRequestType.put(AddSmRunAsCommand.class.getSimpleName(),
				SmCreateRunAsAccountRequest.class);

		commandToRequestType.put(GetSmRunAsCommand.class.getSimpleName(),
				SmGetRunAsAccountRequest.class);

		commandToRequestType.put(RestoreSmBackupCommand.class.getSimpleName(),
				SmRecoveryRequest.class);

		commandToRequestType.put(
				CreateSmOracleCloneSpecificationCommand.class.getSimpleName(),
				SmGetBackupDetailsRequest.class);

		commandToRequestType.put(NewSmCloneCommand.class.getSimpleName(),
				SmCreateCloneRequest.class);

		commandToRequestType.put(GetSmCloneCommand.class.getSimpleName(),
				SmGetCloneRequest.class);

		commandToRequestType.put(RemoveSmCloneCommand.class.getSimpleName(),
				SmDeleteCloneRequest.class);

		commandToRequestType.put(MountSmBackupCommand.class.getSimpleName(),
				SmCreateCloneRequest.class);

		commandToRequestType.put(UnmountSmBackupCommand.class.getSimpleName(),
				SmDeleteCloneRequest.class);

		commandToRequestType.put(SetSmRunAsCommand.class.getSimpleName(),
				SmModifyRunAsAccountRequest.class);

		commandToRequestType.put(
				SetSmConfigSettingsCommand.class.getSimpleName(),
				SmSetConfigSettingsRequest.class);

		commandToRequestType.put(
				GetSmConfigSettingsCommand.class.getSimpleName(),
				SmGetConfigSettingsRequest.class);

		commandToRequestType.put(CancelSmJobCommand.class.getSimpleName(),
				SmCancelJobRequest.class);
		
		commandToRequestType.put(RefreshSmCloneCommand.class.getSimpleName(),
				SmCreateCloneRequest.class);

		/*
		 * //TODO delete
		 * commandToRequestType.put(SetSmLogSettingsCommand.class.getSimpleName
		 * (), SmSetLogSettingsRequest.class);
		 */

		commandToRequestType.put(RemoveSmRunAsCommand.class.getSimpleName(),
				SmDeleteRunAsAccountRequest.class);

		commandToRequestType.put(
				CatalogSmBackupWithOracleRMANCommand.class.getSimpleName(),
				SmCatalogBackupRequest.class);

		commandToRequestType.put(
				UncatalogSmBackupWithOracleRMANCommand.class.getSimpleName(),
				SmUncatalogBackupRequest.class);

        commandToRequestType.put(GetSmSMTPServerCommand.class.getSimpleName(),
                SmGetSMTPServerRequest.class);

        commandToRequestType.put(SetSmSMTPServerCommand.class.getSimpleName(),
                SmGetSMTPServerRequest.class);

		return commandToRequestType;
	}

	public static ClassMapBuilder<?, ?> buildOpenSmConnectionCommandClassMapper(
			MapperFactory mapperFactory) {

		return mapperFactory
				.classMap(OpenSmConnectionCommand.class, SmLoginRequest.class)
				.field("userName", "userOperationContext.user.name")
				.field("password", "userOperationContext.user.passphrase")
				.field("roleName", "userOperationContext.user.roleName")
				.field("tokenNeverExpires",
						"userOperationContext.user.tokenNeverExpires");

	}

	public static ClassMapBuilder<?, ?> buildAddSmHostCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(AddSmHostCommand.class, SmRegisterHostsRequest.class)
				.field("hostName", "hostEx.hostName")
				/*
				 * Server null check missing in case of RAC. Server should check
				 * for null. Till then we will have below initialization.
				 */
				.field("", "hostEx.members")
				.field("coloHost", "hostEx.coloHost")
				.field("port", "hostEx.port")
				.field("discoverPlugins", "hostEx.discoverPlugin")
				.field("osType", "hostEx.osInfo.operatingSystemType")
				.field("runAsName", "hostEx.auth.runAsName")
				.customize(
						new CustomMapper<AddSmHostCommand, SmRegisterHostsRequest>() {
							@Override
							public void mapAtoB(
									AddSmHostCommand addSmHostCommand,
									SmRegisterHostsRequest smRegisterHostsRequest,
									MappingContext context) {
								smRegisterHostsRequest
										.getHostEx()
										.setClusterHost(
												!addSmHostCommand
														.isDoNotAddClusterNodes());

								ArrayOfSmString hostIpAddresses = new ArrayOfSmString();
								if (!addSmHostCommand.getHostIpAddresses()
										.isEmpty()) {
									for (String ip : addSmHostCommand
											.getHostIpAddresses()) {
										SmString hostIp = new SmString();
										hostIp.setValue(ip);
										hostIpAddresses.getSmString().add(
												hostIp);
									}
								}
								smRegisterHostsRequest.getHostEx().setIPs(
										hostIpAddresses);
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildRemoveSmHostCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RemoveSmHostCommand.class,
						SmUnRegisterHostsRequest.class)
				.field("hostNames", "hostInfo.hosts.smHost")
				.field("hostNames{}", "hostInfo.hosts.smHost{hostName}")
				.field("force", "force");
	}

	public static ClassMapBuilder<?, ?> buildGetSmHostCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmHostCommand.class, SmGetHostRequest.class)
				.field("includeHostPluginInfo", "includeHostPluginInfo")
				.field("displayAll", "displayAll")
				.field("hostNames", "hostInfo.hosts.smHost")
				.field("hostNames{}", "hostInfo.hosts.smHost{hostName}");
	}

	public static ClassMapBuilder<?, ?> buildGetSmResourcesCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmResourcesCommand.class,
						SmDiscoverResourcesRequest.class)
				.field("hostName", "hostEx.hostName")
				.field("pluginCode",
						"resourceInfo.smDiscoverResourceInfo[0].pluginCode")
				.field("resources",
						"resourceInfo.smDiscoverResourceInfo[0].resources.smObject")
				.field("useKnownResources", "useKnownResources")
				.customize(
						new CustomMapper<GetSmResourcesCommand, SmDiscoverResourcesRequest>() {
							@Override
							public void mapAtoB(
									GetSmResourcesCommand getSmResourcesCommand,
									SmDiscoverResourcesRequest smDiscoverResourcesRequest,
									MappingContext context) {

								// Set context value of plugin code, which will
								// be used in server to delete the particular
								// nsm table
								if (getSmResourcesCommand.getPluginCode()
										.equals(PluginCode.SCO)) {
									smDiscoverResourcesRequest
											.setContext(CustomSMSObjectMapperHelper
													.setContextVariable(
															ORACLEDATABASE,
															Boolean.toString(true)));
								} else if (getSmResourcesCommand
										.getPluginCode().equals(PluginCode.SCU)) {
									smDiscoverResourcesRequest
											.setContext(CustomSMSObjectMapperHelper
													.setContextVariable(
															SMSNAPDRIVERESOURCE,
															Boolean.toString(true)));
								}
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildAddSmResourceGroupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(AddSmResourceGroupCommand.class,
						SmCreateProtectionGroupRequest.class)
				.field("resourceGroupName", "protectionGroup.name")
				.field("pluginCode", "pluginCode")
                .field("tag","protectionGroup.tag")
				.field("description", "protectionGroup.description")
				.field("backupResourceGroupParameters.smProtectionGroupType",
						"protectionGroup.protectionGroupType")
				.field("backupResourceGroupParameters.customSnapshot",
						"protectionGroup.isCustomSnapshot")
				.field("backupResourceGroupParameters.customSnapshotFormat",
						"protectionGroup.customSnapshotFormat")
				.field("backupResourceGroupParameters.customText",
						"protectionGroup.customText")
                .field("", "protectionGroup.auth")
                .field("backupResourceGroupParameters.schedulerRunAsName",
                        "protectionGroup.auth.runAsName")
				.field("backupResourceGroupParameters.notificationParameters.enableEmail",
						"protectionGroup.enableEmail")
				.field("backupResourceGroupParameters.notificationParameters.emailNotificationPreference",
						"protectionGroup.emailNotificationPreference")
				.field("backupResourceGroupParameters.notificationParameters.emailFrom",
						"protectionGroup.emailFrom")
				.field("backupResourceGroupParameters.notificationParameters.emailTo",
						"protectionGroup.emailTo")
				.field("backupResourceGroupParameters.notificationParameters.emailSubject",
						"protectionGroup.emailSubject")
				.field("backupResourceGroupParameters.notificationParameters.enableEmailAttachment",
						"protectionGroup.enableEmailAttachment")
				.customize(
						new CustomMapper<AddSmResourceGroupCommand, SmCreateProtectionGroupRequest>() {
							@Override
							public void mapAtoB(
									AddSmResourceGroupCommand addSmResourceGroupCommand,
									SmCreateProtectionGroupRequest smCreateProtectionGroupRequest,
									MappingContext context) {

								CLIUtils.writeToLog(
										"running mapping function for buildAddSmResourceGroupCommandClassMapper",
										logger, LogLevel.DEBUG);

								CLIUtils.writeToLog(
										"Setting HostResoueceMapping in SmCreateProtectionGroupRequest",
										logger, LogLevel.DEBUG);
								// Set Host Resource Mapping
								ArrayOfSmHostResourceMapping arrayOfSmHostResourceMapping = new ArrayOfSmHostResourceMapping();
								arrayOfSmHostResourceMapping
										.getSmHostResourceMapping()
										.addAll(CustomSMSObjectMapperHelper
												.createListOfSmHostResourceMappingsFromResources(addSmResourceGroupCommand
														.getResources()));

								smCreateProtectionGroupRequest
										.getProtectionGroup().setIsInternal(
												addSmResourceGroupCommand
														.getIsInternal());

								smCreateProtectionGroupRequest
										.getProtectionGroup()
										.setHostResourceMaping(
												arrayOfSmHostResourceMapping);

								ArrayOfSmPolicy arrayOfSmPolicy = new ArrayOfSmPolicy();
								if (CLIUtils
										.isNotNullAndNotEmptyCollection(addSmResourceGroupCommand
												.getResourceGroupPolicyList())) {
									arrayOfSmPolicy
											.getSmPolicy()
											.addAll(addSmResourceGroupCommand
													.getResourceGroupPolicyList());
								}
								smCreateProtectionGroupRequest
										.getProtectionGroup().setPolicies(
												arrayOfSmPolicy);

								// Set Verification Schedules
								if (addSmResourceGroupCommand
										.isVerificationEnabledForPolicy()) {
									ArrayOfSmBackupPolicyVerificationSettings arrayOfSmBackupPolicyVerificationSettings = new ArrayOfSmBackupPolicyVerificationSettings();
									arrayOfSmBackupPolicyVerificationSettings
											.getSmBackupPolicyVerificationSettings()
											.addAll(addSmResourceGroupCommand
													.getSmBackupPolicyVerificationSettings());
									smCreateProtectionGroupRequest
											.getProtectionGroup()
											.setVerificationSettings(
													arrayOfSmBackupPolicyVerificationSettings);
								}

								// Set Configuration for smo parameters
								CLIUtils.writeToLog(
										"Setting SCO specific parameters in SmCreateProtectionGroupRequest",
										logger, LogLevel.DEBUG);
								if (addSmResourceGroupCommand.getPluginCode()
										.equals(PluginCode.SCO)) {
									SmCloneConfiguration smCloneConfiguration = new SmCloneConfiguration();

									SmKeyValueCollection smKeyValueCollection = CustomSMSObjectMapperHelper
											.createSmKeyValueCollectionFromSmoBackupDatasetParameters(addSmResourceGroupCommand
													.getSmoBackupDataSetParameters());
									smCloneConfiguration
											.setDataCollection(smKeyValueCollection);
									smCloneConfiguration
											.setConfigurationType(SmConfigurationType.PROTECTION_GROUP);
									smCloneConfiguration
											.setName(addSmResourceGroupCommand
													.getResourceGroupName());

									// Set secondary locators for verification
									smCloneConfiguration
											.setArchivedLocators(CustomSMSObjectMapperHelper
													.createArrayOfSmArchivedLocatorFromSecondaryLocators(addSmResourceGroupCommand
															.getBackupResourceGroupParameters()
															.getSecondaryLocators()));

									smCreateProtectionGroupRequest
											.getProtectionGroup()
											.setConfiguration(
													smCloneConfiguration);

								}
							}

						});
	}

	public static ClassMapBuilder<?, ?> buildModifySmResourceGroupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(ModifySmResourceGroupCommand.class,
						SmModifyProtectionGroupRequest.class)
				.field("resourceGroupName", "protectionGroup.name")
				.field("pluginCode", "pluginCode")
                .field("tag","protectionGroup.tag")
				.field("description", "protectionGroup.description")
				.field("backupResourceGroupParameters.smProtectionGroupType",
						"protectionGroup.protectionGroupType")
				.field("backupResourceGroupParameters.customSnapshotFormat",
						"protectionGroup.customSnapshotFormat")
				.field("backupResourceGroupParameters.customText",
						"protectionGroup.customText")
                .field("", "protectionGroup.auth")
                .field("backupResourceGroupParameters.schedulerRunAsName",
                        "protectionGroup.auth.runAsName")
				.field("backupResourceGroupParameters.notificationParameters.enableEmail",
						"protectionGroup.enableEmail")
				.field("backupResourceGroupParameters.notificationParameters.emailNotificationPreference",
						"protectionGroup.emailNotificationPreference")
				.field("backupResourceGroupParameters.notificationParameters.emailFrom",
						"protectionGroup.emailFrom")
				.field("backupResourceGroupParameters.notificationParameters.emailTo",
						"protectionGroup.emailTo")
				.field("backupResourceGroupParameters.notificationParameters.emailSubject",
						"protectionGroup.emailSubject")
				.field("backupResourceGroupParameters.notificationParameters.enableEmailAttachment",
						"protectionGroup.enableEmailAttachment")
				.field("skipPolicyModification", "skipPolicyModification")
				.customize(
						new CustomMapper<ModifySmResourceGroupCommand, SmModifyProtectionGroupRequest>() {
							@Override
							public void mapAtoB(
									ModifySmResourceGroupCommand modifySmBackupDataSetCommand,
									SmModifyProtectionGroupRequest smModifyProtectionGroupRequest,
									MappingContext context) {

								if (modifySmBackupDataSetCommand
										.isSkipResourceModification()) {
									smModifyProtectionGroupRequest
											.setContext(CustomSMSObjectMapperHelper
													.setContextVariable(
															ModifySmResourceGroupCommand.SKIP_RESOURCE_MODIFICATION_CONTEXT_VARIABLE,
															Boolean.toString(modifySmBackupDataSetCommand
																	.isSkipResourceModification())));
								}

								smModifyProtectionGroupRequest
										.getProtectionGroup()
										.setIsCustomSnapshot(
												modifySmBackupDataSetCommand
														.getBackupResourceGroupParameters()
														.isCustomSnapshot());

								// Set Host Resource Mapping
								ArrayOfSmHostResourceMapping arrayOfSmHostResourceMapping = new ArrayOfSmHostResourceMapping();
								arrayOfSmHostResourceMapping
										.getSmHostResourceMapping()
										.addAll(CustomSMSObjectMapperHelper
												.createListOfSmHostResourceMappingsFromResources(modifySmBackupDataSetCommand
														.getResources()));
								smModifyProtectionGroupRequest
										.getProtectionGroup()
										.setHostResourceMaping(
												arrayOfSmHostResourceMapping);

								// set policy and schedule parameters
								// TODO : see if the
								// smModifyProtectionGroupRequest.getProtectionGroup().setPolicies(arrayOfSmPolicy);
								// command can be done during fields mapping
								// itself.Orika can handle this case.

								ArrayOfSmPolicy arrayOfSmPolicy = new ArrayOfSmPolicy();
								if (CLIUtils
										.isNotNullAndNotEmptyCollection(modifySmBackupDataSetCommand
												.getResourceGroupPolicyList())) {
									arrayOfSmPolicy
											.getSmPolicy()
											.addAll(modifySmBackupDataSetCommand
													.getResourceGroupPolicyList());
								}
								smModifyProtectionGroupRequest
										.getProtectionGroup().setPolicies(
												arrayOfSmPolicy);

								// Set Verification Schedules
								if (modifySmBackupDataSetCommand
										.isVerificationEnabledForPolicy()) {
									ArrayOfSmBackupPolicyVerificationSettings arrayOfSmBackupPolicyVerificationSettings = new ArrayOfSmBackupPolicyVerificationSettings();
									arrayOfSmBackupPolicyVerificationSettings
											.getSmBackupPolicyVerificationSettings()
											.addAll(modifySmBackupDataSetCommand
													.getSmBackupPolicyVerificationSettings());
									smModifyProtectionGroupRequest
											.getProtectionGroup()
											.setVerificationSettings(
													arrayOfSmBackupPolicyVerificationSettings);
								}

								// Set Configuration for smo parameters
								if (modifySmBackupDataSetCommand
										.getPluginCode().equals(PluginCode.SCO)) {
									SmCloneConfiguration smCloneConfiguration = new SmCloneConfiguration();

									SmKeyValueCollection smKeyValueCollection = CustomSMSObjectMapperHelper
											.createSmKeyValueCollectionFromSmoBackupDatasetParameters(modifySmBackupDataSetCommand
													.getSmoBackupDataSetParameters());
									smCloneConfiguration
											.setDataCollection(smKeyValueCollection);
									smCloneConfiguration
											.setConfigurationType(SmConfigurationType.PROTECTION_GROUP);
									smCloneConfiguration
											.setName(modifySmBackupDataSetCommand
													.getResourceGroupName());

									// Set secondary locators for verification
									smCloneConfiguration
											.setArchivedLocators(CustomSMSObjectMapperHelper
													.createArrayOfSmArchivedLocatorFromSecondaryLocators(modifySmBackupDataSetCommand
															.getBackupResourceGroupParameters()
															.getSecondaryLocators()));

									smModifyProtectionGroupRequest
											.getProtectionGroup()
											.setConfiguration(
													smCloneConfiguration);

								}

							}
						});
	}

	public static ClassMapBuilder<?, ?> buildGetSmResourceGroupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmResourceGroupCommand.class,
						SmGetProtectionGroupsRequest.class)
				.field("datasetName", "protectionGroup.name")
				.field("displayAll", "displayAll")
				.field("listPolicies", "isGetAllPolicies")
				.field("listResources", "isGetAllHostResourceMapping");
	}

	public static ClassMapBuilder<?, ?> buildAddSmPolicyCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(AddSmPolicyCommand.class, SmCreatePolicyRequest.class)
				.field("pluginPolicyType", "pluginCode")
				.customize(
						new CustomMapper<AddSmPolicyCommand, SmCreatePolicyRequest>() {
							@Override
							public void mapAtoB(
									AddSmPolicyCommand addSmPolicyCommand,
									SmCreatePolicyRequest smCreatePolicyRequest,
									MappingContext context) {
								smCreatePolicyRequest
										.setPolicy(CustomSMSPolicyObjectMapperHelper
												.createSmPolicy(addSmPolicyCommand));

								SmVerificationPolicy smVerificationPolicy = new SmVerificationPolicy();
								smVerificationPolicy.setName(addSmPolicyCommand
										.getPolicyName()
										+ VERIFICATION_POLICY_NAME_SUFFIX);
								CustomSMSPolicyObjectMapperHelper
										.fillSmVerificationPolicy(
												smVerificationPolicy,
												addSmPolicyCommand
														.getVerificationPrePostScriptParameters()
														.getVerificationScriptOptions());
								smCreatePolicyRequest
										.setVerificationPolicy(smVerificationPolicy);
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildGetSmPolicyCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmPolicyCommand.class, SmGetPolicyRequest.class)
				.field("policyName", "policy.name")
				.field("allowMultipleSchedules",
						"policy.allowMultipleSchedules")
				.field("datasetName", "protectionGroupName")
				.field("getAttachedPoliciesOnly", "getAttachedPoliciesOnly");
	}

	public static ClassMapBuilder<?, ?> buildModifySmPolicyCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(ModifySmPolicyCommand.class,
						SmModifyPolicyRequest.class)
				.field("pluginPolicyType", "pluginCode")
				.field("modifySchedulerType", "modifySchedulerType")
				.customize(
						new CustomMapper<ModifySmPolicyCommand, SmModifyPolicyRequest>() {
							@Override
							public void mapAtoB(
									ModifySmPolicyCommand modifySmPolicyCommand,
									SmModifyPolicyRequest smModifyPolicyRequest,
									MappingContext context) {
								smModifyPolicyRequest
										.setPolicy(CustomSMSPolicyObjectMapperHelper
												.modifySmPolicy(modifySmPolicyCommand));
								SmVerificationPolicy smVerificationPolicy = new SmVerificationPolicy();
								CustomSMSPolicyObjectMapperHelper
										.fillSmVerificationPolicy(
												smVerificationPolicy,
												modifySmPolicyCommand
														.getVerificationPrePostScriptParameters()
														.getVerificationScriptOptions());
								smModifyPolicyRequest
										.setVerificationPolicy(smVerificationPolicy);

							}
						});
	}

	public static ClassMapBuilder<?, ?> buildNewSmBackupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(NewSmBackupCommand.class, SmCreateBackupRequest.class)
				.field("policyName", "policy.name")
				.field("protectoinGroupName", "protectionGroup.name")
				.field("", "context.metadata.data")
				.customize(
						new CustomMapper<NewSmBackupCommand, SmCreateBackupRequest>() {
							@Override
							public void mapAtoB(
									NewSmBackupCommand newSmBackupCommand,
									SmCreateBackupRequest smCreateBackupRequest,
									MappingContext context) {

								// In line verification job was never getting
								// triggered, backup runs successfully and but
								// verification job was getting queued, since
								// there is not parent job to report. So below
								// hack is added to be in which verification
								// works correctly.
								SmJob smJob = new SmJob();
								smJob.setIsVisible(true);
								smCreateBackupRequest.setJob(smJob);

								if (newSmBackupCommand.isEnableVerification()) {
									if (smCreateBackupRequest
											.getProtectionGroup()
											.getVerificationSettings() == null) {
										ArrayOfSmBackupPolicyVerificationSettings arrayOfSmBackupPolicyVerificationSettings = new ArrayOfSmBackupPolicyVerificationSettings();
										smCreateBackupRequest
												.getProtectionGroup()
												.setVerificationSettings(
														arrayOfSmBackupPolicyVerificationSettings);
									}

									SmBackupPolicyVerificationSettings verificationSettings = new SmBackupPolicyVerificationSettings();
									verificationSettings
											.setDeferedVerificationBackupCnt(DEFERRED_VERIFICATION_BACKUP_COUNT);
									verificationSettings
											.setEnumVerificationType(SmBackupVerificationType.VERIFY_AFTER_BACKUP);
									verificationSettings
											.setBackupPolicyName(newSmBackupCommand
													.getPolicyName());
									verificationSettings
											.setScheduleType(SmSchedulerType.NONE);
									if (newSmBackupCommand
											.isVerifyOnSecondary()) {
										verificationSettings
												.setVerifyOnSecondaryStorage(newSmBackupCommand
														.isVerifyOnSecondary());
									}
									smCreateBackupRequest
											.getProtectionGroup()
											.getVerificationSettings()
											.getSmBackupPolicyVerificationSettings()
											.add(verificationSettings);
								}
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildGetSmBackupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmBackupCommand.class, SmGetBackupRequest.class)
				// This is done to mask null error from server
				.field("", "backup").field("backupName", "backup.backupName")
				.field("backupId", "backup.backupId")
				.field("appObjectName", "appObject.name")
				.field("appObjectId", "appObject.id")
				.field("secondary", "backupFilter.isSecondary")
				.field("backupType", "backupFilter.backupType")
				.field("jobId", "backupFilter.jobId")
				.field("pluginCode", "backupFilter.pluginCode");
	}

	public static ClassMapBuilder<?, ?> buildGetSmSecondaryDetailsCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmSecondaryDetailsCommand.class,
						SmGetArchivedDetailsRequest.class)
				.field("backupId", "backup.backupId")
				.field("pluginCode", "pluginCode")
				.field("appObjectId", "appObjects.smObject[0].id")
				.field("appObjectType", "appObjects.smObject[0].type");
	}

	public static ClassMapBuilder<?, ?> buildGetSmCloneCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmCloneCommand.class, SmGetCloneRequest.class)
				.field("cloneName", "clone.cloneName")
				.field("cloneId", "clone.cloneID")
				.field("appObjectName", "appObject.name")
				.field("backupName", "backup.backupName")
				.field("backupId", "backup.backupId")
				.field("pluginCode", "pluginCode");
	}

	public static ClassMapBuilder<?, ?> buildMountSmBackupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(MountSmBackupCommand.class,
						SmCreateCloneRequest.class)
				.field("backupName",
						"backups.smBackupInfo[0].primaryBackup.backupName")
				.field("appPluginCode", "pluginCode")
				.field("fileSystemMount", "fileSystemMount")
				.field("mountPath", "fileSystemMountPrefix")
				.field("hostName", "hostEx.hostName")
				// Server will cry if not initiated
				.field("", "hostEx.hostPluginInfos")
				.customize(
						new CustomMapper<MountSmBackupCommand, SmCreateCloneRequest>() {
							@Override
							public void mapAtoB(
									MountSmBackupCommand mountSmBackupCommand,
									SmCreateCloneRequest smCreateCloneRequest,
									MappingContext context) {
								SmCloneConfiguration smCloneConfiguration = new SmCloneConfiguration();

								SmCloneConfigurationApplication smCloneConfigurationApplication = CustomSMSObjectMapperHelper
										.buildApplicationConfiguration(
												mountSmBackupCommand,
												smCreateCloneRequest);

								ArrayOfSmHostResourceMapping arrayOfSmHostResourceMapping = new ArrayOfSmHostResourceMapping();
								arrayOfSmHostResourceMapping
										.getSmHostResourceMapping()
										.addAll(CustomSMSObjectMapperHelper
												.createListOfSmHostResourceMappingsForOracleDatabaseFromAppObjectId(mountSmBackupCommand
														.getAppObjectId()));
								smCloneConfigurationApplication
										.setResources(arrayOfSmHostResourceMapping);
								smCloneConfiguration
										.setCloneConfigurationApplication(smCloneConfigurationApplication);

								// Set Secondary mapping
								CustomSMSObjectMapperHelper.setArchiveLocators(
										mountSmBackupCommand,
										smCloneConfiguration);

								smCreateCloneRequest
										.setCloneConfiguration(smCloneConfiguration);
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildGetSmBackupDetailsCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmBackupDetailsCommand.class,
						SmGetBackupDetailsRequest.class)
				.field("backupName", "backup.backupName")
				.field("backupId", "backup.backupId")
				.field("appObjectId", "appObjects.smObject[0].id");
	}

	public static ClassMapBuilder<?, ?> buildCreateSmOracleCloneSpecificationCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(CreateSmOracleCloneSpecificationCommand.class,
						SmGetBackupDetailsRequest.class)
				.field("backupName", "backup.backupName")
				.field("backupId", "backup.backupId")
				.field("appObjectId", "appObjects.smObject[0].id");
	}

	public static ClassMapBuilder<?, ?> buildNewSmCloneCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(NewSmCloneCommand.class, SmCreateCloneRequest.class)
				.field("backupName",
						"backups.smBackupInfo[0].primaryBackup.backupName")
				.field("appPluginCode", "pluginCode")
				.field("", "policy")
				.field("prePostScriptParameters.preScriptPath",
						"policy.preScriptPath")
				.field("prePostScriptParameters.preScriptArguments",
						"policy.preScriptArguments")
				.field("prePostScriptParameters.postScriptPath",
						"policy.postScriptPath")
				.field("prePostScriptParameters.postScriptArguments",
						"policy.postScriptArguments")
				.field("prePostScriptParameters.scriptTimeout",
						"policy.scriptTimeOut")
				.field("notificationParameters.enableEmail",
						"protectionGroup.enableEmail")
				.field("notificationParameters.emailNotificationPreference",
						"protectionGroup.emailNotificationPreference")
				.field("notificationParameters.emailFrom",
						"protectionGroup.emailFrom")
				.field("notificationParameters.emailTo",
						"protectionGroup.emailTo")
				.field("notificationParameters.emailSubject",
						"protectionGroup.emailSubject")
				.field("notificationParameters.enableEmailAttachment",
						"protectionGroup.enableEmailAttachment")
				.customize(new CustomCloneCommandToRequestMapper());
	}
	
	public static ClassMapBuilder<?, ?> buildRefreshSmCloneCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RefreshSmCloneCommand.class, SmCreateCloneRequest.class)
				.field("backupName",
						"backups.smBackupInfo[0].primaryBackup.backupName")
				.field("appPluginCode", "pluginCode")
				.field("", "policy")
				.field("prePostScriptParameters.preScriptPath",
						"policy.preScriptPath")
				.field("prePostScriptParameters.preScriptArguments",
						"policy.preScriptArguments")
				.field("prePostScriptParameters.postScriptPath",
						"policy.postScriptPath")
				.field("prePostScriptParameters.postScriptArguments",
						"policy.postScriptArguments")
				.field("prePostScriptParameters.scriptTimeout",
						"policy.scriptTimeOut")
				.field("notificationParameters.enableEmail",
						"protectionGroup.enableEmail")
				.field("notificationParameters.emailNotificationPreference",
						"protectionGroup.emailNotificationPreference")
				.field("notificationParameters.emailFrom",
						"protectionGroup.emailFrom")
				.field("notificationParameters.emailTo",
						"protectionGroup.emailTo")
				.field("notificationParameters.emailSubject",
						"protectionGroup.emailSubject")
				.field("notificationParameters.enableEmailAttachment",
						"protectionGroup.enableEmailAttachment")
				.customize(new CustomRefreshCloneCommandToRequestMapper());
	}

	public static ClassMapBuilder<?, ?> buildRemoveSmBackupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RemoveSmBackupCommand.class,
						SmDeleteBackupsRequest.class)
				.field("deleteSecondaryMetadata", "deleteSecondaryMetadata")
				.field("cleanupSecondaryBackups", "cleanupSecondaryBackups")
				.field("cleanupBackups", "cleanupBackups")
				.customize(
						new CustomMapper<RemoveSmBackupCommand, SmDeleteBackupsRequest>() {
							@Override
							public void mapAtoB(
									RemoveSmBackupCommand removeSmBackupCommand,
									SmDeleteBackupsRequest smDeleteBackupsRequest,
									MappingContext context) {
								ArrayOfSmBackup arrayOfSmBackup = new ArrayOfSmBackup();

								if (CLIUtils
										.isNotNullAndNotEmptyCollection(removeSmBackupCommand
												.getBackupNames())) {
									for (String backupName : removeSmBackupCommand
											.getBackupNames()) {
										SmBackup smBackup = new SmBackup();
										smBackup.setBackupName(backupName);
										arrayOfSmBackup.getSmBackup().add(
												smBackup);
									}

								} else if (CLIUtils
										.isNotNullAndNotEmptyCollection(removeSmBackupCommand
												.getBackupIds())) {
									for (String backupId : removeSmBackupCommand
											.getBackupIds()) {
										SmBackup smBackup = new SmBackup();
										smBackup.setBackupId(backupId);
										arrayOfSmBackup.getSmBackup().add(
												smBackup);
									}
								}

								smDeleteBackupsRequest
										.setBackups(arrayOfSmBackup);
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildCatalogSmBackupWithOracleRMANCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(CatalogSmBackupWithOracleRMANCommand.class,
						SmCatalogBackupRequest.class)
				.field("pluginCode", "pluginCode")
				.customize(
						new CustomMapper<CatalogSmBackupWithOracleRMANCommand, SmCatalogBackupRequest>() {
							@Override
							public void mapAtoB(
									CatalogSmBackupWithOracleRMANCommand catalogSmBackupCommand,
									SmCatalogBackupRequest smCatalogBackupsRequest,
									MappingContext context) {

								ArrayOfSmBackupInfo arrayOfSmBackupInfo = new ArrayOfSmBackupInfo();

								SmBackupInfo smBackupInfo = new SmBackupInfo();
								SmBackup smBackup = new SmBackup();
								smBackup.setBackupName(catalogSmBackupCommand
										.getBackupName());
								smBackup.setBackupId(catalogSmBackupCommand
										.getBackupId());
								smBackupInfo.setPrimaryBackup(smBackup);
								arrayOfSmBackupInfo.getSmBackupInfo().add(
										smBackupInfo);

								smCatalogBackupsRequest
										.setBackups(arrayOfSmBackupInfo);
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildUncatalogSmBackupWithOracleRMANCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(UncatalogSmBackupWithOracleRMANCommand.class,
						SmUncatalogBackupRequest.class)
				.field("pluginCode", "pluginCode")
				.customize(
						new CustomMapper<UncatalogSmBackupWithOracleRMANCommand, SmUncatalogBackupRequest>() {
							@Override
							public void mapAtoB(
									UncatalogSmBackupWithOracleRMANCommand uncatalogSmBackupCommand,
									SmUncatalogBackupRequest smUncatalogBackupRequest,
									MappingContext context) {

								ArrayOfSmBackupInfo arrayOfSmBackupInfo = new ArrayOfSmBackupInfo();

								if (CLIUtils
										.isNotNullAndNotEmptyCollection(uncatalogSmBackupCommand
												.getBackupNames())) {
									for (String backupName : uncatalogSmBackupCommand
											.getBackupNames()) {
										SmBackupInfo backupInfo = new SmBackupInfo();
										SmBackup smBackup = new SmBackup();
										smBackup.setBackupName(backupName);
										backupInfo.setPrimaryBackup(smBackup);
										arrayOfSmBackupInfo.getSmBackupInfo()
												.add(backupInfo);
									}

								} else if (CLIUtils
										.isNotNullAndNotEmptyCollection(uncatalogSmBackupCommand
												.getBackupIds())) {
									for (String backupId : uncatalogSmBackupCommand
											.getBackupIds()) {
										SmBackupInfo backupInfo = new SmBackupInfo();
										SmBackup smBackup = new SmBackup();
										smBackup.setBackupId(backupId);
										backupInfo.setPrimaryBackup(smBackup);
										arrayOfSmBackupInfo.getSmBackupInfo()
												.add(backupInfo);
									}
								}

								smUncatalogBackupRequest
										.setBackups(arrayOfSmBackupInfo);
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildGetSmJobSummaryReportCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmJobSummaryReportCommand.class,
						SmCreateJobSummaryReportRequest.class)
				.field("date", "date").field("jobId", "jobId");
	}

	public static ClassMapBuilder<?, ?> buildGetSmBackupReportCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmBackupReportCommand.class,
						SmCreateBackupReportRequest.class)
				.field("backupId", "backupId").field("jobId", "jobId")
				.field("fromDate", "fromDateTime")
				.field("toDate", "toDateTime")
				.field("backupName", "backupName")
				.field("policyName", "policyName")
				.field("resourceName", "resourceName")
				.field("datasetName", "datasetName").field("status", "status");
	}

	public static ClassMapBuilder<?, ?> buildRemoveSmDataSetCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RemoveSmDataSetCommand.class,
						SmDeleteProtectionGroupRequest.class)
				.field("datasets", "protectionGroups.smProtectionGroup")
				.field("datasets{}", "protectionGroups.smProtectionGroup{name}")
				.field("force", "force");
	}

	public static ClassMapBuilder<?, ?> buildRemoveSmProtectResourceCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RemoveSmProtectResourceCommand.class,
						SmDeleteProtectionGroupRequest.class)
				.field("protectionGroups", "protectionGroups.smProtectionGroup")
				.field("protectionGroups{}",
						"protectionGroups.smProtectionGroup{name}")
				.field("force", "force");
	}

	public static ClassMapBuilder<?, ?> buildRemoveSmPolicyCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RemoveSmPolicyCommand.class,
						SmDeletePolicyRequest.class)
				.field("policies", "policies.smPolicy")
				.field("policies{}", "policies.smPolicy{name}");
	}

	public static ClassMapBuilder<?, ?> buildRenameSmBackupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RenameSmBackupCommand.class,
						SmUpdateBackupRequest.class)
				.field("backupName", "backup.backupName")
				.field("newBackupName", "newBackupName")
				.field("pluginType", "pluginCode");

	}

	public static ClassMapBuilder<?, ?> buildCopySmPolicyCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(CopySmPolicyCommand.class, SmCopyPolicyRequest.class)
				.field("policyName", "policy.name")
				.field("allowMultipleSchedules",
						"policy.allowMultipleSchedules")
				.field("newPolicyName", "newPolicyName");
	}

	public static ClassMapBuilder<?, ?> buildRemoveSmCloneCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RemoveSmCloneCommand.class,
						SmDeleteCloneRequest.class)
				.field("appPluginCode", "pluginCode")
				.field("cloneNames", "clones.smClone")
				.field("force", "forceMetadataDelete")
				.field("cloneNames{}", "clones.smClone{cloneName}");
	}

	public static ClassMapBuilder<?, ?> buildUnmountSmBackupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(UnmountSmBackupCommand.class,
						SmDeleteCloneRequest.class)
				.field("appPluginCode", "pluginCode")
				.field("backupName", "clones.smClone[0].backupName")
				.field("fileSystemMount", "fileSystemMount");
	}

	public static ClassMapBuilder<?, ?> buildSetSmStorageConnectionCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(SetSmStorageConnectionCommand.class,
						SmAddStorageConnectionRequest.class)
				.field("storageVM", "storageConnection.name")
				.field("port", "storageConnection.port")
				.field("protocol", "storageConnection.transportType")
				.field("smOperationMode", "mode")
				.field("svmUserName", "storageConnection.userName")
				.field("svmPassword", "storageConnection.passphrase")
				.field("preferredIpAddress",
						"storageConnection.preferredIpAddress")
				.field("timeout", "storageConnection.timeout")
				.field("isSecondary", "isSecondary")
				.customize(
						new CustomMapper<SetSmStorageConnectionCommand, SmAddStorageConnectionRequest>() {
							@Override
							public void mapAtoB(
									SetSmStorageConnectionCommand setSmStorageConnectionCommand,
									SmAddStorageConnectionRequest smAddStorageConnectionRequest,
									MappingContext context) {
								if (setSmStorageConnectionCommand
										.getEmsAsupConfigParameters()
										.isDisableAsupOnFailure()) {
									smAddStorageConnectionRequest
											.setEnableAsupOnFailure(false);
								} else {
									smAddStorageConnectionRequest
											.setEnableAsupOnFailure(true);
								}

								if (setSmStorageConnectionCommand
										.getEmsAsupConfigParameters()
										.isDisableSysLog()) {
									smAddStorageConnectionRequest
											.setEnableSysLog(false);
								} else {
									smAddStorageConnectionRequest
											.setEnableSysLog(true);
								}
							}

						});
	}

	public static ClassMapBuilder<?, ?> buildAddSmStorageConnectionCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(AddSmStorageConnectionCommand.class,
						SmAddStorageConnectionRequest.class)
				.field("storageVM", "storageConnection.name")
				.field("port", "storageConnection.port")
				.field("protocol", "storageConnection.transportType")
				.field("smOperationMode", "mode")
				.field("svmUserName", "storageConnection.userName")
				.field("svmPassword", "storageConnection.passphrase")
				.field("preferredIpAddress",
						"storageConnection.preferredIpAddress")
				.field("timeout", "storageConnection.timeout")
				.field("isSecondary", "isSecondary")
				.customize(
						new CustomMapper<AddSmStorageConnectionCommand, SmAddStorageConnectionRequest>() {
							@Override
							public void mapAtoB(
									AddSmStorageConnectionCommand addSmStorageConnectionCommand,
									SmAddStorageConnectionRequest smAddStorageConnectionRequest,
									MappingContext context) {
								if (addSmStorageConnectionCommand
										.getEmsAsupConfigParameters()
										.isDisableAsupOnFailure()) {
									smAddStorageConnectionRequest
											.setEnableAsupOnFailure(false);
								} else {
									smAddStorageConnectionRequest
											.setEnableAsupOnFailure(true);
								}

								if (addSmStorageConnectionCommand
										.getEmsAsupConfigParameters()
										.isDisableSysLog()) {
									smAddStorageConnectionRequest
											.setEnableSysLog(false);
								} else {
									smAddStorageConnectionRequest
											.setEnableSysLog(true);
								}
							}

						});
	
	}

	public static ClassMapBuilder<?, ?> buildRemoveSmStorageConnectionCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory.classMap(RemoveSmStorageConnectionCommand.class,
				SmRemoveStorageConnectionRequest.class).field("storageVM",
				"storageConnections.SDStorageSystemId[0].name");
	}

	public static ClassMapBuilder<?, ?> buildGetSmStorageConnectionCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmStorageConnectionCommand.class,
						SmGetStorageConnectionRequest.class)
				.field("displayAll", "displayAll")
				.customize(
						new CustomMapper<GetSmStorageConnectionCommand, SmGetStorageConnectionRequest>() {
							@Override
							public void mapAtoB(
									GetSmStorageConnectionCommand getSmStorageConnectionCommand,
									SmGetStorageConnectionRequest smGetStorageConnectionRequest,
									MappingContext context) {
								ArrayOfSDStorageSystemId storageConnection = new ArrayOfSDStorageSystemId();

								if (getSmStorageConnectionCommand
										.getStorageVM() != null) {

									SDStorageSystemId sdStorageSystemId = new SDStorageSystemId();
									sdStorageSystemId
											.setName(getSmStorageConnectionCommand
													.getStorageVM());
									storageConnection.getSDStorageSystemId()
											.add(sdStorageSystemId);
								}
								smGetStorageConnectionRequest
										.setStorageConnection(storageConnection);
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildAddSmRunAsCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(AddSmRunAsCommand.class,
						SmCreateRunAsAccountRequest.class)
				.field("runAsName", "auth.runAsName")
				.field("runAsUserName", "auth.userName")
				.field("runAsPassword", "auth.passpharse")
				.customize(
						new CustomMapper<AddSmRunAsCommand, SmCreateRunAsAccountRequest>() {

							@Override
							public void mapAtoB(
									AddSmRunAsCommand addSmRunAsCommand,
									SmCreateRunAsAccountRequest smCreateRunAsAccountRequest,
									MappingContext context) {
								smCreateRunAsAccountRequest
										.getAuth()
										.setAuthMode(
												SmAuthMode
														.valueOf(addSmRunAsCommand
																.getAuthMode()
																.value()));
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildSetSmRunAsCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(SetSmRunAsCommand.class,
						SmModifyRunAsAccountRequest.class)
				.field("runAsName", "auth.runAsName")
				.field("runAsUserName", "auth.userName")
				.field("runAsPassword", "auth.passpharse")
				.customize(
						new CustomMapper<SetSmRunAsCommand, SmModifyRunAsAccountRequest>() {

							@Override
							public void mapAtoB(
									SetSmRunAsCommand setSmRunAsCommand,
									SmModifyRunAsAccountRequest smModifyRunAsAccountRequest,
									MappingContext context) {
								smModifyRunAsAccountRequest
										.getAuth()
										.setAuthMode(
												SmAuthMode
														.valueOf(setSmRunAsCommand
																.getAuthMode()
																.value()));
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildGetSmRunAsCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(GetSmRunAsCommand.class,
						SmGetRunAsAccountRequest.class).field("", "auth")
				.field("runAsName", "auth.runAsName")
				.field("authMode", "auth.authMode");
	}

	public static ClassMapBuilder<?, ?> buildRemoveSmRunAsCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RemoveSmRunAsCommand.class,
						SmDeleteRunAsAccountRequest.class).field("", "auth")
				.field("runAsName", "auth.runAsName");
	}

	public static ClassMapBuilder<?, ?> buildRestoreSmBackupCommandClassMapper(
			MapperFactory mapperFactory) {
		return mapperFactory
				.classMap(RestoreSmBackupCommand.class, SmRecoveryRequest.class)
				.field("pluginCode", "pluginCode")
				.field("appObjectId", "appObject.id")
				.field("backupId", "backupInfo.primaryBackup.backupId")
				.field("backupName", "backupInfo.primaryBackup.backupName")
				// Server will throw null pointer if this is not initialized.
				.field("", "backupInfo.coBackups")
				.field("notificationParameters.emailNotificationPreference",
						"emailNotificationPreference")
				.field("notificationParameters.enableEmail",
						"enableEmail")
				.field("notificationParameters.emailFrom",
						"emailFrom")
				.field("notificationParameters.emailTo",
						"emailTo")
				.field("notificationParameters.emailSubject",
						"emailSubject")
				.field("notificationParameters.enableEmailAttachment",
						"enableEmailAttachment")
				.customize(
						new CustomMapper<RestoreSmBackupCommand, SmRecoveryRequest>() {
							@Override
							public void mapAtoB(
									RestoreSmBackupCommand restoreSmBackupCommand,
									SmRecoveryRequest smRecoveryRequest,
									MappingContext context) {
								CustomSMSRestoreObjectMapperHelper
										.populateRecoveryRequest(
												restoreSmBackupCommand,
												smRecoveryRequest);
								smRecoveryRequest
										.setArchivedLocators(CustomSMSObjectMapperHelper
												.createArrayOfSmArchivedLocatorFromSecondaryLocators(restoreSmBackupCommand
														.getSecondaryLocators()));
							}
						});
	}

	public static ClassMapBuilder<?, ?> buildSetSmConfigSettingsCommandClassMapper(
			MapperFactory mapperFactory) {

		logger.debug("Entering the buildSetSmConfigSettingsCommandClassMapper method");
		return mapperFactory
				.classMap(SetSmConfigSettingsCommand.class,
						SmSetConfigSettingsRequest.class)
				.field("configSettingsType",
						"configSettings.configSettingsType")
				.field("hostname", "configSettings.hostName")
				.customize(
						new CustomMapper<SetSmConfigSettingsCommand, SmSetConfigSettingsRequest>() {
							SmKeyValueCollection smKeyValueCollection = new SmKeyValueCollection();

							@Override
							public void mapAtoB(
									SetSmConfigSettingsCommand setSmConfigSettingsCommand,
									SmSetConfigSettingsRequest smSetConfigSettingsRequest,
									MappingContext context) {
								ArrayOfSmKeyValue arrayOfSmKeyValue = new ArrayOfSmKeyValue();
								for (Map<String, String> config : setSmConfigSettingsCommand
										.getConfigSettingsList()) {
									String key = config.get("KEY");
									String value = config.get("VALUE");
									SmKeyValue smKeyValue = new SmKeyValue();
									smKeyValue.setKey(key);
									smKeyValue.setValue(value);
									arrayOfSmKeyValue.getSmKeyValue().add(
											smKeyValue);
								}
								smKeyValueCollection.setData(arrayOfSmKeyValue);
								smSetConfigSettingsRequest
										.getConfigSettings()
										.setConfigSettings(smKeyValueCollection);
								if (setSmConfigSettingsCommand
										.getConfigSettingsType() == ConfigSettingsType.PLUGIN
										&& setSmConfigSettingsCommand
												.getPluginCode() != null) {
									smSetConfigSettingsRequest.setPluginCode(SmPluginCode
											.fromValue(setSmConfigSettingsCommand
													.getPluginCode().name()));
								}

							}

						});
	}

	public static ClassMapBuilder<?, ?> buildGetSmConfigSettingsCommandClassMapper(
			MapperFactory mapperFactory) {

		return mapperFactory
				.classMap(GetSmConfigSettingsCommand.class,
						SmGetConfigSettingsRequest.class).field("key", "key")
				.field("configSettingsType", "configSettingsType")
				.field("pluginCode", "pluginCode")
				.field("hostName", "hostName");
	}

	public static ClassMapBuilder<?, ?> buildCancelSmJobCommandClassMapper(
			MapperFactory mapperFactory) {

		return mapperFactory.classMap(CancelSmJobCommand.class,
				SmCancelJobRequest.class).field("jobId", "job.id");
	}

    public static ClassMapBuilder<?, ?> buildGetSmSMTPServerCommandClassMapper(
            MapperFactory mapperFactory) {

        return mapperFactory.classMap(GetSmSMTPServerCommand.class,
                SmGetSMTPServerRequest.class);
    }

    public static ClassMapBuilder<?, ?> buildSetSmSMTPServerCommandClassMapper(
            MapperFactory mapperFactory) {

        return mapperFactory
                .classMap(SetSmSMTPServerCommand.class,
                        SmGetSMTPServerRequest.class)
                .customize(
                        new CustomMapper<SetSmSMTPServerCommand, SmGetSMTPServerRequest>() {
                            @Override
                            public void mapAtoB(
                                    SetSmSMTPServerCommand setSmSMTPServerCommand,
                                    SmGetSMTPServerRequest smGetSMTPServerRequest,
                                    MappingContext context) {
                                smGetSMTPServerRequest
                                        .setSMTPEmailFrom(setSmSMTPServerCommand
                                                .getEmailFrom());
                                smGetSMTPServerRequest
                                        .setSMTPEmailTo(setSmSMTPServerCommand
                                                .getEmailTo());
                                smGetSMTPServerRequest
                                        .setSMTPServer(setSmSMTPServerCommand
                                                .getSmtpServer());
                                smGetSMTPServerRequest
                                        .setEmailSubject(setSmSMTPServerCommand
                                                .getEmailSubject());
                                smGetSMTPServerRequest
                                        .setIsSendEmail(setSmSMTPServerCommand
                                                .getIsSendEmail());

                                smGetSMTPServerRequest.setDisplayAll(true);

                            }
                        });
    }

	/*
	 * 
	 * TODO remove this after the log settings code is checked in
	 * 
	 * public static ClassMapBuilder<?, ?>
	 * buildSetSmLogSettingsCommandClassMapper( MapperFactory mapperFactory) {
	 * 
	 * logger.debug("Mapper factory " + mapperFactory.
	 * classMap(SetSmLogSettingsCommand.class, SmSetLogSettingsRequest.class)
	 * .field("universalTime", "logSettings.universalTime")
	 *//* .field("hostname", "logSettings.hostName") *//*
														 * .field(
														 * "maxSizeRollBackups",
														 * "logSettings.maxSizeRollBackups"
														 * ) .field(
														 * "jobLogsMaxFileSize",
														 * "logSettings.jobLogsMaxFileSize"
														 * ) .field("logLevel",
														 * "logSettings.logLevel"
														 * )
														 * .field("maxFileSize",
														 * "logSettings.maxFileSize"
														 * )
														 * .field("pluginCode",
														 * "logSettings.pluginCode"
														 * ));
														 * 
														 * 
														 * 
														 * return mapperFactory.
														 * classMap
														 * (SetSmLogSettingsCommand
														 * .class,
														 * SmSetLogSettingsRequest
														 * .class )
														 * .field("","logSettings"
														 * )
														 *//*
															 * .field("hostId",
															 * "logSettings.hostId"
															 * )
															 *//*
																 * .field(
																 * "universalTime"
																 * ,
																 * "logSettings.universalTime"
																 * )
																 * .field("hostname"
																 * ,
																 * "logSettings.hostName"
																 * )
																 *//*
																	 * .field(
																	 * "logSettingsId"
																	 * ,
																	 * "logSettings.logSettingsId"
																	 * )
																	 *//*
                *//*
																			 * .
																			 * field
																			 * (
																			 * "logSettingsType"
																			 * ,
																			 * "logSettings.logSettingsType"
																			 * )
																			 *//*
																				 * .
																				 * field
																				 * (
																				 * "maxSizeRollBackups"
																				 * ,
																				 * "logSettings.maxSizeRollBackups"
																				 * )
																				 * .
																				 * field
																				 * (
																				 * "jobLogsMaxFileSize"
																				 * ,
																				 * "logSettings.jobLogsMaxFileSize"
																				 * )
																				 * .
																				 * field
																				 * (
																				 * "logLevel"
																				 * ,
																				 * "logSettings.logLevel"
																				 * )
																				 * .
																				 * field
																				 * (
																				 * "maxFileSize"
																				 * ,
																				 * "logSettings.maxFileSize"
																				 * )
																				 *//*
																					 * .
																					 * field
																					 * (
																					 * "pluginInfoId"
																					 * ,
																					 * "logSettings.pluginInfoId"
																					 * )
																					 *//*
                *//*
																							 * .
																							 * field
																							 * (
																							 * "pluginEndpoint"
																							 * ,
																							 * "logSettings.pluginEndpoint"
																							 * )
																							 *//*
																								 * .
																								 * field
																								 * (
																								 * "pluginCode"
																								 * ,
																								 * "logSettings.pluginCode"
																								 * )
																								 * ;
																								 * 
																								 * 
																								 * 
																								 * }
																								 */

}

class CustomCloneCommandToRequestMapper extends
		CustomMapper<NewSmCloneCommand, SmCreateCloneRequest> {
	@Override
	public void mapAtoB(NewSmCloneCommand newSmCloneCommand,
			SmCreateCloneRequest smCreateCloneRequest, MappingContext context) {
		SmCloneConfiguration smCloneConfiguration = new SmCloneConfiguration();
		if (newSmCloneCommand.getAppPluginCode().equals(PluginCode.SCO)) {
			SmOracleCloneConfiguration smOracleCloneConfiguration = SmoCloneCliUtil
					.createSmOracleCloneConfiguration(newSmCloneCommand);
			smOracleCloneConfiguration.setSkipNIDCreation(newSmCloneCommand
					.getSmoCloneParameters().isSkipNIDCreation());
			smCloneConfiguration
					.setCloneConfigurationApplication(smOracleCloneConfiguration);
			smCloneConfiguration
					.setSuffix(smOracleCloneConfiguration.getName());
			smCloneConfiguration
					.setArchivedLocators(SmoCloneCliUtil
							.createSecondaryLocatorsForDatafilesFromOracleCloneSpecification(newSmCloneCommand
									.getSmoCloneParameters()
									.getOracleCloneSpecification()));
			smCloneConfiguration
					.setLogArchivedLocators(SmoCloneCliUtil
							.createSecondaryLocatorsForArchivelogsFromOracleCloneSpecification(newSmCloneCommand
									.getSmoCloneParameters()
									.getOracleCloneSpecification()));
		}
		smCreateCloneRequest.setCloneConfiguration(smCloneConfiguration);
	}
}
class CustomRefreshCloneCommandToRequestMapper extends
CustomMapper<RefreshSmCloneCommand, SmCreateCloneRequest> {
@Override
public void mapAtoB(RefreshSmCloneCommand newSmCloneCommand,
	SmCreateCloneRequest smCreateCloneRequest, MappingContext context) {
SmCloneConfiguration smCloneConfiguration = new SmCloneConfiguration();
if (newSmCloneCommand.getAppPluginCode().equals(PluginCode.SCO)) {
	SmOracleCloneConfiguration smOracleCloneConfiguration = SmoCloneCliUtil
			.createSmOracleCloneConfiguration(newSmCloneCommand);
	smOracleCloneConfiguration.setSkipNIDCreation(newSmCloneCommand
			.getSmoCloneParameters().isSkipNIDCreation());
	smCloneConfiguration
			.setCloneConfigurationApplication(smOracleCloneConfiguration);
	smCloneConfiguration
			.setSuffix(smOracleCloneConfiguration.getName());
	smCloneConfiguration
			.setArchivedLocators(SmoCloneCliUtil
					.createSecondaryLocatorsForDatafilesFromOracleCloneSpecification(newSmCloneCommand
							.getSmoCloneParameters()
							.getOracleCloneSpecification()));
	smCloneConfiguration
			.setLogArchivedLocators(SmoCloneCliUtil
					.createSecondaryLocatorsForArchivelogsFromOracleCloneSpecification(newSmCloneCommand
							.getSmoCloneParameters()
							.getOracleCloneSpecification()));
}
smCreateCloneRequest.setCloneConfiguration(smCloneConfiguration);
}
}