/*
 * $Id: //depot/prod/orion/sc_main/mintaka/spl/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/mapper/CommandMapperImpl.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.mapper;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;

import ma.glasnost.orika.MapperFacade;
import ma.glasnost.orika.MapperFactory;
import ma.glasnost.orika.impl.DefaultMapperFactory;
import ma.glasnost.orika.metadata.ClassMapBuilder;

import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.command.CommandInterface;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

/**
 * Orika based mapping of command objects to target request objects
 */
public class CommandMapperImpl implements CommandMapperInterface {
    
    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CommandMapperImpl.class);
    
    private static MapperFactory mapperFactory;
    private static Map<String, Class<?>> commandToRequestType;

    public CommandMapperImpl() {
        if (mapperFactory == null) {
            registerCommandMappers();
        }
    }

    @SuppressWarnings("unchecked")
    public <T> T map(CommandInterface cmdObj) {
        ClassMapBuilder<?, ?> commandClassMapBuilder = getCommandClassMapBuilder(
                cmdObj, mapperFactory);
        mapperFactory.registerClassMap(commandClassMapBuilder);
        MapperFacade mapper = mapperFactory.getMapperFacade();
        Class<?> commandMappedType = getMappedType(cmdObj);
        T smRequest = (T) mapper.map(cmdObj, commandMappedType);

        return smRequest;
    }

    private ClassMapBuilder<?, ?> getCommandClassMapBuilder(
            CommandInterface cmdObj, MapperFactory mapperFactory) {
        ClassMapBuilder<?, ?> commandClassMapBuilder = null;
        try {
            Method method = CommandClassMapBuilder.class.getMethod("build"
                    + cmdObj.getClass().getSimpleName() + "ClassMapper",
                    MapperFactory.class);
            commandClassMapBuilder = (ClassMapBuilder<?, ?>) method.invoke(
                    null, mapperFactory);
        } catch (NoSuchMethodException | SecurityException
                | IllegalAccessException | IllegalArgumentException
                | InvocationTargetException e) {
            CLIUtils.writeToLog(e.getLocalizedMessage(), logger, LogLevel.DEBUG);
        }
        return commandClassMapBuilder;
    }

    private static Class<?> getMappedType(CommandInterface cmdObj) {
        Class<?> commandMappedType = commandToRequestType.get(cmdObj.getClass()
                .getSimpleName());
        return commandMappedType;
    }

    private static MapperFactory registerCommandMappers() {

        mapperFactory = new DefaultMapperFactory.Builder().build();
        commandToRequestType = CommandClassMapBuilder
                .createCommandToRequestTypeMap();
        return mapperFactory;
    }

}
