/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/mapper/CustomSMSObjectMapperHelper.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.mapper;

import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.ORACLE_DATABASE;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_HOST;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_NAMES;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_NAMES_DELIMETER;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToHostResourcesMapConverter.RESOURCE_TYPE;
import static com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil.PRIMARY;
import static com.netapp.snapcenter.cli.command.validation.SmsCommandArgumentsValidationUtil.SECONDARY;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.ArrayOfSmArchivedLocator;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackupInfo;
import com.netapp.smcore.plugin.contracts.ArrayOfSmKeyValue;
import com.netapp.smcore.plugin.contracts.ArrayOfSmObject;
import com.netapp.smcore.plugin.contracts.ArrayOfSmString;
import com.netapp.smcore.plugin.contracts.ArrayOfString;
import com.netapp.smcore.plugin.contracts.SmArchivedLocator;
import com.netapp.smcore.plugin.contracts.SmAttachPolicyRequest;
import com.netapp.smcore.plugin.contracts.SmAuth;
import com.netapp.smcore.plugin.contracts.SmAuthMode;
import com.netapp.smcore.plugin.contracts.SmBackup;
import com.netapp.smcore.plugin.contracts.SmBackupInfo;
import com.netapp.smcore.plugin.contracts.SmCloneConfiguration;
import com.netapp.smcore.plugin.contracts.SmCloneConfigurationApplication;
import com.netapp.smcore.plugin.contracts.SmContext;
import com.netapp.smcore.plugin.contracts.SmCreateCloneRequest;
import com.netapp.smcore.plugin.contracts.SmHostResourceMapping;
import com.netapp.smcore.plugin.contracts.SmKeyValue;
import com.netapp.smcore.plugin.contracts.SmKeyValueCollection;
import com.netapp.smcore.plugin.contracts.SmOracleCloneConfiguration;
import com.netapp.smcore.plugin.contracts.SmOracleDb;
import com.netapp.smcore.plugin.contracts.SmPolicy;
import com.netapp.smcore.plugin.contracts.SmProtectionGroup;
import com.netapp.smcore.plugin.contracts.SmRequest;
import com.netapp.smcore.plugin.contracts.SmString;
import com.netapp.smcore.plugin.contracts.SmVerifyBackupRequest;
import com.netapp.smcore.plugin.contracts.SnapshotLocator;
import com.netapp.snapcenter.cli.command.MountSmBackupCommand;
import com.netapp.snapcenter.cli.command.parameters.SMOBackupDataSetParameters;
import com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;

/**
 * 
 * This class contains the generic custom mappers for SMS request objects.
 *
 */
public class CustomSMSObjectMapperHelper {

    private CustomSMSObjectMapperHelper() {

    }

    public static List<SmHostResourceMapping> createListOfSmHostResourceMappingsFromResources(
            List<Map<String, String>> resources) {

        List<SmHostResourceMapping> smHostResourceMappings = new ArrayList<>();

        for (Map<String, String> resource : resources) {

            SmHostResourceMapping smHostResourceMapping = new SmHostResourceMapping();

            String[] dbs = resource.get(RESOURCE_NAMES).split(
                    RESOURCE_NAMES_DELIMETER);

            ArrayOfSmObject arrayOfSmObject = new ArrayOfSmObject();

            for (int i = 0; i < dbs.length; i++) {
                String db = dbs[i].trim();
                if (resource.get(RESOURCE_TYPE).equalsIgnoreCase(
                        ORACLE_DATABASE)) {
                    SmOracleDb smOracleDb = new SmOracleDb();
                    smOracleDb.setHost(resource.get(RESOURCE_HOST));
                    smOracleDb.setId(resource.get(RESOURCE_HOST) + "\\" + db);
                    smOracleDb.setName(db);
                    smOracleDb.setType(ORACLE_DATABASE);
                    arrayOfSmObject.getSmObject().add(smOracleDb);
                } else {
                    /**
                     * TODO: Ideally host resource mapping creation logic for
                     * other plugin should be written here but since we support
                     * only Oracle database for Now let's validate only for
                     * Oracle database. Once we support other plugin resources
                     * creation we can remove this and add the implementation
                     * code.
                     */
                    throw new IllegalArgumentException(
                            MessageBundle
                                    .localizeMessage(CLIMsgKey.CLI_HOST_RESOURCE_MAPPING_FOR_SMO_RESOURCE_TYPE_SHOULD_BE_ORACLE_DATABASE));

                    /*
                     * SmObject smObject = new SmObject();
                     * smObject.setHost(resource.get(RESOURCE_HOST));
                     * smObject.setId(dbs[i]); smObject.setName(dbs[i]);
                     * smObject.setType(resource.get(RESOURCE_TYPE));
                     * arrayOfSmObject.getSmObject().add(smObject);
                     */
                }
            }
            smHostResourceMapping.setHostName(resource.get(RESOURCE_HOST));
            smHostResourceMapping.setAppObjects(arrayOfSmObject);
            smHostResourceMappings.add(smHostResourceMapping);

        }

        return smHostResourceMappings;

    }

    public static SmKeyValueCollection createSmKeyValueCollectionFromSmoBackupDatasetParameters(
            SMOBackupDataSetParameters smoBackupDataSetParameters) {
        SmKeyValueCollection smKeyValueCollection = new SmKeyValueCollection();
        ArrayOfSmKeyValue arrayOfSmKeyValue = new ArrayOfSmKeyValue();

        SmKeyValue backupArchiveLogsBeforeMissingFiles = new SmKeyValue();
        backupArchiveLogsBeforeMissingFiles
                .setKey(SMOBackupDataSetParameters.BACKUP_ARCHIVELOG_BEFORE_MISSINGFILES);
        backupArchiveLogsBeforeMissingFiles.setValue(Boolean
                .toString(!smoBackupDataSetParameters
                        .isBackupArchiveLogsAfterRecentMissingOne()));
        arrayOfSmKeyValue.getSmKeyValue().add(
                backupArchiveLogsBeforeMissingFiles);

        ArrayOfSmString archiveLogsExclusion = new ArrayOfSmString();
        for (String exclArchLog : smoBackupDataSetParameters
                .getExcludeArchiveLogDestinationsFromBackup()) {
            SmString smString = new SmString();
            smString.setValue(exclArchLog.trim());
            archiveLogsExclusion.getSmString().add(smString);
        }
        SmKeyValue archiveLogsBackupExcludeFilesLike = new SmKeyValue();
        archiveLogsBackupExcludeFilesLike
                .setKey(SMOBackupDataSetParameters.ARCHIVED_LOGS_EXCLUDE_FILESLIKE);
        archiveLogsBackupExcludeFilesLike.setValues(archiveLogsExclusion);
        arrayOfSmKeyValue.getSmKeyValue()
                .add(archiveLogsBackupExcludeFilesLike);

        smKeyValueCollection.setData(arrayOfSmKeyValue);
        return smKeyValueCollection;
    }

    public static List<SmHostResourceMapping> createListOfSmHostResourceMappingsForOracleDatabaseFromAppObjectId(
            String appObjectId) {

        List<SmHostResourceMapping> smHostResourceMappings = new ArrayList<>();
        SmHostResourceMapping smHostResourceMapping = new SmHostResourceMapping();
        ArrayOfSmObject arrayOfSmObject = new ArrayOfSmObject();

        SmOracleDb smOracleDb = new SmOracleDb();
        smOracleDb.setName(StringUtils.split(appObjectId, "\\")[1]);
        smOracleDb.setId(appObjectId);
        smOracleDb.setType(ORACLE_DATABASE);

        arrayOfSmObject.getSmObject().add(smOracleDb);
        smHostResourceMapping.setAppObjects(arrayOfSmObject);
        smHostResourceMappings.add(smHostResourceMapping);

        return smHostResourceMappings;
    }

    public static SmCloneConfigurationApplication buildApplicationConfiguration(
            MountSmBackupCommand command, SmCreateCloneRequest mountRequest) {

        switch (mountRequest.getPluginCode()) {
            case SCO:
                SmOracleCloneConfiguration smOracleCloneConfiguration = new SmOracleCloneConfiguration();

                // Set Db Auth so that server won't hit null
                SmAuth smAuth = new SmAuth();
                smAuth.setAuthMode(SmAuthMode.NONE);
                smOracleCloneConfiguration.setAuth(smAuth);

                // Set Asm Auth
                smOracleCloneConfiguration.setAsmPort(command.getAsmPort());

                SmAuth asmAuth = new SmAuth();

                if (StringUtils.isNotBlank(command.getAsmRunAsName())) {
                    asmAuth.setAuthMode(SmAuthMode.ORACLE_ASM);
                    asmAuth.setRunAsName(command.getAsmRunAsName().trim());
                } else {
                    asmAuth.setAuthMode(SmAuthMode.NONE);
                }
                smOracleCloneConfiguration.setSmAsmAuth(asmAuth);

                return smOracleCloneConfiguration;

            default:
                return new SmCloneConfigurationApplication();
        }
    }

	public static ArrayOfSmArchivedLocator createArrayOfSmArchivedLocatorFromSecondaryLocators(
            List<Map<String, String>> secondaryLocators) {
        ArrayOfSmArchivedLocator arrayOfSmArchivedLocator = new ArrayOfSmArchivedLocator();
        if (secondaryLocators != null) {
            for (Map<String, String> sl : secondaryLocators) {
                SmArchivedLocator smArchivedLocator = createSmArchivedLocator(sl);
                arrayOfSmArchivedLocator.getSmArchivedLocator().add(
                        smArchivedLocator);
            }
        }
        return arrayOfSmArchivedLocator;
    }

	public static void setArchiveLocators(
			MountSmBackupCommand mountSmBackupCommand,
			SmCloneConfiguration smCloneConfiguration) {

		SmsBackupValidationHelper smsBackupValidationHelper = new SmsBackupValidationHelper(
				mountSmBackupCommand.getSnapCenterServerServiceHelper());

		ArrayOfSmArchivedLocator arrayOfSmArchivedLocator = new ArrayOfSmArchivedLocator();

		if (mountSmBackupCommand.getSecondaryLocators() != null) {
			for (Map<String, String> sl : mountSmBackupCommand
					.getSecondaryLocators()) {
				SmArchivedLocator smArchivedLocator = createSmArchivedLocator(sl);
				arrayOfSmArchivedLocator.getSmArchivedLocator().add(
						smArchivedLocator);
			}
		}

		if (mountSmBackupCommand.isDataBackup()) {

			smCloneConfiguration.setArchivedLocators(arrayOfSmArchivedLocator);

		} else {
			smCloneConfiguration
					.setLogArchivedLocators(arrayOfSmArchivedLocator);
		}

	}

    private static SmArchivedLocator createSmArchivedLocator(
            Map<String, String> sl) {
        SmArchivedLocator smArchivedLocator = new SmArchivedLocator();

        SnapshotLocator primary = new SnapshotLocator();
        String[] primarySvmVol = CLIUtils.getIPandExportPath(sl.get(PRIMARY));
        primary.setStorageSystemName(primarySvmVol[0].trim());
        primary.setStorageVolumeName(primarySvmVol[1].trim());
        smArchivedLocator.setPrimaryLocator(primary);

        SnapshotLocator secondary = new SnapshotLocator();
        String[] secondarySvmVol = CLIUtils.getIPandExportPath(sl.get(SECONDARY));
        secondary.setStorageSystemName(secondarySvmVol[0].trim());
        secondary.setStorageVolumeName(secondarySvmVol[1].trim());
        smArchivedLocator.setSecondaryLocator(secondary);

        return smArchivedLocator;
    }

    public static SmContext setContextVariable(String variableName,
            String variableValue) {
        SmContext smContext = new SmContext();
        SmKeyValueCollection smKeyValueCollection = new SmKeyValueCollection();
        ArrayOfSmKeyValue arrayOfSmKeyValue = new ArrayOfSmKeyValue();

        SmKeyValue smkeyvalue = new SmKeyValue();
        smkeyvalue.setKey(variableName);
        smkeyvalue.setValue(variableValue);

        arrayOfSmKeyValue.getSmKeyValue().add(smkeyvalue);
        smKeyValueCollection.setData(arrayOfSmKeyValue);
        smContext.setMetadata(smKeyValueCollection);
        return smContext;
    }


}
