/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/mapper/CustomSMSPolicyObjectMapperHelper.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.mapper;

import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.POST_SCRIPT_PATH;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.PRE_SCRIPT_PATH;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.POST_SCRIPT_ARGUMENTS;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.PRE_SCRIPT_ARGUMENTS;
import static com.netapp.snapcenter.cli.command.parser.CustomStringToPrePostScriptMapConverter.SCRIPT_TIMEOUT;

import java.util.List;
import java.util.Map;

import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.ToStringBuilder;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmAppPolicy;
import com.netapp.smcore.plugin.contracts.ArrayOfSmRetention;
import com.netapp.smcore.plugin.contracts.ArrayOfSmSchedule;
import com.netapp.smcore.plugin.contracts.SmBackupPolicy;
import com.netapp.smcore.plugin.contracts.SmBackupType;
import com.netapp.smcore.plugin.contracts.SmOracleBackupScope;
import com.netapp.smcore.plugin.contracts.SmOracleBackupType;
import com.netapp.smcore.plugin.contracts.SmOraclePolicy;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.smcore.plugin.contracts.SmPolicy;
import com.netapp.smcore.plugin.contracts.SmPolicyType;
import com.netapp.smcore.plugin.contracts.SmRetention;
import com.netapp.smcore.plugin.contracts.SmSchedule;
import com.netapp.smcore.plugin.contracts.SmSchedulerType;
import com.netapp.smcore.plugin.contracts.SmVerificationPolicy;
import com.netapp.smcore.plugin.contracts.SmoArchiveLogBackupRetentionType;
import com.netapp.smcore.plugin.contracts.SmoPruneArchiveLogDestinationType;
import com.netapp.smcore.plugin.contracts.SmoPruneArchiveLogType;
import com.netapp.snapcenter.cli.command.AbstractSMSCommand;
import com.netapp.snapcenter.cli.command.AddSmPolicyCommand;
import com.netapp.snapcenter.cli.command.ModifySmPolicyCommand;
import com.netapp.snapcenter.cli.command.parameters.BackupPolicyParameters;
import com.netapp.snapcenter.cli.command.parameters.CommonBackupCloneVerificationPolicyParameters;
import com.netapp.snapcenter.cli.command.parameters.PolicyRetentionSettingsParameters;
import com.netapp.snapcenter.cli.command.parameters.PrePostScriptParameters;
import com.netapp.snapcenter.cli.command.parameters.SMOBackupPolicyParameters;
import com.netapp.snapcenter.cli.command.parameters.VerificationPolicyParameters;
import com.netapp.snapcenter.cli.contracts.DaysOfTheWeek;
import com.netapp.snapcenter.cli.contracts.MonthsOfTheYear;
import com.netapp.snapcenter.cli.contracts.OracleBackupScope;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.contracts.ScheduleType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

public class CustomSMSPolicyObjectMapperHelper {

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CustomSMSPolicyObjectMapperHelper.class);

    private CustomSMSPolicyObjectMapperHelper() {

    }

    // TODO: SHould improve it further to support all policies
    public static SmPolicy createSmPolicy(AddSmPolicyCommand addSmPolicyCommand) {

        SmPolicy smPolicy = null;
        SmPolicyType smPolicyType = SmPolicyType.valueOf(addSmPolicyCommand
                .getPolicyType().name());

        switch (smPolicyType) {
            case BACKUP:
                smPolicy = createSmBackupPolicy(
                        addSmPolicyCommand.getBackupPolicyParameters(),
                        addSmPolicyCommand
                                .getListPolicyRetentionSettingsParameters(),
                        addSmPolicyCommand
                                .getCommonBackupClonePolicyParameters().getListVerificationScheduleTypes());
                addAppSpecificBackupPolicy(
                        addSmPolicyCommand.getPluginPolicyType(),
                        addSmPolicyCommand, smPolicy);
                break;
            // Not in SCO 1.1 release
            /*
             * case CLONE: smPolicy = createSmClonePolicy(addSmPolicyCommand
             * .getClonePolicyParameters()); break;
             */
            /*case VERIFICATION:
                smPolicy = createSmVerificationPolicy(addSmPolicyCommand
                        .getVerificationPolicyParameters());
                break;*/
            default:
                smPolicy = new SmPolicy();
                break;
        }
        smPolicy.setName(addSmPolicyCommand.getPolicyName());
        smPolicy.setPolicyType(smPolicyType);
        smPolicy.setDescription(addSmPolicyCommand.getDescription());
        smPolicy.setPluginPolicyType(SmPluginCode.fromValue(addSmPolicyCommand.getPluginPolicyType().name()));
        populateGenericPolicy(addSmPolicyCommand.getPrePostScriptParameters(),
                addSmPolicyCommand.getCommonBackupClonePolicyParameters(), 
                smPolicy);
        smPolicy.setAllowMultipleSchedules(true);

        return smPolicy;
    }

    public static void fillSmVerificationPolicy(
            SmVerificationPolicy smVerificationPolicy,
            Map<String, String> verificationScriptOptionsMap) {

        if (StringUtils.isNotEmpty(verificationScriptOptionsMap
                .get(SCRIPT_TIMEOUT.toLowerCase()))) {

            smVerificationPolicy.setScriptTimeOut(Integer
                    .parseInt(verificationScriptOptionsMap.get(SCRIPT_TIMEOUT
                            .toLowerCase())));

        }

        if (StringUtils.isNotEmpty(verificationScriptOptionsMap
                .get(PRE_SCRIPT_PATH.toLowerCase()))) {
            smVerificationPolicy.setPreScriptPath(verificationScriptOptionsMap
                    .get(PRE_SCRIPT_PATH.toLowerCase()));

        }

        if (StringUtils.isNotEmpty(verificationScriptOptionsMap
                .get(PRE_SCRIPT_ARGUMENTS.toLowerCase()))) {
            smVerificationPolicy
                    .setPreScriptArguments(verificationScriptOptionsMap
                            .get(PRE_SCRIPT_ARGUMENTS.toLowerCase()));

        }
        if (StringUtils.isNotEmpty(verificationScriptOptionsMap
                .get(POST_SCRIPT_PATH.toLowerCase()))) {
            smVerificationPolicy.setPostScriptPath(verificationScriptOptionsMap
                    .get(POST_SCRIPT_PATH.toLowerCase()));

        }
        if (StringUtils.isNotEmpty(verificationScriptOptionsMap
                .get(POST_SCRIPT_ARGUMENTS.toLowerCase()))) {
            smVerificationPolicy
                    .setPostScriptArguments(verificationScriptOptionsMap
                            .get(POST_SCRIPT_ARGUMENTS.toLowerCase()));

        }

    }

    private static SmBackupPolicy createSmBackupPolicy(
            BackupPolicyParameters backupPolicyParameters,
            List<PolicyRetentionSettingsParameters> listPolicyRetentionSettingsParametersMappers,
            List<ScheduleType> verifcationScheduleTypes) {
        SmBackupPolicy smBackupPolicy = new SmBackupPolicy();
        /*
         * Now retention is applied based on the values in the nsm_Retention and this flag is not necessarily required.
         * But Applying rentention checks for this flag as well. So setting it to true always.
         * While we can remove this flag it self, not doing so because we would need this if we want to add back no retention flag.
         * This is taken from powershell.
         */
         smBackupPolicy.setApplyRetention(true);

        smBackupPolicy.setApplySnapMirrorUpdate(backupPolicyParameters
                .isUpdateSnapMirrorAfterbackup());
        smBackupPolicy.setApplySnapvaultUpdate(backupPolicyParameters
                .isUpdateSnapVaultAfterbackup());

        smBackupPolicy.setMirrorVaultUpdateRetryCount(backupPolicyParameters
                .getMirrorVaultUpdateRetryCount());

        if (StringUtils.isNotEmpty(backupPolicyParameters.getSnapVaultLabel())) {
            if (backupPolicyParameters.isUpdateSnapVaultAfterbackup()) {
                smBackupPolicy.setSnapVaultLabel(backupPolicyParameters
                        .getSnapVaultLabel());
            } else {
                CLIUtils.writeToLogAndConsole(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_SMPOLICY_IGNORE_SNAPVAULT_LABEL),
                        logger, LogLevel.INFO);
            }

        }

        ArrayOfSmRetention arrayOfSmRetention = new ArrayOfSmRetention();
        for (PolicyRetentionSettingsParameters policyRetentionSettingsParametersMapper : listPolicyRetentionSettingsParametersMappers) {
            SmRetention smRetention = new SmRetention();

            ScheduleType scheduleType = policyRetentionSettingsParametersMapper
                    .getScheduleType();

            OracleBackupScope oracleBackupScope = policyRetentionSettingsParametersMapper
                    .getOracleBackupScope();

            smRetention.setBackupType(SmBackupType.fromValue(oracleBackupScope
                    .toString()));
            smRetention.setSchedulerType(SmSchedulerType.fromValue(scheduleType
                    .value()));
            smRetention
                    .setRetentionCount(policyRetentionSettingsParametersMapper
                            .getDeleteBackupInExcess());
            smRetention
                    .setRetentionDays(policyRetentionSettingsParametersMapper
                            .getDeleteBackupOlderThan());

            if (verifcationScheduleTypes.contains(scheduleType)
                    && (!oracleBackupScope.equals(OracleBackupScope.LOG))) {
                smRetention.setVerificationEnabled(true);
            } else {
                smRetention.setVerificationEnabled(false);
            }

            arrayOfSmRetention.getSmRetention().add(smRetention);
        }

        smBackupPolicy.setRetentions(arrayOfSmRetention);

        return smBackupPolicy;
    }

    // Not in SCO 1.1 release
    /*
     * private static SmClonePolicy createSmClonePolicy( ClonePolicyParameters
     * clonePolicyParameters) { SmClonePolicy smClonePolicy = new
     * SmClonePolicy();
     * 
     * smClonePolicy.setCloneFromType(SmCloneType
     * .valueOf(clonePolicyParameters.getCloneType().name()));
     * smClonePolicy.setBackupPolicyName(clonePolicyParameters
     * .getBackupPolicyName());
     * smClonePolicy.setDeleteCloneOnScheduleExpiry(clonePolicyParameters
     * .isDeleteCloneOnScheduleExpiry());
     * 
     * return smClonePolicy; }
     */

   /* private static SmPolicy createSmVerificationPolicy(
            VerificationPolicyParameters verificationPolicyParameters) {
        SmVerificationPolicy smVerificationPolicy = new SmVerificationPolicy();

        smVerificationPolicy.setNumBackups(verificationPolicyParameters
                .getNumOfBackups());
        smVerificationPolicy
                .setVerifyOnSnapMirrorDestination(verificationPolicyParameters
                        .isVerifyOnSecondary());
        smVerificationPolicy
                .setVerifyOnSnapVaultDestination(verificationPolicyParameters
                        .isVerifyOnSecondary());

        return smVerificationPolicy;
    }
*/
    public static SmPolicy modifySmPolicy(
            ModifySmPolicyCommand modifySmPolicyCommand) {
        SmPolicy smPolicy = null;
        SmPolicyType smPolicyType = SmPolicyType.valueOf(modifySmPolicyCommand
                .getPolicyType().name());

        switch (smPolicyType) {
            case BACKUP:
                smPolicy = createSmBackupPolicy(
                        modifySmPolicyCommand.getBackupPolicyParameters(),
                        modifySmPolicyCommand
                                .getListPolicyRetentionSettingsParameters(),modifySmPolicyCommand.getCommonBackupClonePolicyParameters().getListVerificationScheduleTypes());
                addAppSpecificBackupPolicy(
                        modifySmPolicyCommand.getPluginPolicyType(),
                        modifySmPolicyCommand, smPolicy);
                break;
            // Clone policy Not in SCO 1.1 release
            /*
             * case CLONE: smPolicy = createSmClonePolicy(modifySmPolicyCommand
             * .getClonePolicyParameters()); break;
             */
            /*case VERIFICATION:
                smPolicy = createSmVerificationPolicy(modifySmPolicyCommand
                        .getVerificationPolicyParameters());
                break;*/
            default:
                smPolicy = new SmPolicy();
                break;
        }

        smPolicy.setName(modifySmPolicyCommand.getPolicyName());
        smPolicy.setPolicyType(smPolicyType);
        smPolicy.setDescription(modifySmPolicyCommand.getDescription());
        smPolicy.setAllowMultipleSchedules(true);
        smPolicy.setPluginPolicyType(SmPluginCode.fromValue(modifySmPolicyCommand.getPluginPolicyType().name()));
        populateGenericPolicy(
                modifySmPolicyCommand.getPrePostScriptParameters(),
                modifySmPolicyCommand.getCommonBackupClonePolicyParameters(),
                smPolicy);


        return smPolicy;
    }

    private static void populateGenericPolicy(PrePostScriptParameters prePostScriptParameters,
            CommonBackupCloneVerificationPolicyParameters commonBackupCloneVerificationPolicyParameters,
             SmPolicy smPolicy) {

        CLIUtils.writeToLog(String.format("Setting script params"), logger, LogLevel.DEBUG);
        smPolicy.setPreScriptPath(prePostScriptParameters.getPreScriptPath());
        smPolicy.setPreScriptArguments(prePostScriptParameters.getPreScriptArguments());
        smPolicy.setPostScriptPath(prePostScriptParameters.getPostScriptPath());
        smPolicy.setPostScriptArguments(prePostScriptParameters.getPostScriptArguments());
        smPolicy.setScriptTimeOut(prePostScriptParameters.getScriptTimeout());

        List<ScheduleType> schedules = commonBackupCloneVerificationPolicyParameters.getScheduleTypeAsListOfScheduleType();

        if (schedules != null) {
            CLIUtils.writeToLog(String.format("Setting schedule params"),
                    logger, LogLevel.DEBUG);
            if (smPolicy.getPolicyType().equals(SmPolicyType.BACKUP)) {

                if (smPolicy.getSchedules() == null) {
                    ArrayOfSmSchedule arrayOfSmSchedule = new ArrayOfSmSchedule();
                    smPolicy.setSchedules(arrayOfSmSchedule);
                }

                for (ScheduleType schedule : schedules) {

                    SmSchedule smSchedule = new SmSchedule();
                    // by default its a window scheduler
                    /*
                     * Below two lines are commented, since server it setting it
                     * to windows scheduler
                     */
                    // smSchedule.setLocalScheduler(false);
                    // smSchedule.setAppType(false);
                    SmSchedulerType smSchedulerType = SmSchedulerType
                            .fromValue(schedule.value());
                    smSchedule.setSchedulerType(smSchedulerType);
                    smPolicy.getSchedules().getSmSchedule().add(smSchedule);
                }

            } 
            /*else {
                SmSchedule smSchedule = new SmSchedule();
                // TODO: Until Server side changes are done, Verification policy
                // takes only one Schedule Type
                if (schedules.get(0).toString() != null) {

                    SmSchedulerType smSchedulerType = SmSchedulerType
                            .valueOf(schedules.get(0).toString());

                    smSchedule.setSchedulerType(smSchedulerType);
                    addPolicyTriggers(
                            commonBackupCloneVerificationPolicyParameters
                                    .getDaysInterval(),
                            commonBackupCloneVerificationPolicyParameters
                                    .getDaysOfTheWeek(),
                            commonBackupCloneVerificationPolicyParameters
                                    .getDaysOfTheMonth(),
                            commonBackupCloneVerificationPolicyParameters
                                    .getMonthsOfTheYear(),
                            commonBackupCloneVerificationPolicyParameters
                                    .getRepeatTaskEveryHour(),
                            commonBackupCloneVerificationPolicyParameters
                                    .getOccursEveryMinute(),
                            commonBackupCloneVerificationPolicyParameters
                                    .getOccursEveryMonth(), smSchedule);

                    if (commonBackupCloneVerificationPolicyParameters
                            .getStartTime() != null) {
                        XMLGregorianCalendar startTime = CLIUtils
                                .getXMLGregorianCalendarFromDate(commonBackupCloneVerificationPolicyParameters
                                        .getStartTime());
                        smSchedule.setStartTime(startTime);
                    }

                    if (commonBackupCloneVerificationPolicyParameters
                            .getEndTime() != null) {
                        XMLGregorianCalendar endTime = CLIUtils
                                .getXMLGregorianCalendarFromDate(commonBackupCloneVerificationPolicyParameters
                                        .getEndTime());
                        smSchedule.setEndTime(endTime);
                    }
                    smPolicy.setSchedule(smSchedule);
                }

            }*/
        }

    }

    public static void addPolicyTriggers(int daysinterval,
            List<DaysOfTheWeek> daysOfTheWeek, List<Integer> daysoftheMonth,
            List<MonthsOfTheYear> monthofTheYear, int repeatTaskEveryHour, int repeatTaskEveryMins, int occursEveryMinute, int occursEveryMonth,
            SmSchedule smSchedule) {

        CLIUtils.writeToLog(String.format("Adding policy triggers"), logger,
                LogLevel.DEBUG);

        switch (smSchedule.getSchedulerType()) {
            case ONE_TIME:
                break;

            case HOURLY:
                smSchedule.setRepeatTaskEveryHour(repeatTaskEveryHour);
                smSchedule.setRepeatTaskEveryMins(repeatTaskEveryMins);
                break;

            case DAILY:
                if (daysinterval != -1) {
                    smSchedule.setDaysInterval((short) daysinterval);
                }
                break;

            case WEEKLY:
                if (daysOfTheWeek != null) {
                    String daysOfTheWeekToString = StringUtils
                            .substringBetween(daysOfTheWeek.toString(), "[",
                                    "]");
                    smSchedule.setDaysOfTheWeek(daysOfTheWeekToString);
                 }
                break;

            case MONTHLY:
                if (daysoftheMonth != null) {
                    String daysoftheMonthToString = StringUtils
                            .substringBetween(daysoftheMonth.toString(), "[",
                                    "]");
                    smSchedule.setDaysoftheMonth(daysoftheMonthToString);

                    if (monthofTheYear != null) {
                        String monthofTheYearToString = StringUtils
                                .substringBetween(monthofTheYear.toString(), "[",
                                        "]");
                        smSchedule.setMonthsofTheYear(monthofTheYearToString);
                    } else {
                        smSchedule.setMonthlyFrequency(occursEveryMonth);
                    }
                }
                break;

            case NONE:

            default:
                break;
        }
        CLIUtils.writeToLog(String.format("Setting SmSchedule params to %s ", ToStringBuilder.reflectionToString(smSchedule) ), logger, LogLevel.DEBUG);

    }

    private static void addAppSpecificBackupPolicy(PluginCode appPolicyType,
            AbstractSMSCommand smPolicyCommand, SmPolicy smPolicy) {

        ArrayOfSmAppPolicy arrayOfSmAppPolicy = new ArrayOfSmAppPolicy();

        switch (appPolicyType) {
            case SCO:
                SmOraclePolicy smOraclePolicy = null;
                // TODO: Needs redesign
                if (smPolicyCommand instanceof AddSmPolicyCommand) {
                    smOraclePolicy = createSmOraclePolicy(((AddSmPolicyCommand) smPolicyCommand)
                            .getSmoBackupPolicyParameters());
                } else if (smPolicyCommand instanceof ModifySmPolicyCommand) {
                    smOraclePolicy = createSmOraclePolicy(((ModifySmPolicyCommand) smPolicyCommand)
                            .getSmoBackupPolicyParameters());
                }
                arrayOfSmAppPolicy.getSmAppPolicy().add(smOraclePolicy);
                break;

            default:
                break;
        }

        smPolicy.setAppPolicies(arrayOfSmAppPolicy);
    }

    private static SmOraclePolicy createSmOraclePolicy(
            SMOBackupPolicyParameters smoBackupPolicyParameters) {
        SmOraclePolicy smOraclePolicy = new SmOraclePolicy();

        SmOracleBackupScope smOracleBackupScope = SmOracleBackupScope
                .valueOf(smoBackupPolicyParameters.getOracleBackupScope()
                        .name());
        SmOracleBackupType smOracleBackupType = SmOracleBackupType
                .valueOf(smoBackupPolicyParameters.getOracleBackupType().name());

        smOraclePolicy.setSmOracleBackupScope(smOracleBackupScope);
        smOraclePolicy.setSmOracleBackupType(smOracleBackupType);
        // Negating to match the logic
        smOraclePolicy.setSmOraclePDBSaveState(!smoBackupPolicyParameters
                .isOracleSkipPDBSaveState());

        // Verification related parameters
        smOraclePolicy.setVerificationAfterBackup(smoBackupPolicyParameters
                .isVerificationAfterBackup());
        SmVerificationPolicy smVerificationPolicy = new SmVerificationPolicy();
        smVerificationPolicy.setName(smoBackupPolicyParameters
                .getVerificationPolicy());
        smOraclePolicy.setVerifyPolicy(smVerificationPolicy);

        // Cataloging related parameters
        smOraclePolicy.setCatalogBackup(smoBackupPolicyParameters.isCatalogBackupWithOracleRMAN());

        // Archive Log Pruning Parameters
        smOraclePolicy.setPruneArchiveLog(smoBackupPolicyParameters
                .isOraclePruneArchiveLog());
        if (smoBackupPolicyParameters.getOraclePruneArchiveLogType() != null) {
            smOraclePolicy.setPruneArchiveLogType(SmoPruneArchiveLogType
                    .fromValue(smoBackupPolicyParameters
                            .getOraclePruneArchiveLogType().name()));
        }
        smOraclePolicy
                .setPruneArchiveLogOlderThanDays(smoBackupPolicyParameters
                        .getOraclePruneArchiveLogOlderThanDays());
        if (smoBackupPolicyParameters.getOraclePruneArchiveLogDestinationType() != null) {
            smOraclePolicy
                    .setPruneArchiveLogDestinationType(SmoPruneArchiveLogDestinationType
                            .fromValue(smoBackupPolicyParameters
                                    .getOraclePruneArchiveLogDestinationType()
                                    .name()));
        }

        // Archive Log Backup Retention Parameters
        smOraclePolicy.setDeleteArchiveLogBackup(smoBackupPolicyParameters
                .isOracleDeleteArchiveLogBackup());
        if (smoBackupPolicyParameters.getOracleArchiveLogBackupRetentionType() != null) {
            smOraclePolicy
                    .setArchiveLogBackupRetentionType(SmoArchiveLogBackupRetentionType
                            .fromValue(smoBackupPolicyParameters
                                    .getOracleArchiveLogBackupRetentionType()
                                    .name()));
        }
        smOraclePolicy
                .setDeleteArchiveLogBackupCounts(smoBackupPolicyParameters
                        .getOracleDeleteArchiveLogBackupCounts());
        smOraclePolicy.setDeleteArchiveLogBackupDays(smoBackupPolicyParameters
                .getOracleDeleteArchiveLogBackupDays());

        return smOraclePolicy;
    }
}
