/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/output/formatter/GetSmPolicyCommandOutputFormatter.java#1 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command.output.formatter;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.*;
import com.netapp.snapcenter.cli.contracts.PluginCode;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.*;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

import org.apache.commons.lang.StringUtils;

import com.netapp.snapcenter.cli.utils.CLIUtils;



public class GetSmPolicyCommandOutputFormatter implements
        CommandOutputFormatter {


    private PluginCode pluginCode;

    public GetSmPolicyCommandOutputFormatter(PluginCode pluginCode) {
        this.pluginCode = pluginCode;
    }

    @Override
    public <T> void formatAndPrintCommandResponseObject(T smResponse,
                                                        FormatterType formatterType) throws CLIException {

        SmGetPolicyResponse smGetPolicyResponse = (SmGetPolicyResponse) smResponse;
        ArrayOfSmPolicy arrayOfSmPolicy = smGetPolicyResponse.getPolicyInfo();
        if (arrayOfSmPolicy == null
                || CLIUtils.isNullOrEmptyCollection(arrayOfSmPolicy
                .getSmPolicy())) {
            return;
        }

        switch (formatterType) {
            case TEXT:
                printCommandResponseInTextFormat(smGetPolicyResponse);
                break;
            case JSON:
            case XML:
                Formatter formatter = FormatterFactory.getFormatter(formatterType);
                formatter.writeFormattedObject(smGetPolicyResponse);
                break;
            case TEXT_TABLE:
            default:
                printCommandResponseInTableFormat(smGetPolicyResponse);
                break;

        }
    }

    private void printCommandResponseInTableFormat(
            SmGetPolicyResponse smGetPolicyResponse) {
        String headerPolicyName = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_TABLE_POLICY_NAME_HEADER);
        String headerPolicyType = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_TABLE_POLICY_TYPE_HEADER);
        String headerPolicySchedulerType = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_TABLE_POLICY_SCHEDULETYPE_HEADER);
        String headerPolicyDescription = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_TABLE_POLICY_DESCRIPTION_HEADER);

        String[] policyTableHeaders = { headerPolicyName, headerPolicyType,
                headerPolicySchedulerType, headerPolicyDescription };
        TextTableInterface policyTableFormatter = new TextTable(
                policyTableHeaders);

        for (SmPolicy smPolicy : smGetPolicyResponse.getPolicyInfo()
                .getSmPolicy()) {

            if ((smPolicy.getPluginPolicyType() != null) && (smPolicy.getPluginPolicyType().equals(
                    SmPluginCode.fromValue(pluginCode.toString()))
                    && (smPolicy.getPolicyType().equals(SmPolicyType.BACKUP)))) {
                String columnPolicyName = CLIUtils.getStringValue(smPolicy
                        .getName());
                String columnPolicyType = CLIUtils.getEnumName(smPolicy
                        .getPolicyType());

                StringBuilder stringBuilder = new StringBuilder();

                if (smPolicy.getSchedules() != null) {
                    for (SmSchedule scheduleType : smPolicy.getSchedules()
                            .getSmSchedule()) {
                        if (StringUtils.isNotBlank(scheduleType
                                .getSchedulerType().toString())) {
                            stringBuilder.append(
                                    scheduleType.getSchedulerType())
                                    .append(",");
                        }
                    }
                }

                // Delete the leading extra comma
                if (StringUtils.isNotBlank(stringBuilder.toString())) {
                    stringBuilder.deleteCharAt(stringBuilder.length() - 1);
                }

                String columnPolicySchedulerType = stringBuilder.toString();
                String columnPolicyDescription = CLIUtils
                        .getStringValue(smPolicy.getDescription());
                policyTableFormatter.addRow(columnPolicyName, columnPolicyType,
                        columnPolicySchedulerType, columnPolicyDescription);

            }
        }

        policyTableFormatter.displayFormattedTable();

    }

    private void printCommandResponseInTextFormat(
            SmGetPolicyResponse smGetPolicyResponse) {

        StringBuilder commandOutput = new StringBuilder();

        for (SmPolicy smPolicy : smGetPolicyResponse.getPolicyInfo()
                .getSmPolicy()) {

            commandOutput = new StringBuilder();
            commandOutput.append(printGenericPolicyInformation(smPolicy));


            SmPolicyType smPolicyType = smPolicy.getPolicyType();
            switch(smPolicyType) {
                case BACKUP:
                    commandOutput.append(printBackupPolicyInformation(smPolicy));
                    break;
                // Verification policy is removed in 2.0         
               /* case VERIFICATION:
                    commandOutput.append(printVerificationPolicyInformation(smPolicy));
                    break;*/
                default :
            }
            commandOutput.append(printPrePostScriptsInformation(smPolicy));
            commandOutput.append(printSchedulePolicyInformation(smPolicy));
            if(StringUtils.isNotBlank(commandOutput.toString())){
                CLIUtils.writeToConsole(commandOutput.toString());
            }


        }
    }

    private StringBuilder printPrePostScriptsInformation(SmPolicy smPolicy) {

        StringBuilder prePostScriptsOutput = new StringBuilder();

        //Pre-scripts
        prePostScriptsOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_PRESCRIPTFULLPATH, smPolicy.getPreScriptPath()));
        prePostScriptsOutput.append(CLIUtils.NEW_LINE);
        prePostScriptsOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_PRESCRIPTARGUMENTS, smPolicy.getPreScriptArguments()));
        prePostScriptsOutput.append(CLIUtils.NEW_LINE);

        //Post-scripts
        prePostScriptsOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_POSTSCRIPTFULLPATH, smPolicy.getPostScriptPath()));
        prePostScriptsOutput.append(CLIUtils.NEW_LINE);
        prePostScriptsOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_POSTSCRIPTARGUMENTS, smPolicy.getPostScriptArguments()));
        prePostScriptsOutput.append(CLIUtils.NEW_LINE);

        return prePostScriptsOutput;
    }


    private StringBuilder printVerificationPolicyInformation(SmPolicy smPolicy) {

        /*
        *   Information specific to a verification policy will be printed in this function
        */
        StringBuilder verificationPolicyParametersOutput = new StringBuilder();

        SmVerificationPolicy smVerificationPolicy = (SmVerificationPolicy) smPolicy;
        verificationPolicyParametersOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_NUMBEROFBACKUPSTOVERIFY, smVerificationPolicy.getNumBackups().toString()));
        verificationPolicyParametersOutput.append(CLIUtils.NEW_LINE);
        verificationPolicyParametersOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_VERIFYONSECONDARYSTORAGE, (smVerificationPolicy.isVerifyOnSnapMirrorDestination() || smVerificationPolicy.isVerifyOnSnapVaultDestination())));
        verificationPolicyParametersOutput.append(CLIUtils.NEW_LINE);

        return verificationPolicyParametersOutput;
    }



    private StringBuilder printBackupPolicyInformation(SmPolicy smPolicy) {

        StringBuilder backupPolicyParametersOutput = new StringBuilder();

        SmPluginCode pluginCode = smPolicy.getPluginPolicyType();
        switch(pluginCode) {
            case SCO : backupPolicyParametersOutput.append(printSCOSpecificPolicyInformation(smPolicy));
                break;
            case SCU : backupPolicyParametersOutput.append(printSCUSpecificPolicyInformation(smPolicy));
                break;
            default :
        }

        SmBackupPolicy smBackupPolicy = (SmBackupPolicy) smPolicy;
        backupPolicyParametersOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_SNAPMIRRORUPDATEENABLED, smBackupPolicy.isApplySnapMirrorUpdate()));
        backupPolicyParametersOutput.append(CLIUtils.NEW_LINE);

        if (smBackupPolicy.isApplySnapMirrorUpdate() || smBackupPolicy.isApplySnapvaultUpdate()) {
            backupPolicyParametersOutput.append(MessageBundle.localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_SNAPMIRRORUPDATEERRORRETRYCOUNT, CLIUtils.getStringValue(smBackupPolicy.getMirrorVaultUpdateRetryCount().toString())));
            backupPolicyParametersOutput.append(CLIUtils.NEW_LINE);
        }
        backupPolicyParametersOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_SNAPVAULTUPDATEENABLED, smBackupPolicy.isApplySnapvaultUpdate()));
        backupPolicyParametersOutput.append(CLIUtils.NEW_LINE);
        if (smBackupPolicy.isApplySnapMirrorUpdate() || smBackupPolicy.isApplySnapvaultUpdate()) {
            backupPolicyParametersOutput.append(MessageBundle.localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_SNAPMIRRORUPDATEERRORRETRYCOUNT, CLIUtils.getStringValue(smBackupPolicy.getMirrorVaultUpdateRetryCount().toString())));
            backupPolicyParametersOutput.append(CLIUtils.NEW_LINE);
        }
        if(smBackupPolicy.isApplySnapvaultUpdate()) {
            backupPolicyParametersOutput.append(MessageBundle.localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_SNAPVAULTUPDATELABEL, CLIUtils.getStringValue(smBackupPolicy.getSnapVaultLabel())));
            backupPolicyParametersOutput.append(CLIUtils.NEW_LINE);
        }

        //Check for the retention isRetentionEnabled flag - If true then, days and counts, otherwise it will be the retention of the storage system
        if(smBackupPolicy.isApplyRetention()) {
            backupPolicyParametersOutput.append(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_ISRETENTIONSET, "true"));
            backupPolicyParametersOutput.append(CLIUtils.NEW_LINE);
            if(smBackupPolicy.getRetentionDays() != null && !smBackupPolicy.getRetentionDays().toString().isEmpty()){
                backupPolicyParametersOutput.append(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_RETENTIONDAYS, CLIUtils.getStringValue(smBackupPolicy.getRetentionDays().toString())));
                backupPolicyParametersOutput.append(CLIUtils.NEW_LINE);
            }
        } else{
            backupPolicyParametersOutput.append(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_ISRETENTIONSET, MessageBundle.localizeMessage(CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_DEFAULTRETENTIONOFSTORAGEAPPLICABLE)));
            backupPolicyParametersOutput.append(CLIUtils.NEW_LINE);

        }


        backupPolicyParametersOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_RETENTIONCOUNT, smBackupPolicy.getRetentionCount()));
        backupPolicyParametersOutput.append(CLIUtils.NEW_LINE);
        return backupPolicyParametersOutput;

    }




    private StringBuilder printSchedulePolicyInformation(SmPolicy smPolicy) {

        StringBuilder scheduleOutput = new StringBuilder();

        if(smPolicy.getSchedule().getSchedulerType() != null) {
            SmSchedulerType scheduleType = smPolicy.getSchedule().getSchedulerType();
            scheduleOutput.append(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_SCHEDULETYPE, scheduleType.value()));
            scheduleOutput.append(CLIUtils.NEW_LINE);
            switch(scheduleType) {

                case ONE_TIME :
                    scheduleOutput.append(printScheduleStartTimeAndEndTime(smPolicy));
                    break;

                case HOURLY :
                    scheduleOutput.append(printScheduleStartTimeAndEndTime(smPolicy));
                    scheduleOutput.append(MessageBundle.localizeMessage(
                            CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_HOURLYREPEATINTERVAL, smPolicy.getSchedule().getRepeatTaskEveryHour()));
                    scheduleOutput.append(CLIUtils.NEW_LINE);
                    break;

                case DAILY:
                    scheduleOutput.append(printScheduleStartTimeAndEndTime(smPolicy));
                    scheduleOutput.append(MessageBundle.localizeMessage(
                            CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_DAILYREPEATINTERVAL, smPolicy.getSchedule().getDaysInterval()));
                    scheduleOutput.append(CLIUtils.NEW_LINE);
                    break;

                case MONTHLY:

                    scheduleOutput.append(printScheduleStartTimeAndEndTime(smPolicy));
                    if(smPolicy.getSchedule().getMonthsofTheYear() != null) {
                        scheduleOutput.append(MessageBundle.localizeMessage(
                                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_REPEATMONTHNAMES, smPolicy.getSchedule().getMonthsofTheYear()));
                        scheduleOutput.append(CLIUtils.NEW_LINE);
                        scheduleOutput.append((MessageBundle.localizeMessage(
                                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_DAYSOFTHEMONTH, smPolicy.getSchedule().getDaysoftheMonth())));
                        scheduleOutput.append(CLIUtils.NEW_LINE);

                    } else {
                        scheduleOutput.append(MessageBundle.localizeMessage(
                                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_OCCURSMONTHLYINTERVAL, smPolicy.getSchedule().getMonthlyFrequency()));
                        scheduleOutput.append(CLIUtils.NEW_LINE);
                        scheduleOutput.append((MessageBundle.localizeMessage(
                                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_DAYSOFTHEMONTH, smPolicy.getSchedule().getDaysoftheMonth())));
                        scheduleOutput.append(CLIUtils.NEW_LINE);
                    }
                    break;

                case WEEKLY:

                    scheduleOutput.append(printScheduleStartTimeAndEndTime(smPolicy));
                    scheduleOutput.append(MessageBundle.localizeMessage(
                            CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_DAYSOFTHEWEEK, smPolicy.getSchedule().getDaysOfTheWeek()));
                    scheduleOutput.append(CLIUtils.NEW_LINE);
                    break;
                default :

            }
        }
        scheduleOutput.append(CLIUtils.NEW_LINE);
        return scheduleOutput;
    }



    private StringBuilder printScheduleStartTimeAndEndTime(SmPolicy smPolicy) {

        StringBuilder scheduleStartAndEndTime = new StringBuilder();

        scheduleStartAndEndTime.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_SCHEDULESTARTTIME, CLIUtils.localizeDateAndGetFormattedString(smPolicy.getSchedule().getStartTime())));

        scheduleStartAndEndTime.append(CLIUtils.NEW_LINE);
        if (smPolicy.getSchedule().getEndTime() != null) {
            scheduleStartAndEndTime.append(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_SCHEDULEENDTIME, CLIUtils.localizeDateAndGetFormattedString(smPolicy.getSchedule().getEndTime())));
            scheduleStartAndEndTime.append(CLIUtils.NEW_LINE);
        }

        return scheduleStartAndEndTime;
    }

    private StringBuilder printSCUSpecificPolicyInformation(SmPolicy smPolicy) {
        /*
        * This function will have print the information of a SCU policy
        */
        StringBuilder scuSpecificPolicyInformationOutput = new StringBuilder();
        scuSpecificPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_PLUGINNAME, smPolicy.getPluginPolicyType()));
        scuSpecificPolicyInformationOutput.append(CLIUtils.NEW_LINE);

        return scuSpecificPolicyInformationOutput;
    }

    private StringBuilder printSCOSpecificPolicyInformation(SmPolicy smPolicy) {

        /*
        * This function will have print the information of a SCO policy
        */
        StringBuilder scoSpecificPolicyInformationOutput = new StringBuilder();

        SmOraclePolicy smOraclePolicy = (SmOraclePolicy) smPolicy.getAppPolicies().getSmAppPolicy().get(0);


        scoSpecificPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_ORACLEBACKUPSCOPE, CLIUtils.getEnumName(smOraclePolicy.getSmOracleBackupScope())));
        scoSpecificPolicyInformationOutput.append(CLIUtils.NEW_LINE);
        scoSpecificPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_ORACLEBACKUPTYPE, CLIUtils.getEnumName(smOraclePolicy.getSmOracleBackupType())));
        scoSpecificPolicyInformationOutput.append(CLIUtils.NEW_LINE);
        scoSpecificPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_ISPDBSAVESTATEENABLED, smOraclePolicy.isSmOraclePDBSaveState()));
        scoSpecificPolicyInformationOutput.append(CLIUtils.NEW_LINE);

        //Pruning related information
        scoSpecificPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_ISPRUNEARCHIVELOGENABLED, smOraclePolicy.isPruneArchiveLog()));
        scoSpecificPolicyInformationOutput.append(CLIUtils.NEW_LINE);

        if(smOraclePolicy.isPruneArchiveLog()) {
            scoSpecificPolicyInformationOutput.append(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_PRUNEARCHIVELOGDESTINATIONTYPE, CLIUtils.getEnumName(smOraclePolicy.getPruneArchiveLogDestinationType())));
            scoSpecificPolicyInformationOutput.append(CLIUtils.NEW_LINE);
            scoSpecificPolicyInformationOutput.append(MessageBundle.localizeMessage(
                    CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_PRUNEARCHIVELOGTYPE, CLIUtils.getEnumName(smOraclePolicy.getPruneArchiveLogType())));
            scoSpecificPolicyInformationOutput.append(CLIUtils.NEW_LINE);
            if(CLIUtils.getEnumName(smOraclePolicy.getPruneArchiveLogType()).equals("OLDER_LOGS")) {
                scoSpecificPolicyInformationOutput.append(MessageBundle.localizeMessage(
                        CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_PRUNEARCHIVELOGSOLDERTHANDAYS, smOraclePolicy.getPruneArchiveLogOlderThanDays().toString()));
                scoSpecificPolicyInformationOutput.append(CLIUtils.NEW_LINE);
            }
        }

        return scoSpecificPolicyInformationOutput;

    }



    private StringBuilder printGenericPolicyInformation(
            SmPolicy smPolicy) {

        StringBuilder genericPolicyInformationOutput = new StringBuilder();

        genericPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_POLICYNAME, CLIUtils.getStringValue(smPolicy.getName())));
        genericPolicyInformationOutput.append(CLIUtils.NEW_LINE);
        genericPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_POLICYDESCRIPTION, CLIUtils.getStringValue(smPolicy.getDescription())));
        genericPolicyInformationOutput.append(CLIUtils.NEW_LINE);

        genericPolicyInformationOutput.append(printPolicyPluginCode(smPolicy));

        genericPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_POLICYTYPE, smPolicy.getPolicyType()));
        genericPolicyInformationOutput.append(CLIUtils.NEW_LINE);
        genericPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_POLICYDATECREATED, CLIUtils.localizeDateAndGetFormattedString(smPolicy.getDateCreated())));
        genericPolicyInformationOutput.append(CLIUtils.NEW_LINE);
        genericPolicyInformationOutput.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_POLICYDATEMODIFIED, CLIUtils.localizeDateAndGetFormattedString(smPolicy.getDateModified())));
        genericPolicyInformationOutput.append(CLIUtils.NEW_LINE);

        return genericPolicyInformationOutput;

    }

    private StringBuilder printPolicyPluginCode(SmPolicy smPolicy) {

        StringBuilder policyPluginCode = new StringBuilder();

        policyPluginCode.append(MessageBundle.localizeMessage(
                CLIMsgKey.CLI_GETSMPOLICY_TEXTVIEW_PLUGINNAME, smPolicy.getPluginPolicyType()));
        policyPluginCode.append(CLIUtils.NEW_LINE);

        return policyPluginCode;

    }


}