/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/parameters/CommonBackupClonePolicyParameters.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.parameters;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.builder.ToStringBuilder;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToDateConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToIntegerListConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToSchedulesTypeConverter;
import com.netapp.snapcenter.cli.contracts.DaysOfTheWeek;
import com.netapp.snapcenter.cli.contracts.MonthsOfTheYear;
import com.netapp.snapcenter.cli.contracts.ScheduleType;

@Parameters(resourceBundle = "snapcenter-cli")
public class CommonBackupCloneVerificationPolicyParameters implements
        ParameterInterface {

    @Parameter(names = { "-ScheduleType" }, descriptionKey = "cli.smpolicy.command.scheduleType.option.description", listConverter = CustomStringToSchedulesTypeConverter.class, splitter = CommaParameterSplitter.class)
    private List<String> scheduleFrequency = new ArrayList<String>();

    @Parameter(names = { "-VerificationScheduleType" }, descriptionKey = "cli.smpolicy.command.verification.scheduleType.option.description", listConverter = CustomStringToSchedulesTypeConverter.class, splitter = CommaParameterSplitter.class)
    private List<String> verificationScheduleFrequency = new ArrayList<String>();

    //TODO: Except scheduleFrequency all other parameters(member variables) has to be removed once Clone and Verification policies are implemented in server
    @Parameter(names = { "-StartDateTime" }, descriptionKey = "cli.smpolicy.command.startDateTime.option.description", converter = CustomStringToDateConverter.class)
    private Date startTime;

    @Parameter(names = { "-EndDateTime" }, descriptionKey = "cli.smpolicy.command.endDateTime.option.description", converter = CustomStringToDateConverter.class)
    private Date endTime;

    @Parameter(names = { "-RepeatTask_Every_Hour" }, descriptionKey = "cli.smpolicy.command.repeatTask_Every_Hour.option.description")
    private int repeatTaskEveryHour = 1;

    @Parameter(names = { "-Occurs_Every_Minute" }, descriptionKey = "cli.smpolicy.command.occursEveryMinute.option.description")
    private int occursEveryMinute;

    @Parameter(names = { "-Occurs_Every_Month" }, descriptionKey = "cli.smpolicy.command.occursEveryMonth.option.description")
    private int occursEveryMonth;

    @Parameter(names = { "-DaysInterval" }, descriptionKey = "cli.smpolicy.command.daysInterval.option.description")
    private int daysInterval = 1;

    @Parameter(names = { "-DaysOfTheWeek" }, descriptionKey = "cli.smpolicy.command.daysOfTheWeek.option.description", splitter = CommaParameterSplitter.class)
    private List<String> daysOfTheWeekAsStringList = new ArrayList<>();

    @Parameter(names = { "-MonthsOfTheYear" }, descriptionKey = "cli.smpolicy.command.monthsOfTheYear.option.description", splitter = CommaParameterSplitter.class)
    private List<String> monthsOfTheYearAsStringList = new ArrayList<>();

    @Parameter(names = { "-DaysOfTheMonth" }, descriptionKey = "cli.smpolicy.command.daysOfTheMonth.option.description", listConverter = CustomStringToIntegerListConverter.class, splitter = CommaParameterSplitter.class)

    private List<Integer> daysOfTheMonth = new ArrayList<>();

    private List<DaysOfTheWeek> daysOfTheWeek;

    private List<MonthsOfTheYear> monthsOfTheYear;

    private List<ScheduleType> listScheduleTypes = new ArrayList<ScheduleType>();

    private List<ScheduleType> listVerificationScheduleTypes = new ArrayList<ScheduleType>();

    public List<String> getScheduleType() {
        // In order to enable On-Demand schedules, need to populate NONE in
        // Schedules 
        scheduleFrequency.add(ScheduleType.NONE.toString());
        return scheduleFrequency;
    }

    public void setScheduleType(List<String> scheduleType) {
        this.scheduleFrequency = scheduleType;
    }

    public List<ScheduleType> getScheduleTypeAsListOfScheduleType() {
        return listScheduleTypes;
    }

    public void setScheduleTypeAsListOfScheduleType(
            List<ScheduleType> listOfScheduleTypes) {
        listScheduleTypes = listOfScheduleTypes;
    }

    public List<ScheduleType> getListVerificationScheduleTypes() {
        return listVerificationScheduleTypes;
    }

    public void setListVerificationScheduleTypes(
            List<ScheduleType> listVerificationScheduleTypes) {
        this.listVerificationScheduleTypes = listVerificationScheduleTypes;
    }

    public List<String> getVerificationScheduleFrequency() {
        return verificationScheduleFrequency;
    }

    public void setVerificationScheduleFrequency(
            List<String> verificationScheduleFrequency) {
        this.verificationScheduleFrequency = verificationScheduleFrequency;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    public void setRepeatTaskEveryHour(int repeatTaskEveryHour) {
        this.repeatTaskEveryHour = repeatTaskEveryHour;
    }

    public int getRepeatTaskEveryHour() {
        return repeatTaskEveryHour;
    }

    public int getOccursEveryMinute() {
        return occursEveryMinute;
    }

    public void setOccursEveryMinute(int occursEveryMinute) {
        this.occursEveryMinute = occursEveryMinute;
    }

    public int getOccursEveryMonth() {
        return occursEveryMonth;
    }

    public void setOccursEveryMonth(int occursEveryMonth) {
        this.occursEveryMonth = occursEveryMonth;
    }

    public int getDaysInterval() {
        return daysInterval;
    }

    public void setDaysInterval(int daysInterval) {
        this.daysInterval = daysInterval;
    }

    public List<DaysOfTheWeek> getDaysOfTheWeek() {
        return daysOfTheWeek;
    }

    public void setDaysOfTheWeek(List<DaysOfTheWeek> daysOfTheWeek) {
        this.daysOfTheWeek = daysOfTheWeek;
    }

    public List<MonthsOfTheYear> getMonthsOfTheYear() {
        return monthsOfTheYear;
    }

    public void setMonthsOfTheYear(List<MonthsOfTheYear> monthsOfTheYear) {
        this.monthsOfTheYear = monthsOfTheYear;
    }

    public List<Integer> getDaysOfTheMonth() {
        return daysOfTheMonth;
    }

    public void setDaysOfTheMonth(List<Integer> daysOfTheMonth) {
        this.daysOfTheMonth = daysOfTheMonth;
    }

    public List<String> getDaysOfTheWeekAsStringList() {
        return daysOfTheWeekAsStringList;
    }

    public void setDaysOfTheWeekAsStringList(
            List<String> daysOfTheWeekAsStringList) {
        this.daysOfTheWeekAsStringList = daysOfTheWeekAsStringList;
    }

    public List<String> getMonthsOfTheYearAsStringList() {
        return monthsOfTheYearAsStringList;
    }

    public void setMonthsOfTheYearAsStringList(
            List<String> monthsOfTheYearAsStringList) {
        this.monthsOfTheYearAsStringList = monthsOfTheYearAsStringList;
    }

    @Override
    public String toString() {
        return ToStringBuilder.reflectionToString(this);
    }

}
