/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/parameters/SMORestoreParameters.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.parameters;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToDateConverter;

@Parameters(resourceBundle = "snapcenter-cli")
public class SMORestoreParameters implements ParameterInterface {

    public static final String PARAMETER_ORACLE_FULL_RESTORE = "-OracleFullRestore";
    public static final String PARAMETER_ORACLE_TABLESPACES = "-OracleTablespaces";
    public static final String PARAMETER_ORACLE_PLUGGABLE_DATABASES = "-OraclePluggableDatabases";
    public static final String PARAMETER_OPEN_TABLESPACE_AFTER_RECOVERY = "-OracleOpenTablespaceAfterRecovery";
    public static final String PARAMETER_OPEN_PLUGGABLE_DATABASE_AFTER_RECOVERY = "-OracleOpenPluggableDatabaseAfterRecovery";


    @Parameter(names = { "-OracleControlFileRestore" }, descriptionKey = "cli.restoresmbackup.command.oracleControlFileRestore.option.description")
    private boolean oracleControlFileRestore;

    @Parameter(names = { "-RestoreOracleRedoLogFile" }, descriptionKey = "cli.restoresmbackup.command.oracleRedologFileRestore.option.description")
    private boolean oracleRedologFileRestore;

    @Parameter(names = { "-OracleFullRestore" }, descriptionKey = "cli.restoresmbackup.command.oracleFullRestore.option.description")
    private boolean oracleFullRestore;

    @Parameter(names = { "-OracleSkipRecovery" }, descriptionKey = "cli.restoresmbackup.command.oracleSkipRecovery.option.description")
    private boolean oracleSkipRecovery;

    @Parameter(names = { "-OracleUntilScn" }, descriptionKey = "cli.restoresmbackup.command.oracleUntilScn.option.description")
    private Long oracleUntilScn;

    @Parameter(names = { "-OracleUntilTime" }, descriptionKey = "cli.restoresmbackup.command.oracleUntilTime.option.description", converter = CustomStringToDateConverter.class)
    private Date oracleUntilTime;

    @Parameter(names = { "-OracleTablespaces" }, descriptionKey = "cli.restoresmbackup.command.oracleTablespaces.option.description", splitter = CommaParameterSplitter.class)
    private List<String> oracleTablespaces = new ArrayList<>();

    @Parameter(names = { "-OracleChangeState" }, descriptionKey = "cli.restoresmbackup.command.oracleChangeState.option.description")
    private boolean oracleChangeState;

    @Parameter(names = { "-OracleOpenDatabaseAfterRecovery" }, descriptionKey = "cli.restoresmbackup.command.oracleOpenDatabaseAfterRecovery.option.description")
    private boolean oracleOpenDatabaseAfterRecovery;

    @Parameter(names = { "-OracleOpenPluggableDatabasesAfterRecovery" }, descriptionKey = "cli.restoresmbackup.command.oracleOpenPluggableDatabasesAfterRecovery.option.description")
    private boolean oracleOpenPluggableDatabaseAfterRecovery;

    @Parameter(names = { "-OracleOpenTablespacesAfterRecovery" }, descriptionKey = "cli.restoresmbackup.command.oracleOpenTablespacesAfterRecovery.option.description")
    private boolean oracleOpenTablespaceAfterRecovery;

    @Parameter(names = { "-OraclePluggableDatabases" }, descriptionKey = "cli.restoresmbackup.command.oraclePluggableDatabases.option.description", splitter = CommaParameterSplitter.class)
    private List<String> oraclePluggableDatabases = new ArrayList<>();

    @Parameter(names = { "-AlternateArchiveLogPaths" }, descriptionKey = "cli.restoresmbackup.command.alternateArchiveLogPaths.option.description", splitter = CommaParameterSplitter.class)
    private List<String> alternateArchiveLogPaths = new ArrayList<>();

    @Parameter(names = { "-RestoreOnClusterHost" }, descriptionKey = "cli.restoresmbackup.command.restoreOnClusterHost.option.description")
    private String restoreOnClusterHost;


    public boolean isOracleControlFileRestore() {
        return oracleControlFileRestore;
    }

    public void setOracleControlFileRestore(boolean oracleControlFileRestore) {
        this.oracleControlFileRestore = oracleControlFileRestore;
    }

    public boolean isOracleRedologFileRestore() {
        return oracleRedologFileRestore;
    }

    public void setOracleRedologFileRestore(boolean oracleRedologFileRestore) {
        this.oracleRedologFileRestore = oracleRedologFileRestore;
    }

    public boolean isOracleFullRestore() {
        return oracleFullRestore;
    }

    public void setOracleFullRestore(boolean oracleFullRestore) {
        this.oracleFullRestore = oracleFullRestore;
    }

    public boolean isOracleSkipRecovery() {
        return oracleSkipRecovery;
    }

    public void setOracleSkipRecovery(boolean oracleSkipRecovery) {
        this.oracleSkipRecovery = oracleSkipRecovery;
    }

    public Long getOracleUntilScn() {
        return oracleUntilScn;
    }

    public void setOracleUntilScn(Long oracleUntilScn) {
        this.oracleUntilScn = oracleUntilScn;
    }

    public Date getOracleUntilTime() {
        return oracleUntilTime;
    }

    public void setOracleUntilTime(Date oracleUntilTime) {
        this.oracleUntilTime = oracleUntilTime;
    }

    public List<String> getOracleTablespaces() {
        return oracleTablespaces;
    }

    public void setOracleTablespaces(List<String> oracleTablespaces) {
        this.oracleTablespaces = oracleTablespaces;
    }

    public boolean isOracleChangeState() {
        return oracleChangeState;
    }

    public void setOracleChangeState(boolean oracleChangeState) {
        this.oracleChangeState = oracleChangeState;
    }

    public List<String> getOraclePluggableDatabases() {
        return oraclePluggableDatabases;
    }

    public void setOraclePluggableDatabases(
            List<String> oraclePluggableDatabases) {
        this.oraclePluggableDatabases = oraclePluggableDatabases;
    }

    public void setAlternateArchiveLogPaths(
            List<String> alternateArchiveLogPaths) {
        this.alternateArchiveLogPaths = alternateArchiveLogPaths;
    }

    public List<String> getAlternateArchiveLogPaths() {
        return alternateArchiveLogPaths;
    }

    public void setOracleOpenDatabaseAfterRecovery(
            boolean oracleOpenDatabaseAfterRecovery) {
        this.oracleOpenDatabaseAfterRecovery = oracleOpenDatabaseAfterRecovery;
    }

    public boolean isOracleOpenDatabaseAfterRecovery() {
        return oracleOpenDatabaseAfterRecovery;
    }

    public boolean isOracleOpenPluggableDatabaseAfterRecovery() {
        return oracleOpenPluggableDatabaseAfterRecovery;
    }

    public void setOracleOpenPluggableDatabaseAfterRecovery(boolean oracleOpenPluggableDatabaseAfterRecovery) {
        this.oracleOpenPluggableDatabaseAfterRecovery = oracleOpenPluggableDatabaseAfterRecovery;
    }

    public boolean isOracleOpenTablespaceAfterRecovery() {
        return oracleOpenTablespaceAfterRecovery;
    }

    public void setOracleOpenTablespaceAfterRecovery(boolean oracleOpenTablespaceAfterRecovery) {
        this.oracleOpenTablespaceAfterRecovery = oracleOpenTablespaceAfterRecovery;
    }

    public void setRestoreOnClusterHost(String restoreOnClusterHost) {
        this.restoreOnClusterHost = restoreOnClusterHost;
    }

    public String getRestoreOnClusterHost() {
        return restoreOnClusterHost;
    }
}
