/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/parser/CustomStringToPolicyRetentionSettingsMapConverter.java#1 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.IStringConverter;
import com.beust.jcommander.ParameterException;
import com.beust.jcommander.converters.BaseConverter;
import com.netapp.common.logging.MessageBundle;
import com.netapp.snapcenter.cli.contracts.RetentionSettingsArguments;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;

/**
 * This class combines and converts the retention settings provided as option
 * like -retentionsettings
 * {'scheduleType=Daily,backuptype=DATA,retentioncount=11,retentiondays=16'}
 * -retentionsettings
 * {'scheduleType=Daily,backuptype=LOG,retentioncount=11,retentiondays=16'}
 * -retentionsettings
 * {'scheduleType=weekly,backuptype=LOG,retentioncount=10,retentiondays=14'} to
 * list of map.
 */
public class CustomStringToPolicyRetentionSettingsMapConverter extends
        BaseConverter<Map<String, String>> implements
        IStringConverter<Map<String, String>> {

    public CustomStringToPolicyRetentionSettingsMapConverter(String optionName) {
        super(optionName);
    }

    @Override
    public Map<String, String> convert(String keyValueMapString) {

        Map<String, String> convertedMap = new HashMap<String, String>();
        if (StringUtils.isBlank(keyValueMapString)) {
            throw new ParameterException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_SMPOLICY_EMPTY_RETENTION_SETTINGS_VALUES));
        }

        String keyValueMapStringWithoutFlowerBraces = StringUtils
                .substringBetween(keyValueMapString, "{", "}");
        keyValueMapString = (keyValueMapStringWithoutFlowerBraces == null) ? keyValueMapString
                : keyValueMapStringWithoutFlowerBraces;

        String[] keyValuePairsArray = keyValueMapString.split(",");

        for (int i = 0; i < keyValuePairsArray.length; i++) {
            String keyValuePair = keyValuePairsArray[i];
            String[] keyValue = keyValuePair.split("=");

            if (keyValue.length != 2) {
                throw new ParameterException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_SMPOLICY_MISSING_RETENTION_SETTINGS_ARGUMENTS_VALUE));
            }
            /*
             * Keys like host, type, names all are in lower case and hence to
             * provide flexibility for user making it case insensitive by
             * converting to lower case. Change this if there is a key which
             * should have fixed case or camel case kind.
             */
            String name = keyValue[0].trim().toLowerCase();
            String val = keyValue[1].trim();
            // Check for duplicate arguments
            if (StringUtils.isNotEmpty(convertedMap.get(name))) {
                throw new ParameterException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_COMMAND_DUPLICATE_ARGUMENTS_PASSED_TO_COMMAND_OPTIONS,
                                        "RetentionSettings"));
            }
            convertedMap.put(name, val);
        }
        validateOptions(convertedMap);
        return convertedMap;
    }

    /*
     * Validates the RetentionSettings arguments for typo and any other than the
     * required arguments
     */
    @SuppressWarnings("unchecked")
    private static void validateOptions(Map<String, String> resource) {

        if (resource.keySet().size() < 2) {
            throw new ParameterException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_SMPOLICY_MISSING_RETENTION_SETTINGS_ARGUMENTS));
        }
        List mapToList = new ArrayList();
        mapToList.addAll(resource.keySet());
        try {
            CLIUtils.convertStringListToEnumList(
                    RetentionSettingsArguments.class, mapToList);
        } catch (CLIException e) {
            throw new ParameterException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_SMPOLICY_INVALID_RETENTION_SETTINGS_ARGUMENTS,resource.keySet()));
        }

    }
}
