/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/storage/preferences/GetSvmPreferredDataPathCommand.java#1 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.storage.preferences;

import java.util.Map.Entry;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.command.CommandInterface;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.formatter.TextTable;
import com.netapp.snapcenter.cli.formatter.TextTableInterface;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.getsvmpreferreddatapath.command.description")
public class GetSvmPreferredDataPathCommand extends
        AbstractStoragePreferencesCommand implements CommandInterface {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_GETSVMPREFERREDDATAPATH_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(GetSvmPreferredDataPathCommand.class);

    @Parameter(names = { "-SVM" }, descriptionKey = "cli.getsvmpreferreddatapath.command.svm.option.description")
    private String storageVM;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {

        initializeStoragePreferencesProperties();

        String headerSVM = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSVMPREFERREDDATAPATH_TABLE_SVM_HEADER);
        String headerDataPath = MessageBundle
                .localizeMessage(CLIMsgKey.CLI_GETSVMPREFERREDDATAPATH_TABLE_DATA_PATH_HEADER);

        String[] svmPreferredDataPathTableHeaders = { headerSVM, headerDataPath };
        TextTableInterface svmPreferredDataPathTable = new TextTable(
                svmPreferredDataPathTableHeaders);

        if (StringUtils.isNotBlank(getStorageVM())
                && !getStorageVM().equals(
                        PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY)) {
            if (getStoragePreferencesProperty(getStorageVM()) != null) {
                svmPreferredDataPathTable.addRow(getStorageVM(),
                        getStoragePreferencesProperty(getStorageVM()));
            }
        } else {
            for (Entry<Object, Object> e : getAllStoragePreferencesProperties()) {
                String svm = (String) e.getKey();
                String dataPath = (String) e.getValue();
                if (!svm.equals(PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY)) {
                    svmPreferredDataPathTable.addRow(svm, dataPath);
                }
            }
        }

        svmPreferredDataPathTable.displayFormattedTable();

        String executionSucess = MessageBundle.localizeMessage(
                CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS, this.getCommandName());
        CLIUtils.writeToLogAndConsole(executionSucess, logger, LogLevel.INFO);
    }

    public String getStorageVM() {
        return storageVM;
    }

    public void setStorageVM(String storageVM) {
        this.storageVM = storageVM;
    }

}