/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/storage/preferences/GetSvmPreferredDataPathCommand.java#1 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.storage.preferences;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.command.CommandInterface;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.removesvmpreferreddatapath.command.description")
public class RemoveSvmPreferredDataPathCommand extends AbstractStoragePreferencesCommand
        implements CommandInterface {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_REMOVESVMPREFERREDDATAPATH_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(RemoveSvmPreferredDataPathCommand.class);

    @Parameter(names = { "-SVM" }, descriptionKey = "cli.removesvmpreferreddatapath.command.svm.option.description", required = true)
    private String storageVM;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {

        initializeStoragePreferencesProperties();

        if (StringUtils.isBlank(getStorageVM())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_SVMPREFERREDDATAPATH_SVM_EMPTY_ERROR));
        }

        if (getStorageVM().equals(PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_SVMPREFERREDDATAPATH_SVM_VALUE_CANNOT_BE_PREFERRED_STORAGE_EXPORT_IP_ADDRESSES_KEY_NAME,
                                    PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY));
        }

        if (getStoragePreferencesProperty(getStorageVM()) == null) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_REMOVESVMPREFERREDDATAPATH_DATA_PATH_ENTRY_FOR_SVM_DOESNOT_EXISTS_ERROR,
                                    getStorageVM()));
        }

        CLIUtils.promptUserToContinue(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_REMOVESVMPREFERREDDATAPATH_PROMPT_USER_REMOVE_VALUES));

        deleteStoragePreferencesProperty(getStorageVM());

        String removeSvmDataPathSucess = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_REMOVESVMPREFERREDDATAPATH_DATA_PATH_ENTRY_FOR_SVM_REMOVE_SUCCESS,
                        getStorageVM());
        CLIUtils.writeToLogAndConsole(removeSvmDataPathSucess, logger,
                LogLevel.INFO);

        String executionSucess = MessageBundle.localizeMessage(
                CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS, this.getCommandName());
        CLIUtils.writeToLogAndConsole(executionSucess, logger, LogLevel.INFO);
    }

    public String getStorageVM() {
        return storageVM;
    }

    public void setStorageVM(String storageVM) {
        this.storageVM = storageVM;
    }

}