/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/storage/preferences/SetPreferredStorageExportIpsCommand.java#1 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.storage.preferences;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.command.CommandInterface;
import com.netapp.snapcenter.cli.command.parser.CommaParameterSplitter;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.setpreferredstorageexportips.command.description")
public class SetPreferredStorageExportIpsCommand extends AbstractStoragePreferencesCommand
        implements CommandInterface {
    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETPREFERREDSTORAGEEXPORTIPS_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(SetPreferredStorageExportIpsCommand.class);

    @Parameter(names = { "-IpAddresses" }, descriptionKey = "cli.setpreferredstorageexportips.command.ipAddresses.option.description", splitter = CommaParameterSplitter.class, required=true)
    private List<String> ipAddresses = new ArrayList<String>();

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {

        initializeStoragePreferencesProperties();

        if (StringUtils
                .isNotBlank(getStoragePreferencesProperty(PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY))) {
            CLIUtils.promptUserToContinue(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_SETPREFERREDSTORAGEEXPORTIPS_PROMPT_USER_OVERWRITE_VALUES));
        }
 
        String ipAddressesValue = StringUtils.substringBetween(getIpAddresses()
                .toString(), "[", "]");
        saveStoragePreferencesProperty(PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY,
                ipAddressesValue);

        String updatePrefereedStorageExportIpAdressSuccess = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_SETPREFERREDSTORAGEEXPORTIPS_COMMAND_IP_ADDRESSES_UPDATE_SUCCESS);
        CLIUtils.writeToLogAndConsole(updatePrefereedStorageExportIpAdressSuccess, logger,
                LogLevel.INFO);

        String executionSucess = MessageBundle.localizeMessage(
                CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS, this.getCommandName());
        CLIUtils.writeToLogAndConsole(executionSucess, logger, LogLevel.INFO);
    }

    public List<String> getIpAddresses() {
        return ipAddresses;
    }

    public void setIpAddresses(List<String> ipAddresses) {
        this.ipAddresses = ipAddresses;
    }
}