/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/storage/preferences/SetSvmPreferredDataPathCommand.java#1 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.storage.preferences;

import org.apache.commons.lang3.StringUtils;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.command.CommandInterface;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

@Parameters(resourceBundle = "snapcenter-cli", commandDescriptionKey = "cli.setsvmpreferreddatapath.command.description")
public class SetSvmPreferredDataPathCommand extends
        AbstractStoragePreferencesCommand implements CommandInterface {

    private static final String COMMANDNAME = MessageBundle
            .localizeMessage(CLIMsgKey.CLI_SETSVMPREFERREDDATAPATH_COMMAND_NAME);

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(SetSvmPreferredDataPathCommand.class);

    @Parameter(names = { "-SVM" }, descriptionKey = "cli.setsvmpreferreddatapath.command.svm.option.description", required = true)
    private String storageVM;

    @Parameter(names = { "-DataPath" }, descriptionKey = "cli.setsvmpreferreddatapath.command.dataPath.option.description", required = true)
    private String dataPath;

    @Override
    public String getCommandName() {
        return COMMANDNAME;
    }

    @Override
    public void run() throws CLIException {

        initializeStoragePreferencesProperties();

        validate();

        CLIUtils.writeToLogAndConsole(
                MessageBundle
                        .localizeMessage(
                                CLIMsgKey.CLI_SETSVMPREFERREDDATAPATH_SVM_IS_CONFIGURED_WITH_DATA_PATH,
                                getStorageVM(),
                                getStoragePreferencesProperty(getStorageVM())),
                logger, LogLevel.INFO);

        CLIUtils.promptUserToContinue(MessageBundle
                .localizeMessage(CLIMsgKey.CLI_SETSVMPREFERREDDATAPATH_PROMPT_USER_OVERWRITE_VALUES));

        saveStoragePreferencesProperty(getStorageVM(), getDataPath());

        String modifySvmDataPathSucess = MessageBundle
                .localizeMessage(
                        CLIMsgKey.CLI_SETSVMPREFERREDDATAPATH_DATA_PATH_ENTRY_FOR_SVM_MODIFY_SUCCESS,
                        getStorageVM());
        CLIUtils.writeToLogAndConsole(modifySvmDataPathSucess, logger,
                LogLevel.INFO);

        String executionSucess = MessageBundle.localizeMessage(
                CLIMsgKey.CLI_COMMAND_EXECUTION_SUCCESS, this.getCommandName());
        CLIUtils.writeToLogAndConsole(executionSucess, logger, LogLevel.INFO);
    }

    private void validate() throws CLIException {
        if (StringUtils.isBlank(getStorageVM())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_SVMPREFERREDDATAPATH_SVM_EMPTY_ERROR));
        }
        if (StringUtils.isBlank(getDataPath())) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_SVMPREFERREDDATAPATH_DATA_PATH_EMPTY_ERROR));
        }
        if (getStorageVM().equals(PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY)) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_SVMPREFERREDDATAPATH_SVM_VALUE_CANNOT_BE_PREFERRED_STORAGE_EXPORT_IP_ADDRESSES_KEY_NAME,
                                    PREFERRED_IPADDRESSES_FOR_STORAGE_EXPORT_KEY));
        }
        if (getStoragePreferencesProperty(getStorageVM()) == null) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(
                                    CLIMsgKey.CLI_SETSVMPREFERREDDATAPATH_DATA_PATH_ENTRY_FOR_SVM_DOESNOT_EXISTS_ERROR,
                                    getStorageVM(),
                                    MessageBundle
                                            .localizeMessage(CLIMsgKey.CLI_ADDSVMPREFERREDDATAPATH_COMMAND_NAME)));
        }
    }

    public String getStorageVM() {
        return storageVM;
    }

    public void setStorageVM(String storageVM) {
        this.storageVM = storageVM;
    }

    public String getDataPath() {
        return dataPath;
    }

    public void setDataPath(String dataPath) {
        this.dataPath = dataPath;
    }

}