/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/validation/CloneCommandParametersValidationUtil.java#1 $
 * 
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command.validation;

import java.io.File;
import java.io.IOException;

import org.apache.commons.lang3.StringUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.command.parameters.SMOCloneParameters;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

/**
 * This class contains clone command related arguments validation.
 *
 */
public class CloneCommandParametersValidationUtil {

    private CloneCommandParametersValidationUtil() {

    }

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(CloneCommandParametersValidationUtil.class);

    public static void validateAndSetSMOCloneParameters(
            SMOCloneParameters smoCloneParams) throws CLIException {

        // Validation for clone spec file
        validateAndSetCloneSpecFile(smoCloneParams);

        // Validation for recovery parameters
        if (!smoCloneParams.isOracleSkipRecovery()) {
            validateForRecoveryOptions(smoCloneParams);
            if (smoCloneParams.getOracleUntilScn() == null
                    && smoCloneParams.getOracleUntilTime() == null
                    && !smoCloneParams.isOracleUntilCancel()) {
                CLIUtils.writeToLogAndConsole(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_ORACLE_DEFAULT_RECOVERY_WILL_BE_PERFORMED),
                        logger, LogLevel.INFO);
                smoCloneParams.setOracleUntilCancel(true);
            }
        } else {
            if (smoCloneParams.getOracleUntilScn() != null
                    || smoCloneParams.getOracleUntilTime() != null
                    || smoCloneParams.isOracleUntilCancel()) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_ORACLE_INVALID_RECOVERY_OPTIONS));
            }
            CLIUtils.writeToLogAndConsole(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_ORACLE_SKIP_RECOVERY_OPTION_IS_CHOSEN),
                    logger, LogLevel.INFO);
        }

    }

    private static void validateAndSetCloneSpecFile(
            SMOCloneParameters smoCloneParams) throws CLIException {
        String oracleCloneSpecificationFile = smoCloneParams
                .getOracleCloneSpecificationFile();

        try {
            if (oracleCloneSpecificationFile == null) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_PARAMETER_NOT_SPECIFIED,
                                        SMOCloneParameters.ORACLE_CLONE_SPECIFICATION_FILE_OPTION));
            }

            if (StringUtils.isBlank(oracleCloneSpecificationFile)) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_CLONE_SPEC_FILE_PATH_CANNOT_BE_EMPTY));
            }

            File cloneSpecFile = new File(oracleCloneSpecificationFile);
            if (!cloneSpecFile.exists()) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(
                                        CLIMsgKey.CLI_NEWSMCLONE_CLONE_SPEC_FILE_DOESNOT_EXISTS,
                                        cloneSpecFile.getCanonicalPath()));
            }
            smoCloneParams.setOracleCloneSpecificationFile(cloneSpecFile
                    .getCanonicalPath());

        } catch (IOException e) {
            throw new CLIException(e.getLocalizedMessage());
        }

    }

    private static void validateForRecoveryOptions(
            SMOCloneParameters smoCloneParams) throws CLIException {

        if (smoCloneParams.getOracleUntilScn() != null
                && smoCloneParams.getOracleUntilTime() != null
                && !smoCloneParams.isOracleUntilCancel()) {
            throw new CLIException(
                    MessageBundle
                            .localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_ORACLE_NOT_A_VALID_CONFIGURATION_FOR_RECOVERY));
        }

        for (String logsPath : smoCloneParams.getAlternateArchiveLogPaths()) {
            if (StringUtils.isBlank(logsPath)) {
                throw new CLIException(
                        MessageBundle
                                .localizeMessage(CLIMsgKey.CLI_NEWSMCLONE_ORACLE_ALTERNATE_ARCHIVE_LOG_PATH_CANNOT_BE_EMPTY));
            }
        }

    }

}
