/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/validation/InvokeSmCloneResourceValidationUtil.java#1 $
 * 
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.command.validation;

import org.apache.commons.lang.StringUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.plugin.contracts.SmClone;
import com.netapp.smcore.plugin.contracts.SmGetCloneNameRequest;
import com.netapp.smcore.plugin.contracts.SmGetCloneNameResponse;
import com.netapp.smcore.plugin.contracts.SmGetCloneRequest;
import com.netapp.smcore.plugin.contracts.SmGetCloneResponse;
import com.netapp.smcore.plugin.contracts.SmObject;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.helper.SnapCenterServerServiceHelper;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

/**
 * 
 * This class validates the input of Invoke-SmResourceSplit options
 *
 */
public class InvokeSmCloneResourceValidationUtil {

    private SnapCenterServerServiceHelper snapCenterServerServiceHelper;

    public InvokeSmCloneResourceValidationUtil(
            SnapCenterServerServiceHelper serviceHelper) {
        this.setSnapCenterServerServiceHelper(serviceHelper);
    }

    public SmGetCloneNameResponse getCloneNameFromAppObjectId(String appObjectId) throws CLIException {

        SmGetCloneNameRequest smGetCloneNameRequest = new SmGetCloneNameRequest();
        SmObject smObject = new SmObject();
        smObject.setId(appObjectId);
        smGetCloneNameRequest.setAppObject(smObject);
        smGetCloneNameRequest.setDisplayAll(true);
        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(smGetCloneNameRequest,
                        SmGetCloneNameResponse.class,
                        SMSRestConfiguration.SMS_CLONE_GET_NAME_SERVICE_PATH);
        SmGetCloneNameResponse response = (SmGetCloneNameResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (!StringUtils.isBlank(response.getResult().getMessage())) {

            throw new CLIException(response.getResult().getMessage());
        }

        if (StringUtils.isBlank(response.getCloneName())) {
            String errorMsg = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_APP_OBJECT_ID_IS_NOT_A_CLONE_ERROR,
                            appObjectId);
            throw new CLIException(errorMsg);
        }
 
        return response;
    }

    public void validateCloneName(String cloneName) throws CLIException {

        SmGetCloneRequest smGetCloneRequest = new SmGetCloneRequest();
        SmClone smClone = new SmClone();
        smClone.setCloneName(cloneName);
        smGetCloneRequest.setClone(smClone);
        smGetCloneRequest.setDisplayAll(true);

        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(smGetCloneRequest,
                        SmGetCloneResponse.class,
                        SMSRestConfiguration.SMS_GET_CLONE_SERVICE_PATH);
        SmGetCloneResponse response = (SmGetCloneResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (!StringUtils.isBlank(response.getResult().getMessage())) {
            throw new CLIException(response.getResult().getMessage());
        }

        if ((response.getClones().getSmClone() != null)
                && (response.getClones().getSmClone().isEmpty())) {
            String errorMsg = MessageBundle
                    .localizeMessage(
                            CLIMsgKey.CLI_COMMAND_INVOKESMCLONERESOURCESPLIT_CLONE_NAME_IS_NOT_VALID_ERROR,
                            cloneName);
            throw new CLIException(errorMsg);
        }
    }

    public SnapCenterServerServiceHelper getSnapCenterServerServiceHelper() {
        return snapCenterServerServiceHelper;
    }

    public void setSnapCenterServerServiceHelper(
            SnapCenterServerServiceHelper snapCenterServerServiceHelper) {
        this.snapCenterServerServiceHelper = snapCenterServerServiceHelper;
    }
}
