/*

 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/validation/ResourceGroupScheduleParametersValidationUtil.java#1 $

 * 

 * Copyright (c) 2016 NetApp, Inc.

 * All rights reserved.

 */

package com.netapp.snapcenter.cli.command.validation;



import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.SmSchedulerType;
import com.netapp.snapcenter.cli.command.parameters.ResourceGroupScheduleParameters;
import com.netapp.snapcenter.cli.command.parser.CustomHHMMStringToHoursAndMinutesConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToDateConverter;
import com.netapp.snapcenter.cli.command.parser.CustomStringToSchedulesMapConverter;
import com.netapp.snapcenter.cli.contracts.DaysOfTheWeek;
import com.netapp.snapcenter.cli.contracts.MonthsOfTheYear;
import com.netapp.snapcenter.cli.contracts.ScheduleType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;



public class ResourceGroupScheduleParametersValidationUtil {



    private static ExtendedGenericLogger logger = LoggerFactory

            .getExtendedClassAwareLogger(ResourceGroupScheduleParametersValidationUtil.class);



    public static List<ResourceGroupScheduleParameters> processScheduleParameters(

            List<Map<String, String>> schedules,

            Map<String, List<SmSchedulerType>> policyVsScheduleTypeMap)

            throws CLIException {



        CLIUtils.writeToLog("Processing SchedulesParameters ", logger,

                LogLevel.DEBUG);

        List<ResourceGroupScheduleParameters> schedulesParamsList = new ArrayList<ResourceGroupScheduleParameters>();



        if(CLIUtils.isNotNullAndNotEmptyCollection(schedules)) {

            for (Map<String, String> schedule : schedules) {

                validateRepeatEveryHourAndScheduleType(schedule);

                ResourceGroupScheduleParameters param = new ResourceGroupScheduleParameters();

                for (Map.Entry<String, String> scheduleEntry : schedule.entrySet()) {

                    if (scheduleEntry.getKey().equalsIgnoreCase(

                            CustomStringToSchedulesMapConverter.POLICY_NAME)) {

                        if ((policyVsScheduleTypeMap.keySet()

                                .contains(scheduleEntry.getValue()))

                                && CLIUtils.isEmpty((policyVsScheduleTypeMap

                                .get(scheduleEntry.getValue())))) {

                            throw new CLIException(

                                    MessageBundle

                                            .localizeMessage(

                                                    CLIMsgKey.CLI_SCHEDULES_MAPPING_SCHEDULES_ARE_NOT_ALLOWED_BY_POLICY,

                                                    scheduleEntry.getValue()));

                        }

                        param.setPolicyName(scheduleEntry.getValue());

                    }



                    fillGenericScheduleParameters(scheduleEntry, param);

                }



                // Check for required options

                if ((StringUtils.isEmpty(param.getPolicyName()))

                        || (param.getStartTime() == null)) {

                    throw new CLIException(

                            MessageBundle

                                    .localizeMessage(CLIMsgKey.CLI_SCHEDULES_MAPPING_REQUIRED_ARGUMENTS));

                }



                // validate the parsed schedule

                PolicyCommandParametersValidationUtil

                        .processAndValidateCommonBackupCloneVerificationPolicyParameters(param);

                schedulesParamsList.add(param);

            }

        }

        return schedulesParamsList;

    }

    private static void validateRepeatEveryHourAndScheduleType(Map<String, String> schedule) throws CLIException {

        //Initializing the schedule type to none and using a flag to check if repeat every hour has been given
        boolean isRepeatEveryHourGiven = false;
        boolean isScheduleTypeNotHourly = false;

        /*Iterating through the map to check the following:
            1. If schedule type is given , then check if the schedule type is hourly.
            2. Also check if repeat every hour is given.
        If both are given, throw exception.*/

        for (Map.Entry<String,String> scheduleEntry : schedule.entrySet()){

            logger.debug("Schedule entry is " + scheduleEntry.getKey() + " and the value is " + scheduleEntry.getValue() );

            if((scheduleEntry.getKey().equalsIgnoreCase(CustomStringToSchedulesMapConverter.SCHEDULE_TYPE)) && (ScheduleType.valueOf(scheduleEntry.getValue().toUpperCase()) != ScheduleType.HOURLY)){
                isScheduleTypeNotHourly = true;
            }
            if(scheduleEntry.getKey().equalsIgnoreCase(CustomStringToSchedulesMapConverter.REPEATE_EVERY_HOUR)){
                isRepeatEveryHourGiven = true;
            }
            if(isScheduleTypeNotHourly && isRepeatEveryHourGiven) {
                throw new CLIException(MessageBundle.localizeMessage(CLIMsgKey.CLI_SMPOLICY_OCCURS_EVERY_MINUTE_INVALID));
            }
        }
    }



    public static void fillGenericScheduleParameters(

            Entry<String, String> scheduleEntry,

            ResourceGroupScheduleParameters resourceGroupScheduleParameters)

            throws CLIException {



        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.SCHEDULE_TYPE)) {

            if (ScheduleType.fromValue(scheduleEntry.getValue()).equals(

                    ScheduleType.NONE)) {

                throw new CLIException(

                        MessageBundle

                                .localizeMessage(CLIMsgKey.CLI_SCHEDULES_MAPPING_REQUIRED_SCHEDULE_TYPE));

            }

            resourceGroupScheduleParameters.setScheduleType(ScheduleType

                    .fromValue(scheduleEntry.getValue()));

        }

        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.START_DATE_TIME)) {

            CustomStringToDateConverter converter = new CustomStringToDateConverter(

                    CustomStringToSchedulesMapConverter.START_DATE_TIME);

            resourceGroupScheduleParameters.setStartTime(converter

                    .convert(scheduleEntry.getValue()));

        }

        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.END_DATE_TIME)) {

            CustomStringToDateConverter converter = new CustomStringToDateConverter(

                    CustomStringToSchedulesMapConverter.END_DATE_TIME);

            resourceGroupScheduleParameters.setEndTime(converter

                    .convert(scheduleEntry.getValue()));

        }

        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.REPEATE_EVERY_HOUR)) {

            //Here you need to put the validation to break the input which is in HH:MM to HH and MM and populate the parameters accordingly
            logger.debug("HH:MM format is " + scheduleEntry.getValue());
            logger.debug("Converting the HH:MM format to Hours and Minutes...");

            Date date = CustomHHMMStringToHoursAndMinutesConverter.convertToHHMMformat(scheduleEntry.getValue());
            int hours = date.getHours();
            int minutes = date.getMinutes();
            logger.debug("The hours is " + hours);
            logger.debug("The minutes is " + minutes);

            resourceGroupScheduleParameters.setRepeatTaskEveryMins(minutes);
            resourceGroupScheduleParameters.setRepeatTaskEveryHour(hours);

        }

        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.OCCURS_EVERY_MINUTE)) {

            resourceGroupScheduleParameters.setOccursEveryMinute(Integer

                    .parseInt(scheduleEntry.getValue()));

        }

        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.OCCURS_EVERY_MONTH)) {

            resourceGroupScheduleParameters.setOccursEveryMonth(Integer

                    .parseInt(scheduleEntry.getValue()));

        }

        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.DAYS_INTERVAL)) {

            resourceGroupScheduleParameters.setDaysInterval(Short

                    .valueOf(scheduleEntry.getValue()));

        }

        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.DAYS_OF_THE_WEEK)) {

            String[] daysOfWeek = scheduleEntry

                    .getValue()

                    .split(CustomStringToSchedulesMapConverter.SCHEDULE_OPTIONS_LIST_DELIMETER);

            List<DaysOfTheWeek> daysOfWeekList = new ArrayList<DaysOfTheWeek>();

            for (String day : daysOfWeek) {

                daysOfWeekList.add(DaysOfTheWeek.fromValue(day.trim()));

            }

            resourceGroupScheduleParameters.setDaysOfTheWeek(daysOfWeekList);

        }

        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.MONTHS_OF_THE_YEAR)) {

            String[] monthsOfYear = scheduleEntry

                    .getValue()

                    .split(CustomStringToSchedulesMapConverter.SCHEDULE_OPTIONS_LIST_DELIMETER);

            List<MonthsOfTheYear> monthsOfYearList = new ArrayList<MonthsOfTheYear>();

            for (String day : monthsOfYear) {

                monthsOfYearList.add(MonthsOfTheYear.fromValue(day.trim()));

            }

            resourceGroupScheduleParameters

                    .setMonthsOfTheYear(monthsOfYearList);

        }



        if (scheduleEntry.getKey().equalsIgnoreCase(

                CustomStringToSchedulesMapConverter.DAYS_OF_THE_MONTH)) {

            String[] daysOfMonth = scheduleEntry

                    .getValue()

                    .split(CustomStringToSchedulesMapConverter.SCHEDULE_OPTIONS_LIST_DELIMETER);

            List<Integer> daysOfMonthList = new ArrayList<Integer>();

            for (String day : daysOfMonth) {

                daysOfMonthList.add(Integer.parseInt(day.trim()));

            }

            resourceGroupScheduleParameters.setDaysOfTheMonth(daysOfMonthList);

        }



    }

}

