/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/command/validation/SmsCommandRequestValidationHelper.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.snapcenter.cli.command.validation;

import static com.netapp.spl.common.restclient.context.GenericRestInvocationContext.APPLICATION_XML;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfLong;
import com.netapp.smcore.plugin.contracts.ArrayOfSmDiscoverResourceInfo;
import com.netapp.smcore.plugin.contracts.ArrayOfSmKeyValue;
import com.netapp.smcore.plugin.contracts.ArrayOfSmObject;
import com.netapp.smcore.plugin.contracts.ArrayOfSmString;
import com.netapp.smcore.plugin.contracts.ArrayOfString;
import com.netapp.smcore.plugin.contracts.SmAuth;
import com.netapp.smcore.plugin.contracts.SmBackup;
import com.netapp.smcore.plugin.contracts.SmContext;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourceInfo;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourcesRequest;
import com.netapp.smcore.plugin.contracts.SmDiscoverResourcesResponse;
import com.netapp.smcore.plugin.contracts.SmGetBackupDetailsRequest;
import com.netapp.smcore.plugin.contracts.SmGetBackupDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmGetPolicySchedulerDetailsRequest;
import com.netapp.smcore.plugin.contracts.SmGetPolicySchedulerDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsRequest;
import com.netapp.smcore.plugin.contracts.SmGetProtectionGroupsResponse;
import com.netapp.smcore.plugin.contracts.SmGetRolesByUserRequest;
import com.netapp.smcore.plugin.contracts.SmGetRolesByUserResponse;
import com.netapp.smcore.plugin.contracts.SmGetRunAsAccountRequest;
import com.netapp.smcore.plugin.contracts.SmGetRunAsAccountResponse;
import com.netapp.smcore.plugin.contracts.SmHost;
import com.netapp.smcore.plugin.contracts.SmKeyValue;
import com.netapp.smcore.plugin.contracts.SmKeyValueCollection;
import com.netapp.smcore.plugin.contracts.SmObject;
import com.netapp.smcore.plugin.contracts.SmPluginCode;
import com.netapp.smcore.plugin.contracts.SmPolicyType;
import com.netapp.smcore.plugin.contracts.SmProtectionGroup;
import com.netapp.smcore.plugin.contracts.SmString;
import com.netapp.smcore.plugin.contracts.SmUser;
import com.netapp.smcore.plugin.contracts.SmvalidatePolicyRequest;
import com.netapp.smcore.plugin.contracts.SmvalidatePolicyResponse;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerResponse;
import com.netapp.smcore.plugin.contracts.SmGetSMTPServerRequest;
import com.netapp.smcore.plugin.contracts.SmOracleBackupMetadata;
import com.netapp.smcore.plugin.contracts.SmGetBackupRequest;
import com.netapp.smcore.plugin.contracts.SmBackupFilter;
import com.netapp.smcore.plugin.contracts.SmUser;
import com.netapp.smcore.plugin.contracts.SmUserOperationContext;
import com.netapp.snapcenter.cli.contracts.PolicyType;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.helper.SnapCenterServerServiceHelper;
import com.netapp.snapcenter.cli.utils.CLIUtils;
import com.netapp.snapcenter.cli.utils.SMSObjectSerializationUtil;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;
import com.netapp.spl.common.restclient.config.HttpMethod;
import com.netapp.spl.common.restclient.config.SMSRestConfiguration;
import com.netapp.spl.common.restclient.context.SMSRestInvocationContext;
import com.netapp.spl.common.restclient.context.GenericRestInvocationContext;
import com.netapp.smcore.plugin.contracts.SmGetHostRequest;
import com.netapp.smcore.plugin.contracts.SmGetHostResponse;
import com.netapp.smcore.plugin.contracts.ArrayOfSmHost;
import com.netapp.smcore.plugin.contracts.SmHostInfo;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

/**
 * This class has snapcenter specific services for validating the command
 * requests. Generic validation like policy existence, policy with schedule
 * check etc can be placed here.
 */
public class SmsCommandRequestValidationHelper {

    private static final String HAS_SCHEDULER = "HasScheduler";

    private SnapCenterServerServiceHelper snapCenterServerServiceHelper;
    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(SmsCommandRequestValidationHelper.class);

    public SmsCommandRequestValidationHelper(
            SnapCenterServerServiceHelper serviceHelper) {
        this.setSnapCenterServerServiceHelper(serviceHelper);
    }

    public boolean checkIfPoliciesHaveSchedules(List<String> policies,
            PolicyType policyType) throws CLIException {
        boolean policiesHaveSchedules = false;

        if (CLIUtils.isNullOrEmptyCollection(policies)) {
            return false;
        }

        SmGetPolicySchedulerDetailsRequest request = new SmGetPolicySchedulerDetailsRequest();

        ArrayOfSmString arrayOfSmString = new ArrayOfSmString();
        for (String pol : policies) {
            SmString smString = new SmString();
            smString.setValue(pol);
            arrayOfSmString.getSmString().add(smString);
        }
        request.setPolicies(arrayOfSmString);

        request.setPolicytype(SmPolicyType.valueOf(policyType.name()).value());

        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(
                        request,
                        SmGetPolicySchedulerDetailsResponse.class,
                        SMSRestConfiguration.SMS_GET_POLICY_SCHEDULER_DETAILS_PATH);
        SmGetPolicySchedulerDetailsResponse response = (SmGetPolicySchedulerDetailsResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (!getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            throw new CLIException(response.getResult().getMessage());
        } else {
            ArrayOfSmKeyValue arrayOfSmKeyValue = response.getKeyValue();
            if (arrayOfSmKeyValue != null) {
                for (SmKeyValue kv : arrayOfSmKeyValue.getSmKeyValue()) {
                    if (kv.getKey().equals(HAS_SCHEDULER)
                            && StringUtils.isNotBlank(kv.getValue())) {
                        policiesHaveSchedules = Boolean.valueOf(kv.getValue());
                        break;
                    }
                }
            }
        }
        return policiesHaveSchedules;
    }

    public SmvalidatePolicyResponse checkIfPoliciesAreValid(
            List<String> policies) throws CLIException {

        SmvalidatePolicyRequest request = new SmvalidatePolicyRequest();

        ArrayOfSmString arrayOfSmString = new ArrayOfSmString();
        for (String pol : policies) {
            SmString smString = new SmString();
            smString.setValue(pol);
            arrayOfSmString.getSmString().add(smString);
        }
        request.setPolicies(arrayOfSmString);

        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(request,
                        SmvalidatePolicyResponse.class,
                        SMSRestConfiguration.SMS_VALIDATE_POLICY_SERVICE_PATH);
        SmvalidatePolicyResponse response = (SmvalidatePolicyResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        return response;
    }

    public SmGetRunAsAccountResponse getRunAsAccountInformation(String runasName)
            throws CLIException {

        SmGetRunAsAccountRequest smGetRunAsAccountRequest = new SmGetRunAsAccountRequest();
        SmAuth smAuth = new SmAuth();
        smAuth.setRunAsName(runasName);
        smGetRunAsAccountRequest.setAuth(smAuth);

        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(
                        smGetRunAsAccountRequest,
                        SmGetRunAsAccountResponse.class,
                        SMSRestConfiguration.SMS_RBAC_GET_RUN_AS_ACCOUNT_SERVICE_PATH);
        SmGetRunAsAccountResponse response = (SmGetRunAsAccountResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        if (!getSnapCenterServerServiceHelper()
                .isSnapCenterRestOperationResponseResultSuccess(
                        response.getResult())) {
            throw new CLIException(response.getResult().getMessage());
        }

        return response;
    }
   public SmGetHostResponse getSmHost(String hostname)
	   throws CLIException {
	SmGetHostRequest smGetHostRequest = new SmGetHostRequest();
	SmHost smHost = new SmHost();
        smHost.setHostName(hostname);

        ArrayOfSmHost arrayOfSmHost = new ArrayOfSmHost();
        arrayOfSmHost.getSmHost().add(smHost);

        SmHostInfo smHostInfo = new SmHostInfo();
        smHostInfo.setHosts(arrayOfSmHost);

        smGetHostRequest.setDisplayAll(true);
        smGetHostRequest.setHostInfo(smHostInfo);
        SMSRestInvocationContext<?,?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
		.createSMSRestInvocationContext(
				smGetHostRequest,
				SmGetHostResponse.class,
				SMSRestConfiguration.SMS_GET_HOSTS_SERVICE_PATH);
        SmGetHostResponse response = (SmGetHostResponse) getSnapCenterServerServiceHelper()
		.makeRestCallToSnapCenterServer(smsRestInvocationContext);
	if (!getSnapCenterServerServiceHelper()
			.isSnapCenterRestOperationResponseResultSuccess(
				response.getResult())) {
		throw new CLIException(response.getResult().getMessage());
	}
	return response;
   }

    /**
     * Function returns the internal protection group details from the CSM
     * server for given SmObject Id.
     * 
     * @param SmObjectGuid
     * @return SmGetProtectionGroupsResponse
     * @throws CLIException
     */
    public SmGetProtectionGroupsResponse getInternalProtectionGroupDetailsFromServer(
            String smObjectGuid) throws CLIException {

        SmGetProtectionGroupsRequest smGetProtectionGroupsRequest = new SmGetProtectionGroupsRequest();
        smGetProtectionGroupsRequest
                .setProtectionGroup(new SmProtectionGroup());
        smGetProtectionGroupsRequest.setIsGetAllHostResourceMapping(true);
        smGetProtectionGroupsRequest.setIsGetAllPolicies(true);
        smGetProtectionGroupsRequest.setDisplayAll(true);
        smGetProtectionGroupsRequest
                .setResourceGuidForInternalGroup(smObjectGuid);

        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(
                        smGetProtectionGroupsRequest,
                        SmGetProtectionGroupsResponse.class,
                        SMSRestConfiguration.SMS_GET_PROTECTION_GROUP_SERVICE_PATH);
        SmGetProtectionGroupsResponse smGetProtectionGroupsResponse = (SmGetProtectionGroupsResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        return smGetProtectionGroupsResponse;

    }

    /**
     * Get Roles details for user
     * @param smGetRolesByUserRequest
     * @return SmGetRolesByUserResponse
     * @throws CLIException
     */
    public SmGetRolesByUserResponse getRolesByUser(
            String userName, String password)
            throws CLIException {

        SmGetRolesByUserRequest smGetRolesByUserRequest = new SmGetRolesByUserRequest();
        //Passphrase is needed to fetch roles of disjoint domain user.
        SmUser smUser = new SmUser();
        smUser.setName(userName);
        smUser.setPassphrase(password);
        ArrayOfLong arrayOfLong = new ArrayOfLong();
        smUser.setGroupIds(arrayOfLong);
        ArrayOfString arrayOfString = new ArrayOfString();
        smUser.setGroupNames(arrayOfString);

        smGetRolesByUserRequest.setUser(smUser);

        SmKeyValue smKeyValue = new SmKeyValue();
        smKeyValue.setKey("IsAuth");
        smKeyValue.setType("str");
        smKeyValue.setValue("true");

        ArrayOfSmKeyValue arrayOfSmKeyValue = new ArrayOfSmKeyValue();
        arrayOfSmKeyValue.getSmKeyValue().add(smKeyValue);

        SmKeyValueCollection smKeyValueCollection = new SmKeyValueCollection();
        smKeyValueCollection.setData(arrayOfSmKeyValue);

        SmContext smContext = new SmContext();
        smContext.setMetadata(smKeyValueCollection);

        smGetRolesByUserRequest.setContext(smContext);

        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(
                        smGetRolesByUserRequest,
                        SmGetRolesByUserResponse.class,
                        SMSRestConfiguration.SMS_RBAC_GET_ROLES_BY_USER_SERVICE_PATH);
		SmGetRolesByUserResponse smGetRolesByUserResponse = null;

		try {
			smGetRolesByUserResponse = (SmGetRolesByUserResponse) getSnapCenterServerServiceHelper()
					.makeRestCallToSnapCenterServer(smsRestInvocationContext);

		} catch (Exception exception) {

			throw new CLIException(MessageBundle.localizeMessage(
					CLIMsgKey.CLI_SERVER_NOT_REACHABLE_ERROR,
					getSnapCenterServerServiceHelper().getSnapCenterBaseURL()));
		}

        if (smGetRolesByUserResponse.getResult().getErrorCode() != 0
                || StringUtils.isNotBlank(smGetRolesByUserResponse.getResult()
                        .getMessage())) {
            throw new CLIException(smGetRolesByUserResponse.getResult()
                    .getMessage());
        }

        return smGetRolesByUserResponse;
    }

    /**
     * Returns the discovered resources details for the given host
     * 
     * @param smPluginCode
     * @param hostname
     * @param useKnownResource
     * @return SmDiscoverResourcesResponse
     * @throws CLIException
     */
    public SmDiscoverResourcesResponse getDiscoverResourceDetailsFromServer(
            SmPluginCode smPluginCode, String hostname, boolean useKnownResource)
            throws CLIException {

        SmDiscoverResourceInfo smDiscoverResourceInfo = new SmDiscoverResourceInfo();
        smDiscoverResourceInfo.setPluginCode(smPluginCode);

        ArrayOfSmDiscoverResourceInfo arrayOfSmDiscoverResourceInfo = new ArrayOfSmDiscoverResourceInfo();
        arrayOfSmDiscoverResourceInfo.getSmDiscoverResourceInfo().add(
                smDiscoverResourceInfo);

        SmHost smHost = new SmHost();
        smHost.setHostName(hostname);

        SmDiscoverResourcesRequest smDiscoverResourcesRequest = new SmDiscoverResourcesRequest();
        smDiscoverResourcesRequest.setUseKnownResources(useKnownResource);
        smDiscoverResourcesRequest.setHostEx(smHost);
        smDiscoverResourcesRequest
                .setResourceInfo(arrayOfSmDiscoverResourceInfo);

        SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
                .createSMSRestInvocationContext(
                        smDiscoverResourcesRequest,
                        SmDiscoverResourcesResponse.class,
                        SMSRestConfiguration.SMS_DISCOVER_RESOURCES_SERVICE_PATH);

        SmDiscoverResourcesResponse smDiscoverResourcesResponse = (SmDiscoverResourcesResponse) getSnapCenterServerServiceHelper()
                .makeRestCallToSnapCenterServer(smsRestInvocationContext);

        return smDiscoverResourcesResponse;

    }

	public SmGetBackupDetailsResponse getBackupDetailsFromServer(
			String appObjectId, String backupId) throws CLIException {

		SmGetBackupDetailsRequest getBackupDetailsRequest = new SmGetBackupDetailsRequest();

		SmObject smObject = new SmObject();
		smObject.setId(appObjectId);
		ArrayOfSmObject arrayOfSmObject = new ArrayOfSmObject();
		arrayOfSmObject.getSmObject().add(smObject);
		getBackupDetailsRequest.setAppObjects(arrayOfSmObject);

		SmBackup smBackup = new SmBackup();
		smBackup.setBackupId(backupId);
		getBackupDetailsRequest.setBackup(smBackup);

		SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
				.createSMSRestInvocationContext(
						getBackupDetailsRequest,
						SmGetBackupDetailsResponse.class,
						SMSRestConfiguration.SMS_GET_BACKUP_DETAILS_SERVICE_PATH);

		SmGetBackupDetailsResponse smGetBackupDetailsResponse = (SmGetBackupDetailsResponse) getSnapCenterServerServiceHelper()
				.makeRestCallToSnapCenterServer(smsRestInvocationContext);

		return smGetBackupDetailsResponse;

	}

	public SmOracleBackupMetadata getSmOracleBackupMetadata(String appObjectId,
			String backupId, String backupName) throws CLIException {

		   SmGetBackupRequest request = new SmGetBackupRequest();

	        SmBackup smBackup = new SmBackup();
	        smBackup.setBackupId(backupId);
	        smBackup.setBackupName(backupName);
	        request.setBackup(smBackup);

	        SmObject smObject = new SmObject();
	        smObject.setId(appObjectId);
	        request.setAppObject(smObject);

	        SmBackupFilter smBackupFilter = new SmBackupFilter();
	        smBackupFilter.setPluginCode(SmPluginCode.SCO);
	        request.setBackupFilter(smBackupFilter);

	      SmUserOperationContext smUserOperationContext = new SmUserOperationContext();
	          SmUser smUser = new SmUser();
	          smUserOperationContext.setUser(smUser);
	          request.setUserOperationContext(smUserOperationContext);

	        GenericRestInvocationContext<Object, SmOracleBackupMetadata> restInvocationContext = new GenericRestInvocationContext<>();
	        restInvocationContext.setAccepts(APPLICATION_XML);
	        restInvocationContext.setHttpMethod(HttpMethod.POST);
	        restInvocationContext.setResponseClass(SmOracleBackupMetadata.class);
	        restInvocationContext.setContentType(APPLICATION_XML);
            restInvocationContext.setRequest(request);
            restInvocationContext
                    .setServiceUrlPath(SMSRestConfiguration.SMS_GET_ORACLE_BACKUP_METADATA_PATH);

            CLIUtils.writeToLog(
                    "SMS Request Object: \n"
                            + SMSObjectSerializationUtil
                                    .getXMLSerializedSMSObject(request),
                    logger, LogLevel.DEBUG);

		SmOracleBackupMetadata smOracleBackupMetadata = (SmOracleBackupMetadata) getSnapCenterServerServiceHelper()
				.makeRestCallToSnapCenterServerWithGenericRestInvocationContext(
						restInvocationContext);

		 CLIUtils.writeToLog(
	                "SMS response Object: \n"
	                        + SMSObjectSerializationUtil
	                                .getXMLSerializedSMSObject(smOracleBackupMetadata),
	                logger, LogLevel.DEBUG);
		
		return smOracleBackupMetadata;

	}

	public SmGetSMTPServerResponse getSmSMTPServerDetailsFromServer()
			throws CLIException {

		SmGetSMTPServerRequest getSMTPServerRequest = new SmGetSMTPServerRequest();

		SMSRestInvocationContext<?, ?> smsRestInvocationContext = getSnapCenterServerServiceHelper()
				.createSMSRestInvocationContext(getSMTPServerRequest,
						SmGetSMTPServerResponse.class,
						SMSRestConfiguration.SMS_GET_SMTP_SERVER_PATH);
		smsRestInvocationContext.setHttpMethod(HttpMethod.GET);;

		SmGetSMTPServerResponse smGetSMTPServerResponse = (SmGetSMTPServerResponse) getSnapCenterServerServiceHelper()
				.makeRestCallToSnapCenterServer(smsRestInvocationContext);

		return smGetSMTPServerResponse;

	}


    public SnapCenterServerServiceHelper getSnapCenterServerServiceHelper() {
        return snapCenterServerServiceHelper;
    }

    public void setSnapCenterServerServiceHelper(
            SnapCenterServerServiceHelper snapCenterServerServiceHelper) {
        this.snapCenterServerServiceHelper = snapCenterServerServiceHelper;
    }

}
