/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/utils/OracleDatabaseDetailsUtils.java#4 $
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.utils;

import org.apache.commons.lang3.StringUtils;

import com.netapp.smcore.plugin.contracts.SmOracleDataBaseRole;
import com.netapp.smcore.plugin.contracts.SmOracleDb;
import com.netapp.smcore.plugin.contracts.SmOracleDbType;

public class OracleDatabaseDetailsUtils {

    // Oracle Database Types
    public static final String ORACLE_RAC = "Oracle RAC";
    public static final String ORACLE_RAC_MULTITENANT = "Oracle RAC (Multitenant)";
    public static final String ORACLE_RAC_PHYSICALSTANDBY = "Oracle RAC Physical Standby";
    public static final String ORACLE_RAC_PHYSICALSTANDBY_MULTITENANT = "Oracle RAC Physical Standby (Multitenant)";
    public static final String ORACLE_SINGLEINSTANCE = "Oracle Single Instance";
    public static final String ORACLE_SINGLEINSTANCE_MULTITENANT = "Oracle Single Instance (Multitenant)";
    public static final String ORACLE_SINGLEINSTANCE_PHYSICALSTANDBY = "Oracle Single Instance Physical Standby";
    public static final String ORACLE_SINGLEINSTANCE_PHYSICALSTANDBY_MULTITENANT = "Oracle Single Instance Physical Standby (Multitenant)";
    public static final String UNKNOWN = "Unknown";

    private OracleDatabaseDetailsUtils() {

    }

    public static String getOracleDatabaseTypeForDisplay(boolean isRAC,
            boolean isCDB, SmOracleDataBaseRole smOracleDataBaseRole) {
        String databaseType = UNKNOWN;

        if (smOracleDataBaseRole.equals(SmOracleDataBaseRole.PRIMARY)) {
            if (isRAC) {
                databaseType = isCDB ? ORACLE_RAC_MULTITENANT : ORACLE_RAC;
            } else {
                databaseType = isCDB ? ORACLE_SINGLEINSTANCE_MULTITENANT
                        : ORACLE_SINGLEINSTANCE;
            }
        } else if (smOracleDataBaseRole
                .equals(SmOracleDataBaseRole.PHYSICAL_STANDBY)) {
            if (isRAC) {
                databaseType = isCDB ? ORACLE_RAC_PHYSICALSTANDBY_MULTITENANT
                        : ORACLE_RAC_PHYSICALSTANDBY;
            } else {
                databaseType = isCDB ? ORACLE_SINGLEINSTANCE_PHYSICALSTANDBY_MULTITENANT
                        : ORACLE_SINGLEINSTANCE_PHYSICALSTANDBY;
            }
        }
        return databaseType;
    }
 
    public static String getOracleDatabaseTypeForDisplay(SmOracleDb smOracleDb) {

        boolean isRAC = StringUtils.isBlank(smOracleDb.getRacName()) ? false
                : true;
        boolean isCDB = (smOracleDb.getDbType().equals(SmOracleDbType.CDB)) ? true
                : false;
       return getOracleDatabaseTypeForDisplay(isRAC,isCDB,smOracleDb.getDatabaseRole());

    }

}