/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/utils/ProgressBar.java#1 $
 * 
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.utils;

import org.apache.commons.lang3.StringUtils;

import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.snapcenter.cli.utils.CLIUtils.LogLevel;

/**
 * 
 * This class is used to display a progress bar which is run as a saperate
 * thread. The charater to be displayed as a progress bar can be configured.
 * This will display progress bar in one line based on console width.
 * 
 *
 */
public class ProgressBar extends Thread {

    private ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(ProgressBar.class);

    private String displayCharacter = ".";

    private String displayMessage = ".";

    public ProgressBar() {

    }

    private volatile boolean running = true;

    public void terminate() {
        running = false;
    }

    @Override
    public void run() {

        /*
         * If System Console is null then don't print progress bar.
         * Just return.
         */
        if (System.console() == null) {
            return;
        }

        int consoleWidth = CLIUtils.getConsoleWidth();

        while (running) {
            System.out.print(displayMessage);
            for (int i = 0; i < (consoleWidth-displayMessage.length()) && running; i++) {
                System.out.print(displayCharacter);
                try {
                    Thread.sleep(500L);
                } catch (InterruptedException e) {
                    CLIUtils.writeToLog(e.getLocalizedMessage(), logger,
                            LogLevel.DEBUG);
                    running = false;
                }
            }
            // Carriage return
            System.out.print(CLIUtils.CARRIAGE_RETURN);
            // Erase Line
            for (int i = 0; i < consoleWidth; i++) {
                System.out.print(" ");
            }
            // Carriage return
            System.out.print(CLIUtils.CARRIAGE_RETURN);
        }
        System.out.print(CLIUtils.NEW_LINE);
    }

    public void setDisplayCharacter(String displayCharacter) {
        if (StringUtils.isBlank(displayCharacter)) {
            return;
        }
        this.displayCharacter = displayCharacter;
    }

    public String getDisplayCharacter() {
        return displayCharacter;
    }

    public void setDisplayMessage(String displayMessage) {
        if (StringUtils.isBlank(displayMessage)) {
            return;
        }
        this.displayMessage = displayMessage;
    }

    public String getDisplayMessage() {
        return displayMessage;
    }
}
