/*
 * $Id: //depot/prod/spl/main/snapcenter-cli/src/main/java/com/netapp/snapcenter/cli/utils/SmoCloneCliUtil.java#4 $
 *
 * Copyright (c) 2016 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.snapcenter.cli.utils;

import static com.netapp.snapcenter.cli.utils.CLIUtils.SEMICOLON;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang3.StringUtils;

import com.netapp.common.logging.MessageBundle;
import com.netapp.smcore.loader.logger.ExtendedGenericLogger;
import com.netapp.smcore.loader.logger.LoggerFactory;
import com.netapp.smcore.plugin.contracts.ArrayOfSmArchivedDetail;
import com.netapp.smcore.plugin.contracts.ArrayOfSmArchivedLocator;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackup;
import com.netapp.smcore.plugin.contracts.ArrayOfSmBackupComponentInfo;
import com.netapp.smcore.plugin.contracts.ArrayOfSmHostResourceMapping;
import com.netapp.smcore.plugin.contracts.ArrayOfSmOracleASMDiskGroup;
import com.netapp.smcore.plugin.contracts.ArrayOfSmOracleControlFileConfiguration;
import com.netapp.smcore.plugin.contracts.ArrayOfSmOracleCustomParameter;
import com.netapp.smcore.plugin.contracts.ArrayOfSmOracleDataFileConfiguration;
import com.netapp.smcore.plugin.contracts.ArrayOfSmOracleGenericQueryConfiguration;
import com.netapp.smcore.plugin.contracts.ArrayOfSmOraclePluggableDatabaseConfiguration;
import com.netapp.smcore.plugin.contracts.ArrayOfSmOracleRedologFileConfiguration;
import com.netapp.smcore.plugin.contracts.ArrayOfSmPath;
import com.netapp.smcore.plugin.contracts.LocationType;
import com.netapp.smcore.plugin.contracts.ResourceType;
import com.netapp.smcore.plugin.contracts.SDStorageFootprint;
import com.netapp.smcore.plugin.contracts.SDStorageResource;
import com.netapp.smcore.plugin.contracts.SmArchivedDetail;
import com.netapp.smcore.plugin.contracts.SmArchivedLocator;
import com.netapp.smcore.plugin.contracts.SmAuth;
import com.netapp.smcore.plugin.contracts.SmAuthMode;
import com.netapp.smcore.plugin.contracts.SmBackup;
import com.netapp.smcore.plugin.contracts.SmGetArchivedDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmGetBackupDetailsResponse;
import com.netapp.smcore.plugin.contracts.SmKeyValue;
import com.netapp.smcore.plugin.contracts.SmOracleASMDiskGroup;
import com.netapp.smcore.plugin.contracts.SmOracleBackupComponentInfo;
import com.netapp.smcore.plugin.contracts.SmOracleCloneConfiguration;
import com.netapp.smcore.plugin.contracts.SmOracleControlFileConfiguration;
import com.netapp.smcore.plugin.contracts.SmOracleCustomParameter;
import com.netapp.smcore.plugin.contracts.SmOracleDataFileConfiguration;
import com.netapp.smcore.plugin.contracts.SmOracleDbEnhanced;
import com.netapp.smcore.plugin.contracts.SmOracleDetail;
import com.netapp.smcore.plugin.contracts.SmOracleGenericQueryConfiguration;
import com.netapp.smcore.plugin.contracts.SmOracleOsUserDetail;
import com.netapp.smcore.plugin.contracts.SmOracleParameter;
import com.netapp.smcore.plugin.contracts.SmOraclePluggableDatabase;
import com.netapp.smcore.plugin.contracts.SmOraclePluggableDatabaseConfiguration;
import com.netapp.smcore.plugin.contracts.SmOracleRecoveryScopeParameters;
import com.netapp.smcore.plugin.contracts.SmOracleRedoLogFile;
import com.netapp.smcore.plugin.contracts.SmOracleRedologFileConfiguration;
import com.netapp.smcore.plugin.contracts.SmPath;
import com.netapp.smcore.plugin.contracts.SmSnapshotInfo;
import com.netapp.smcore.plugin.contracts.SnapshotLocator;
import com.netapp.snapcenter.cli.clone.contracts.AsmAuthentication;
import com.netapp.snapcenter.cli.clone.contracts.ControlFile;
import com.netapp.snapcenter.cli.clone.contracts.ControlFiles;
import com.netapp.snapcenter.cli.clone.contracts.ControlFilesConfiguration;
import com.netapp.snapcenter.cli.clone.contracts.DataFilesConfiguration;
import com.netapp.snapcenter.cli.clone.contracts.DataFilesLocation;
import com.netapp.snapcenter.cli.clone.contracts.DataFilesLocations;
import com.netapp.snapcenter.cli.clone.contracts.DatabaseAuthentication;
import com.netapp.snapcenter.cli.clone.contracts.DatabaseParameter;
import com.netapp.snapcenter.cli.clone.contracts.DatabaseParameters;
import com.netapp.snapcenter.cli.clone.contracts.DatabaseSpecification;
import com.netapp.snapcenter.cli.clone.contracts.OracleCloneSpecification;
import com.netapp.snapcenter.cli.clone.contracts.OracleOsAccount;
import com.netapp.snapcenter.cli.clone.contracts.PdbConfigurationType;
import com.netapp.snapcenter.cli.clone.contracts.PdbsConfiguration;
import com.netapp.snapcenter.cli.clone.contracts.Redogroup;
import com.netapp.snapcenter.cli.clone.contracts.RedologFile;
import com.netapp.snapcenter.cli.clone.contracts.RedologFiles;
import com.netapp.snapcenter.cli.clone.contracts.Redologs;
import com.netapp.snapcenter.cli.clone.contracts.RedologsConfiguration;
import com.netapp.snapcenter.cli.clone.contracts.SecondaryConfiguration;
import com.netapp.snapcenter.cli.clone.contracts.SecondaryLocator;
import com.netapp.snapcenter.cli.clone.contracts.SecondaryLocators;
import com.netapp.snapcenter.cli.clone.contracts.SizeType;
import com.netapp.snapcenter.cli.clone.contracts.SqlStatements;
import com.netapp.snapcenter.cli.clone.contracts.StorageLocationType;
import com.netapp.snapcenter.cli.clone.contracts.StorageMapping;
import com.netapp.snapcenter.cli.clone.contracts.StorageSnapshotLocator;
import com.netapp.snapcenter.cli.clone.contracts.StorageSpecification;
import com.netapp.snapcenter.cli.clone.contracts.UnitType;
import com.netapp.snapcenter.cli.command.CreateSmOracleCloneSpecificationCommand;
import com.netapp.snapcenter.cli.command.NewSmCloneCommand;
import com.netapp.snapcenter.cli.command.RefreshSmCloneCommand;
import com.netapp.snapcenter.cli.command.mapper.CustomSMSObjectMapperHelper;
import com.netapp.snapcenter.cli.command.parameters.SMOCloneParameters;
import com.netapp.snapcenter.cli.command.validation.SmsBackupValidationHelper;
import com.netapp.snapcenter.cli.command.validation.SmsCommandResponseValidationUtil;
import com.netapp.snapcenter.cli.exception.CLIException;
import com.netapp.snapcenter.cli.logging.CLIMsgKey;

public class SmoCloneCliUtil {

    private static final String DB_RECOVERY_FILE_DEST = "DB_RECOVERY_FILE_DEST";

    private static final String DB_PARAM_AUDIT_FILE_DEST = "audit_file_dest";
    private static final String DB_PARAM_LOG_ARCHIVE_DEST_1 = "log_archive_dest_1";
    private static final String DB_PARAM_LOG_ARCHIVE_FORMAT = "log_archive_format";
    private static final String DB_PARAM_AUDIT_TRAIL = "audit_trail";
    private static final String DB_PARAM_PROCESSES = "processes";
    private static final String DB_PARAM_PGA_AGGREGATE_TARGET = "pga_aggregate_target";
    private static final String DB_PARAM_REMOTE_LOGIN_PASSWORDFILE = "remote_login_passwordfile";
    private static final String DB_PARAM_UNDO_TABLESPACE = "undo_tablespace";
    private static final String DB_PARAM_OPEN_CURSORS = "open_cursors";
    private static final String DB_PARAM_SGA_TARGET = "sga_target";
    private static final String DB_PARAM_DB_BLOCK_SIZE = "db_block_size";

    private static final List<String> overridableDatabaseParameters = Arrays
            .asList(DB_PARAM_AUDIT_FILE_DEST, DB_PARAM_LOG_ARCHIVE_DEST_1,
                    DB_PARAM_LOG_ARCHIVE_FORMAT, DB_PARAM_AUDIT_TRAIL,
                    DB_PARAM_PROCESSES, DB_PARAM_PGA_AGGREGATE_TARGET,
                    DB_PARAM_REMOTE_LOGIN_PASSWORDFILE,
                    DB_PARAM_UNDO_TABLESPACE, DB_PARAM_OPEN_CURSORS,
                    DB_PARAM_SGA_TARGET,DB_PARAM_DB_BLOCK_SIZE);

    private static final String PDB_SEED = "PDB$SEED";
    private static final long REDO_BLOCK_SIZE = 512L;
    private static final String ORACLE_CONTROL_FILE_PARAMETER_KEY = "control_files";
    private static final String CONTROL_FILE_DIRECTORY_NAME = "control";
    private static final String REDO_LOGS_DIRECTORY_NAME = "redolog";
    private static final String CONTROL_FILE_NAME_FORMAT = "control%02d.ctl";
    private static final String REDO_LOGS_NAME_FORMAT = "redo%02d_%02d.log";
    private static final int DATABASE_OR_ASM_DEFAULT_PORT = 1521;

    public static final String CLONE_SPEC_DIRECTORY = "/var/opt/snapcenter/sco/clone_specs";
    public static final String ORACLE_CLONE_SPEC_FILE_FORMAT = "oracle_clonespec_%s_%s_%s.xml";

    private static ExtendedGenericLogger logger = LoggerFactory
            .getExtendedClassAwareLogger(SmoCloneCliUtil.class);

    private SmoCloneCliUtil() {

    }

    /**
     * This method will help in populating details required for creating a clone
     * specification file.
     * 
     * @param createSmOracleCloneSpecificationCommand
     * @param metadata
     * @return {@link OracleCloneSpecification}
     * @throws CLIException
     */
    public static OracleCloneSpecification createCloneSpec(
            CreateSmOracleCloneSpecificationCommand createSmOracleCloneSpecificationCommand,
            SmGetBackupDetailsResponse metadata) throws CLIException {
        SmsCommandResponseValidationUtil.checkBackupDetailsResponseInformation(metadata);

        OracleCloneSpecification cloneSpec = new OracleCloneSpecification();

        cloneSpec.setBackupname(createSmOracleCloneSpecificationCommand
                .getBackupName());
        cloneSpec.setAppobjectid(createSmOracleCloneSpecificationCommand
                .getAppObjectId());
        cloneSpec.setCloneDatabaseSid(createSmOracleCloneSpecificationCommand
                .getCloneDatabaseSid());

        StorageSpecification storageSpec = new StorageSpecification();
        populateStorageSpecification(createSmOracleCloneSpecificationCommand,
                metadata, storageSpec);

        cloneSpec.setStorageSpecification(storageSpec);

        DatabaseSpecification dbSpec = new DatabaseSpecification();

        List<String> destinationStorageLocations = getDestinationStorageLocations(storageSpec);
        String destinationControlFileAndRedoLogBaseLocation = new File(
                destinationStorageLocations.iterator().next(),
                createSmOracleCloneSpecificationCommand.getCloneDatabaseSid())
                .getPath();
        populateDatabasebSpecification(
                createSmOracleCloneSpecificationCommand.getCloneDatabaseSid(),
                destinationControlFileAndRedoLogBaseLocation, metadata, dbSpec);
        cloneSpec.setDatabaseSpecification(dbSpec);

        return cloneSpec;
    }

    private static List<String> getDestinationStorageLocations(
            StorageSpecification storageSpec) {
        List<String> destinationLocations = new ArrayList<>();
        for (DataFilesLocation dfLocation : storageSpec.getStorageMapping()
                .getDataFilesConfiguration().getDataFilesLocations()
                .getDataFilesLocation()) {
            destinationLocations.add(dfLocation.getCloneLocation());
        }
        return destinationLocations;
    }


    private static void populateDatabasebSpecification(String cloneDatabaseSID,
            String destinationControlFileAndRedoLogBaseLocation,
            SmGetBackupDetailsResponse metadata, DatabaseSpecification dbSpec) throws CLIException {

        SmOracleBackupComponentInfo smOracleBackupComponentInfo = getSmOracleBackupComponentInfoFromBackupMetadata(metadata);

        SmOracleDbEnhanced smOracleDbEnhanced = smOracleBackupComponentInfo
                .getSmOracleDatabaseInstance();

        DatabaseAuthentication databaseAuthentication = new DatabaseAuthentication();
        databaseAuthentication.setRunasName(StringUtils.EMPTY);
        databaseAuthentication.setPort(DATABASE_OR_ASM_DEFAULT_PORT);
        dbSpec.setDatabaseAuthentication(databaseAuthentication);

        AsmAuthentication asmAuthentication = new AsmAuthentication();
        asmAuthentication.setRunasName(StringUtils.EMPTY);
        asmAuthentication.setPort(DATABASE_OR_ASM_DEFAULT_PORT);
        dbSpec.setAsmAuthentication(asmAuthentication);

        // By default enable Archive Log Mode
        dbSpec.setEnableArchivelogMode(true);

        // populating overridable paramteres
        DatabaseParameters databaseParameters = new DatabaseParameters();
        for (SmOracleParameter smOracleParameter : smOracleDbEnhanced
                .getParameters().getSmOracleParameter()) {
			if (!overridableDatabaseParameters.contains(smOracleParameter
					.getName())) {
				continue;
			}
            DatabaseParameter dbParam = new DatabaseParameter();
            dbParam.setName(smOracleParameter.getName());
            dbParam.setValue(processOverridableDatabaseParameterAndGetValue(
                    cloneDatabaseSID, smOracleParameter, smOracleDbEnhanced));
            databaseParameters.getDatabaseParameter().add(dbParam);
        }
        dbSpec.setDatabaseParameters(databaseParameters);

        SqlStatements sqlStatements = new SqlStatements();
        sqlStatements.getSqlStatement().add(StringUtils.EMPTY);
        dbSpec.setSqlStatements(sqlStatements);

        // Set Oracle Home And version
        SmOracleDetail smOracleDetail = smOracleDbEnhanced.getOracleDetail();
        dbSpec.setOracleHome(smOracleDetail.getOracleHome());
        dbSpec.setOracleVersion(smOracleDetail.getOracleFullVersion());

        // Set Oracle User And Group
        SmOracleOsUserDetail smOracleOsUserDetail = smOracleDbEnhanced
                .getOsUserDetail();
        OracleOsAccount oracleOsAccount = new OracleOsAccount();
        oracleOsAccount.setUsername(smOracleOsUserDetail.getOsUserName());
        oracleOsAccount.setGroup(smOracleOsUserDetail.getOsUserGroupName());
        dbSpec.setOracleOsAccount(oracleOsAccount);

        /**
         * Autogenerate Control Files. We will generate control files with a
         * fixed format and files count will be same as source database.
         */
        ControlFilesConfiguration controlFilesConfiguration = new ControlFilesConfiguration();
        String destinationControlFileLocation = new File(
                destinationControlFileAndRedoLogBaseLocation,
                CONTROL_FILE_DIRECTORY_NAME).getPath();
        int controlFilesCount = getControlFilesNameFromParameters(
                smOracleDbEnhanced.getParameters().getSmOracleParameter())
                .size();
        ControlFiles controlFiles = new ControlFiles();
        for (int i = 1; i <= controlFilesCount; i++) {
            ControlFile controlFile = new ControlFile();
            controlFile.setPath(generateDestinationControlFilePath(
                    destinationControlFileLocation, i));
            controlFiles.getControlFile().add(controlFile);
        }
        controlFilesConfiguration.setControlFiles(controlFiles);
        dbSpec.setControlFilesConfiguration(controlFilesConfiguration);

        /**
         * Autogenerate RedoLog Files. We will generate redo logs with a fixed
         * format and group and its log count will be same as source database.
         */
        RedologsConfiguration redologsConfiguration = new RedologsConfiguration();
        Redologs redologs = new Redologs();
        Map<Long, List<SmOracleRedoLogFile>> redoGroupToFilesMap = new HashMap<>();

        for (SmOracleRedoLogFile smOracleRedoLogFile : smOracleDbEnhanced
                .getRedoLogFiles().getSmOracleRedoLogFile()) {
            if (redoGroupToFilesMap.containsKey(smOracleRedoLogFile
                    .getRedoGroup())) {
                redoGroupToFilesMap.get(smOracleRedoLogFile.getRedoGroup())
                        .add(smOracleRedoLogFile);
            } else {
                List<SmOracleRedoLogFile> redoLogs = new ArrayList<>();
                redoLogs.add(smOracleRedoLogFile);
                redoGroupToFilesMap.put(smOracleRedoLogFile.getRedoGroup(),
                        redoLogs);
            }
        }

        int genRedoNumber = 0;
        String destinationRedoLogsLocation = new File(
                destinationControlFileAndRedoLogBaseLocation,
                REDO_LOGS_DIRECTORY_NAME).getPath();
        for (Long rgp : redoGroupToFilesMap.keySet()) {
            genRedoNumber++;
            Redogroup redogroup = new Redogroup();
            RedologFiles redologFiles = new RedologFiles();

            SizeType totalSize = new SizeType();
            totalSize.setValue(redoGroupToFilesMap.get(rgp).iterator()
                    .next().getTotalSize());
            totalSize.setUnit(UnitType.M);
            redogroup.setTotalSize(totalSize);
            redogroup.setNumber(genRedoNumber);
            int redoLogFilesCount = redoGroupToFilesMap.get(rgp).size();
            for (int i = 1; i <= redoLogFilesCount; i++) {
                RedologFile redologFile = new RedologFile();
                redologFile.setPath(generateDestinationRedoLogFilePath(
                        destinationRedoLogsLocation, genRedoNumber, i));
                redologFiles.getRedologFile().add(redologFile);

            }
            redogroup.setRedologFiles(redologFiles);
            redologs.getRedogroup().add(redogroup);
        }

        redologsConfiguration.setRedologs(redologs);
        dbSpec.setRedologsConfiguration(redologsConfiguration);

        // Populate PDB configuration if 12c cdb database
        if (smOracleDbEnhanced.getPluggableDatabases() != null
                && !smOracleDbEnhanced.getPluggableDatabases()
                        .getSmOraclePluggableDatabase().isEmpty()) {
            PdbsConfiguration pdbsConfiguration = new PdbsConfiguration();
            for (SmOraclePluggableDatabase pdb : smOracleDbEnhanced
                    .getPluggableDatabases().getSmOraclePluggableDatabase()) {
                // PDB$SEED cannot be renamed since it is a template
                if (pdb.getPdbName().equalsIgnoreCase(PDB_SEED)) {
                    continue;
                }
                PdbConfigurationType pdbConfigurationType = new PdbConfigurationType();
                pdbConfigurationType.setCurrentName(pdb.getPdbName());
                // User should provide it else empty
                pdbConfigurationType.setNewName(StringUtils.EMPTY);
                pdbsConfiguration.getPdbConfiguration().add(
                        pdbConfigurationType);
            }
            pdbsConfiguration.getPdbConfiguration();
            dbSpec.setPdbsConfiguration(pdbsConfiguration);
        }

    }

    /**
     * For overridable parameters to be displayed in clone spec. Autogen for
     * audit dest and log dest by appending sid. For rest leave it the same.
     * 
     */
    private static String processOverridableDatabaseParameterAndGetValue(
            String cloneDatabaseSID, SmOracleParameter smOracleParameter,
            SmOracleDbEnhanced smOracleDbEnhanced) {
        String paramValue = StringUtils.EMPTY;
        switch (smOracleParameter.getName()) {
            case DB_PARAM_AUDIT_FILE_DEST:
                if (StringUtils.isNotBlank(smOracleParameter.getValue())) {
                    paramValue = String.format("%s_%s",
                            smOracleParameter.getValue(), cloneDatabaseSID);
                }
                break;
            case DB_PARAM_LOG_ARCHIVE_DEST_1:
                if (StringUtils.isNotBlank(smOracleParameter.getValue())) {
                    paramValue = String.format("LOCATION=%s%s%s", smOracleDbEnhanced
                            .getOracleDetail().getOracleHome(),
                            File.separator, cloneDatabaseSID);
                }
               break;
            default:
                paramValue = smOracleParameter.getValue();
                break;
        }
        return paramValue;
    }

    private static String generateDestinationRedoLogFilePath(
            String destinationRedoLogsLocation, int redoGroupNumber,
            int redoFileNumber) {
        return new File(destinationRedoLogsLocation, String.format(
                REDO_LOGS_NAME_FORMAT, redoGroupNumber, redoFileNumber))
                .getPath();
    }

    private static String generateDestinationControlFilePath(
            String destinationControlFileLocation, int cfNumber) {
        return new File(destinationControlFileLocation, String.format(
                CONTROL_FILE_NAME_FORMAT, cfNumber)).getPath();
    }

    /**
     * This will get control files location from paramteres stored in metadata.
     */
    private static List<String> getControlFilesNameFromParameters(
            List<SmOracleParameter> smOracleParameter) {
        List<String> controlFiles = new ArrayList<>();
        for (SmOracleParameter param : smOracleParameter) {
            if (param.getName().equalsIgnoreCase(
                    ORACLE_CONTROL_FILE_PARAMETER_KEY)) {
                File cf = new File(param.getValue());
                controlFiles.add(cf.getName());
            }
        }
        return controlFiles;
    }

    private static void populateStorageSpecification(
            CreateSmOracleCloneSpecificationCommand createSmOracleCloneSpecCmd,
            SmGetBackupDetailsResponse metadata,
            StorageSpecification storageSpec) throws CLIException {

        StorageMapping storageMapping = new StorageMapping();
        DataFilesConfiguration dataFilesConfiguration = createDataFilesConfiguration(
                createSmOracleCloneSpecCmd.getCloneDatabaseSid(), metadata);
        storageMapping.setDataFilesConfiguration(dataFilesConfiguration);

        if (createSmOracleCloneSpecCmd.isIncludeSecondaryDetails()) {
            SmsBackupValidationHelper smsBackupValidationHelper = new SmsBackupValidationHelper(
                    createSmOracleCloneSpecCmd
                            .getSnapCenterServerServiceHelper());
            ProgressBar progressBar = ProgressBarUtil
                    .createAndStartProgressBar("Fetching secondary backup details ");

            SmGetArchivedDetailsResponse smGetArchivedDetailsResponse = smsBackupValidationHelper
                    .getSecondaryBackupDetails(
                            createSmOracleCloneSpecCmd.getBackupId(),
                            createSmOracleCloneSpecCmd.getAppObjectId(),
                            createSmOracleCloneSpecCmd.getAppPluginCode(),
                            true);

            ProgressBarUtil.stopProgressBar(progressBar);

			SecondaryConfiguration secondaryConfigurationForDatafiles = createSecondaryConfiguration(
					smGetArchivedDetailsResponse.getArchivedDetails(),
					createSmOracleCloneSpecCmd.getSecondaryStorageType(),
					createSmOracleCloneSpecCmd.getBackupName(), false);
			storageMapping
					.setSecondaryConfigurationDatafiles(secondaryConfigurationForDatafiles);

			SecondaryConfiguration secondaryConfigurationForArchivelogs = createSecondaryConfiguration(
					smGetArchivedDetailsResponse.getLogArchivedDetails(),
					createSmOracleCloneSpecCmd.getSecondaryStorageType(),
					createSmOracleCloneSpecCmd.getBackupName(), true);
			storageMapping
					.setSecondaryConfigurationArchivelogs(secondaryConfigurationForArchivelogs);
        }

        storageSpec.setStorageMapping(storageMapping);
    }

	private static SecondaryConfiguration createSecondaryConfiguration(
			ArrayOfSmArchivedDetail arrayOfSmArchivedDetail,
			String secondaryStorageType, String backupId, boolean isLogBackup)
			throws CLIException {
        SecondaryConfiguration secondaryConfiguration = new SecondaryConfiguration();
        SecondaryLocators secondaryLocators = new SecondaryLocators();

        if (arrayOfSmArchivedDetail != null) {
            for (SmArchivedDetail archivedDetail : arrayOfSmArchivedDetail
                    .getSmArchivedDetail()) {

                SnapshotLocator primaryFromResponse = archivedDetail
                        .getPrimary();
                SnapshotLocator firstSecondaryFromResponse = archivedDetail
                        .getSecondaries().getSnapshotLocator().iterator()
                        .next();
                SecondaryLocator firstSecondaryLocator = createSecondaryLocator(
                        primaryFromResponse.getStorageSystemName(),
                        primaryFromResponse.getStorageVolumeName(),
                        firstSecondaryFromResponse.getStorageSystemName(),
                        firstSecondaryFromResponse.getStorageVolumeName());

                
                List<SnapshotLocator> snapshotLocators = archivedDetail.getSecondaries().getSnapshotLocator();
                boolean foundSecondaryLocator = false;
				for (SnapshotLocator secondaryFromResponse : snapshotLocators) {

					if (!StringUtils.isBlank(secondaryStorageType)) {
						// Ignore locators which user has not asked for.
						if (!secondaryStorageType
								.equalsIgnoreCase(secondaryFromResponse
										.getRelationshipType().toString())) {
							logger.debug("Ignoring archive locator "
									+ secondaryFromResponse
											.getStorageSystemName()
									+ ":"
									+ secondaryFromResponse
											.getStorageVolumeName()
									+ " of type "
									+ secondaryFromResponse
											.getRelationshipType().toString());
							continue;
						}
					}

					SecondaryLocator secondaryLocator = createSecondaryLocator(
							primaryFromResponse.getStorageSystemName(),
							primaryFromResponse.getStorageVolumeName(),
							secondaryFromResponse.getStorageSystemName(),
							secondaryFromResponse.getStorageVolumeName());

					secondaryLocators.getSecondaryLocator().add(
							secondaryLocator);
					// After we get the first locator of desired type, ignore other locators
					// as we expect user to edit the clone spec file.
					foundSecondaryLocator = true;
					break;
				}
				
				if (!isLogBackup && !foundSecondaryLocator) {
					// Valid secondary for only data backups.
					if (StringUtils.isBlank(secondaryStorageType)) {
						throw new CLIException(
								MessageBundle
										.localizeMessage(
												CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_NO_SECONDARY_FOUND,
												primaryFromResponse
														.getStorageSystemName(),
												primaryFromResponse
														.getStorageVolumeName(),
												backupId));
					} else {
						throw new CLIException(
								MessageBundle
										.localizeMessage(
												CLIMsgKey.CLI_CREATESMORACLECLONESPECIFICATION_NO_SECONDARY_OF_GIVEN_TYPE_FOUND,
												secondaryStorageType,
												primaryFromResponse
														.getStorageSystemName(),
												primaryFromResponse
														.getStorageVolumeName(),
												backupId));
					}
				} else if (isLogBackup
						&& !StringUtils.isBlank(secondaryStorageType)
						&& !foundSecondaryLocator) {
					// If user asked for Mirror, but latest log backup is in
					// Vault, this will handle it.
					secondaryLocators.getSecondaryLocator().add(
							firstSecondaryLocator);
				}
            }
        }
        secondaryConfiguration.setSecondaryLocators(secondaryLocators);

        return secondaryConfiguration;
    }

    private static SecondaryLocator createSecondaryLocator(String primarySVM,
            String primaryVolume, String secondarySVM, String secondaryVolume) {
        SecondaryLocator secondaryLocator = new SecondaryLocator();

        StorageSnapshotLocator primary = new StorageSnapshotLocator();
        primary.setSvm(primarySVM);
        primary.setVolume(primaryVolume);
        secondaryLocator.setPrimary(primary);

        StorageSnapshotLocator secondary = new StorageSnapshotLocator();
        secondary.setSvm(secondarySVM);
        secondary.setVolume(secondaryVolume);
        secondaryLocator.setSecondary(secondary);

        return secondaryLocator;
    }
    
    public static String getHashCode(String diskgroup) {
        long hashCode = diskgroup.hashCode();
        String returnValue = Long.toString(hashCode & 0xFFFFFFFFL);
        logger.debug("hash code for " + diskgroup + " is " + returnValue);
        return returnValue;
    }

    private static DataFilesConfiguration createDataFilesConfiguration(
            String cloneDatabaseSid, SmGetBackupDetailsResponse metadata) {
        DataFilesConfiguration dataFilesConfiguration = new DataFilesConfiguration();
        DataFilesLocations dataFilesLocations = new DataFilesLocations();

        SmOracleBackupComponentInfo smOracleBackupComponentInfo = getSmOracleBackupComponentInfoFromBackupMetadata(metadata);
        boolean isAsmConfigured = isAsmConfigured(smOracleBackupComponentInfo
                .getSmOracleDatabaseInstance());
        if (isAsmConfigured) {
            ArrayOfSmOracleASMDiskGroup asmDiskGroups = smOracleBackupComponentInfo
                    .getSmOracleDatabaseInstance().getASMDetails()
                    .getASMDiskGroups();
            if (asmDiskGroups != null) {
                for (SmOracleASMDiskGroup asmDiskGroup : asmDiskGroups
                        .getSmOracleASMDiskGroup()) {
                    dataFilesLocations.getDataFilesLocation().add(
                            generateASMDataFilesLocation(asmDiskGroup.getName(), cloneDatabaseSid,
                                    StorageLocationType.ASM_DISK_GROUP));
                }
            }
        }

        // Mount points
        for (SmSnapshotInfo smSnapshotInfo : smOracleBackupComponentInfo
                .getSmSnapshots().getSmSnapshotInfo()) {
            SDStorageFootprint sdStorageFootprint = smSnapshotInfo
                    .getSnapshotLocator().getStorageFootprint();
            SDStorageResource sdStorageResource = sdStorageFootprint
                    .getHostResource();
            if (isAsmConfigured
                    && sdStorageResource.getResourceType().equals(
                            ResourceType.SD_HOST_DEVICE)) {
                /* Do not add SAN device if asm is configured */
                continue;
            }
            dataFilesLocations.getDataFilesLocation().add(
                    generateDataFilesLocation(
                            sdStorageResource.getResourceName(),
                            cloneDatabaseSid, StorageLocationType.MOUNT_POINT));
        }
        dataFilesConfiguration.setDataFilesLocations(dataFilesLocations);
        return dataFilesConfiguration;
    }

    private static SmOracleBackupComponentInfo getSmOracleBackupComponentInfoFromBackupMetadata(
            SmGetBackupDetailsResponse metadata) {
        ArrayOfSmBackup arrayOfSmBackup = metadata.getBackups();

        SmBackup smBackup = arrayOfSmBackup.getSmBackup().iterator().next();

        ArrayOfSmBackupComponentInfo arrayOfSmBackupComponentInfo = smBackup
                .getBackupComponents();

        SmOracleBackupComponentInfo smOracleBackupComponentInfo = (SmOracleBackupComponentInfo) arrayOfSmBackupComponentInfo
                .getSmBackupComponentInfo().iterator().next();

        return smOracleBackupComponentInfo;
    }

    private static boolean isAsmConfigured(SmOracleDbEnhanced smOracleDbEnhanced) {
        return (smOracleDbEnhanced != null
                && smOracleDbEnhanced.getASMDetails() != null && StringUtils
                    .isNotBlank(smOracleDbEnhanced.getASMDetails()
                            .getASMInstanceName()));
    }

    private static DataFilesLocation generateDataFilesLocation(
            String sourceLocation, String cloneSid,
            StorageLocationType storageLocationType) {
        DataFilesLocation dataFilesLocation = new DataFilesLocation();
        dataFilesLocation.setSourceLocation(sourceLocation);
        // Destination Mount point auto population
        dataFilesLocation.setCloneLocation(String.format("%s_%s",
                sourceLocation, cloneSid));
        dataFilesLocation.setLocationType(storageLocationType);
        return dataFilesLocation;
    }
    
    private static DataFilesLocation generateASMDataFilesLocation(
            String sourceLocation, String cloneSid,
            StorageLocationType storageLocationType) {
        DataFilesLocation dataFilesLocation = new DataFilesLocation();
        dataFilesLocation.setSourceLocation(String.format("+%s",sourceLocation));
        // Destination ASM disk group auto population
        dataFilesLocation.setCloneLocation(String.format("+SC_%s_%s",
                getHashCode(sourceLocation), cloneSid));
        dataFilesLocation.setLocationType(storageLocationType);
        return dataFilesLocation;
    }

    /**
     * This method will help in creating Oracle CLone Configuration required for
     * clone creation workflow with the help of clone specification file and
     * user input parameters.
     * 
     * @param newSmCloneCommand
     * @return {@link SmOracleCloneConfiguration}
     */
    public static SmOracleCloneConfiguration createSmOracleCloneConfiguration(
            NewSmCloneCommand newSmCloneCommand) {

        SMOCloneParameters smoCloneParameters = newSmCloneCommand
                .getSmoCloneParameters();
        OracleCloneSpecification cloneSpecification = smoCloneParameters
                .getOracleCloneSpecification();
        DatabaseSpecification databaseSpec = cloneSpecification
                .getDatabaseSpecification();
        StorageSpecification storageSpec = cloneSpecification
                .getStorageSpecification();

        SmOracleCloneConfiguration smOracleCloneConfiguration = new SmOracleCloneConfiguration();

        SmOracleRecoveryScopeParameters recoveryScopeParams = createOracleRecoveryScopeParameters(newSmCloneCommand
                .getSmoCloneParameters());
        smOracleCloneConfiguration
                .setRecoveryScopeParameters(recoveryScopeParams);

        // Set clone sid
        smOracleCloneConfiguration.setName(cloneSpecification
                .getCloneDatabaseSid().trim());

        // Set the host where clone operation need to be performed
        smOracleCloneConfiguration.setHost(newSmCloneCommand.getCloneToHost()
                .trim());

        ArrayOfSmHostResourceMapping arrayOfSmHostResourceMapping = new ArrayOfSmHostResourceMapping();
        arrayOfSmHostResourceMapping
                .getSmHostResourceMapping()
                .addAll(CustomSMSObjectMapperHelper
                        .createListOfSmHostResourceMappingsForOracleDatabaseFromAppObjectId(cloneSpecification
                                .getAppobjectid().trim()));
        smOracleCloneConfiguration.setResources(arrayOfSmHostResourceMapping);

        SmOracleDetail smOracleDetail = new SmOracleDetail();
        // Set Oracle Version
        smOracleDetail.setOracleFullVersion(databaseSpec.getOracleVersion()
                .trim());
        // Set Oracle Home
        smOracleDetail.setOracleHome(databaseSpec.getOracleHome().trim());
        smOracleCloneConfiguration.setOracleDetail(smOracleDetail);

        // Set archive log mode
        smOracleCloneConfiguration.setSetArchiveLogMode(databaseSpec
                .isEnableArchivelogMode());

        // Populate Oracle OS user and group
        OracleOsAccount oracleOsAccount = databaseSpec.getOracleOsAccount();
        if (oracleOsAccount != null) {
            SmOracleOsUserDetail smOracleOsUserDetail = new SmOracleOsUserDetail();
            smOracleOsUserDetail.setOsUserName(oracleOsAccount.getUsername()
                    .trim());
            smOracleOsUserDetail.setOsUserGroupName(oracleOsAccount.getGroup()
                    .trim());
            smOracleCloneConfiguration.setOsUserDetail(smOracleOsUserDetail);
        }

        // Set Db Auth if specified else set os auth
        DatabaseAuthentication databaseAuthentication = databaseSpec
                .getDatabaseAuthentication();
        SmAuth dbAuth = new SmAuth();
        if (databaseAuthentication != null
                && StringUtils
                        .isNotBlank(databaseAuthentication.getRunasName())) {
            dbAuth.setRunAsName(databaseAuthentication.getRunasName().trim());
            dbAuth.setAuthMode(SmAuthMode.ORACLE_DATABASE);
        } else {
            logger.debug(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_ORACLE_CLONESPEC_SET_OS_AUTH_SINCE_DB_AUTH_NOT_SPECIFIED));
            dbAuth.setAuthMode(SmAuthMode.NONE);
            smOracleCloneConfiguration.setOSAuthentication(true);
        }
        smOracleCloneConfiguration.setAuth(dbAuth);
        smOracleCloneConfiguration.setDatabasePort(databaseAuthentication
                .getPort());

        // Set ASM Auth if specified
        AsmAuthentication asmAuthentication = databaseSpec
                .getAsmAuthentication();
        SmAuth asmAuth = new SmAuth();
        if (asmAuthentication != null
                && StringUtils.isNotBlank(asmAuthentication.getRunasName())) {
            asmAuth.setRunAsName(asmAuthentication.getRunasName().trim());
            asmAuth.setAuthMode(SmAuthMode.ORACLE_ASM);
        } else {
            logger.debug(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_ORACLE_CLONESPEC_SKIP_ASM_AUTH_SINCE_NOT_SPECIFIED));
            asmAuth.setAuthMode(SmAuthMode.NONE);
        }
        smOracleCloneConfiguration.setSmAsmAuth(asmAuth);
        smOracleCloneConfiguration.setAsmPort(asmAuthentication.getPort());

        // Set SQL statements
        if (databaseSpec.getSqlStatements() != null
                && !databaseSpec.getSqlStatements().getSqlStatement().isEmpty()) {
            smOracleCloneConfiguration
                    .setGenericSQLQueries(createArrayOfSmOracleGenericQueryConfiguration(databaseSpec
                            .getSqlStatements()));
        }

        // Set Database parameters if any
        if (databaseSpec.getDatabaseParameters() != null
                && !databaseSpec.getDatabaseParameters().getDatabaseParameter()
                        .isEmpty()) {

            smOracleCloneConfiguration
                    .setCustomParameters(createArrayOfSmOracleCustomParameter(databaseSpec
                            .getDatabaseParameters()));
        }

        // Set Storage Mapping if any
        if (storageSpec.getStorageMapping() != null
                && storageSpec.getStorageMapping().getDataFilesConfiguration() != null
                && !storageSpec.getStorageMapping().getDataFilesConfiguration()
                        .getDataFilesLocations().getDataFilesLocation()
                        .isEmpty()) {
            smOracleCloneConfiguration
                    .setDataFileConfigurations(createArrayOfSmOracleDataFileConfiguration(storageSpec
                            .getStorageMapping().getDataFilesConfiguration()
                            .getDataFilesLocations()));
        }

        // Set redo log configuration
        if (databaseSpec.getRedologsConfiguration() != null
                && databaseSpec.getRedologsConfiguration().getRedologs() != null
                && !databaseSpec.getRedologsConfiguration().getRedologs()
                        .getRedogroup().isEmpty()) {
            smOracleCloneConfiguration
                    .setRedologFileConfigurations(createArrayOfSmOracleRedologFileConfiguration(databaseSpec
                            .getRedologsConfiguration().getRedologs()));
        }

        // Set control file configuration
        if (databaseSpec.getControlFilesConfiguration() != null
                && databaseSpec.getControlFilesConfiguration()
                        .getControlFiles() != null
                && !databaseSpec.getControlFilesConfiguration()
                        .getControlFiles().getControlFile().isEmpty()) {
            smOracleCloneConfiguration
                    .setControlFileConfigurations(createArrayOfSmOracleControlFileConfiguration(databaseSpec
                            .getControlFilesConfiguration().getControlFiles()));
        }

        // Setting PDb cOnfiguration if any
        if (databaseSpec.getPdbsConfiguration() != null
                && !databaseSpec.getPdbsConfiguration().getPdbConfiguration()
                        .isEmpty()) {
            smOracleCloneConfiguration
                    .setPluggableDatabasesConfiguration(createArrayOfSmOraclePluggableDatabaseConfiguration(databaseSpec
                            .getPdbsConfiguration().getPdbConfiguration()));
        }

        return smOracleCloneConfiguration;
    }

    /**
     * This method will help in creating Oracle CLone Configuration required for
     * clone creation workflow with the help of clone specification file and
     * user input parameters.
     * 
     * @param newSmCloneCommand
     * @return {@link SmOracleCloneConfiguration}
     */
    public static SmOracleCloneConfiguration createSmOracleCloneConfiguration(
            RefreshSmCloneCommand newSmCloneCommand) {

        SMOCloneParameters smoCloneParameters = newSmCloneCommand
                .getSmoCloneParameters();
        OracleCloneSpecification cloneSpecification = smoCloneParameters
                .getOracleCloneSpecification();
        DatabaseSpecification databaseSpec = cloneSpecification
                .getDatabaseSpecification();
        StorageSpecification storageSpec = cloneSpecification
                .getStorageSpecification();

        SmOracleCloneConfiguration smOracleCloneConfiguration = new SmOracleCloneConfiguration();

        SmOracleRecoveryScopeParameters recoveryScopeParams = createOracleRecoveryScopeParameters(newSmCloneCommand
                .getSmoCloneParameters());
        smOracleCloneConfiguration
                .setRecoveryScopeParameters(recoveryScopeParams);

        // Set clone sid
        smOracleCloneConfiguration.setName(cloneSpecification
                .getCloneDatabaseSid().trim());

        // Set the host where clone operation need to be performed
        smOracleCloneConfiguration.setHost(newSmCloneCommand.getCloneToHost()
                .trim());

        ArrayOfSmHostResourceMapping arrayOfSmHostResourceMapping = new ArrayOfSmHostResourceMapping();
        arrayOfSmHostResourceMapping
                .getSmHostResourceMapping()
                .addAll(CustomSMSObjectMapperHelper
                        .createListOfSmHostResourceMappingsForOracleDatabaseFromAppObjectId(cloneSpecification
                                .getAppobjectid().trim()));
        smOracleCloneConfiguration.setResources(arrayOfSmHostResourceMapping);

        SmOracleDetail smOracleDetail = new SmOracleDetail();
        // Set Oracle Version
        smOracleDetail.setOracleFullVersion(databaseSpec.getOracleVersion()
                .trim());
        // Set Oracle Home
        smOracleDetail.setOracleHome(databaseSpec.getOracleHome().trim());
        smOracleCloneConfiguration.setOracleDetail(smOracleDetail);

        // Set archive log mode
        smOracleCloneConfiguration.setSetArchiveLogMode(databaseSpec
                .isEnableArchivelogMode());

        // Populate Oracle OS user and group
        OracleOsAccount oracleOsAccount = databaseSpec.getOracleOsAccount();
        if (oracleOsAccount != null) {
            SmOracleOsUserDetail smOracleOsUserDetail = new SmOracleOsUserDetail();
            smOracleOsUserDetail.setOsUserName(oracleOsAccount.getUsername()
                    .trim());
            smOracleOsUserDetail.setOsUserGroupName(oracleOsAccount.getGroup()
                    .trim());
            smOracleCloneConfiguration.setOsUserDetail(smOracleOsUserDetail);
        }

        // Set Db Auth if specified else set os auth
        DatabaseAuthentication databaseAuthentication = databaseSpec
                .getDatabaseAuthentication();
        SmAuth dbAuth = new SmAuth();
        if (databaseAuthentication != null
                && StringUtils
                        .isNotBlank(databaseAuthentication.getRunasName())) {
            dbAuth.setRunAsName(databaseAuthentication.getRunasName().trim());
            dbAuth.setAuthMode(SmAuthMode.ORACLE_DATABASE);
        } else {
            logger.debug(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_ORACLE_CLONESPEC_SET_OS_AUTH_SINCE_DB_AUTH_NOT_SPECIFIED));
            dbAuth.setAuthMode(SmAuthMode.NONE);
            smOracleCloneConfiguration.setOSAuthentication(true);
        }
        smOracleCloneConfiguration.setAuth(dbAuth);
        smOracleCloneConfiguration.setDatabasePort(databaseAuthentication
                .getPort());

        // Set ASM Auth if specified
        AsmAuthentication asmAuthentication = databaseSpec
                .getAsmAuthentication();
        SmAuth asmAuth = new SmAuth();
        if (asmAuthentication != null
                && StringUtils.isNotBlank(asmAuthentication.getRunasName())) {
            asmAuth.setRunAsName(asmAuthentication.getRunasName().trim());
            asmAuth.setAuthMode(SmAuthMode.ORACLE_ASM);
        } else {
            logger.debug(MessageBundle
                    .localizeMessage(CLIMsgKey.CLI_ORACLE_CLONESPEC_SKIP_ASM_AUTH_SINCE_NOT_SPECIFIED));
            asmAuth.setAuthMode(SmAuthMode.NONE);
        }
        smOracleCloneConfiguration.setSmAsmAuth(asmAuth);
        smOracleCloneConfiguration.setAsmPort(asmAuthentication.getPort());

        // Set SQL statements
        if (databaseSpec.getSqlStatements() != null
                && !databaseSpec.getSqlStatements().getSqlStatement().isEmpty()) {
            smOracleCloneConfiguration
                    .setGenericSQLQueries(createArrayOfSmOracleGenericQueryConfiguration(databaseSpec
                            .getSqlStatements()));
        }

        // Set Database parameters if any
        if (databaseSpec.getDatabaseParameters() != null
                && !databaseSpec.getDatabaseParameters().getDatabaseParameter()
                        .isEmpty()) {

            smOracleCloneConfiguration
                    .setCustomParameters(createArrayOfSmOracleCustomParameter(databaseSpec
                            .getDatabaseParameters()));
        }

        // Set Storage Mapping if any
        if (storageSpec.getStorageMapping() != null
                && storageSpec.getStorageMapping().getDataFilesConfiguration() != null
                && !storageSpec.getStorageMapping().getDataFilesConfiguration()
                        .getDataFilesLocations().getDataFilesLocation()
                        .isEmpty()) {
            smOracleCloneConfiguration
                    .setDataFileConfigurations(createArrayOfSmOracleDataFileConfiguration(storageSpec
                            .getStorageMapping().getDataFilesConfiguration()
                            .getDataFilesLocations()));
        }

        // Set redo log configuration
        if (databaseSpec.getRedologsConfiguration() != null
                && databaseSpec.getRedologsConfiguration().getRedologs() != null
                && !databaseSpec.getRedologsConfiguration().getRedologs()
                        .getRedogroup().isEmpty()) {
            smOracleCloneConfiguration
                    .setRedologFileConfigurations(createArrayOfSmOracleRedologFileConfiguration(databaseSpec
                            .getRedologsConfiguration().getRedologs()));
        }

        // Set control file configuration
        if (databaseSpec.getControlFilesConfiguration() != null
                && databaseSpec.getControlFilesConfiguration()
                        .getControlFiles() != null
                && !databaseSpec.getControlFilesConfiguration()
                        .getControlFiles().getControlFile().isEmpty()) {
            smOracleCloneConfiguration
                    .setControlFileConfigurations(createArrayOfSmOracleControlFileConfiguration(databaseSpec
                            .getControlFilesConfiguration().getControlFiles()));
        }

        // Setting PDb cOnfiguration if any
        if (databaseSpec.getPdbsConfiguration() != null
                && !databaseSpec.getPdbsConfiguration().getPdbConfiguration()
                        .isEmpty()) {
            smOracleCloneConfiguration
                    .setPluggableDatabasesConfiguration(createArrayOfSmOraclePluggableDatabaseConfiguration(databaseSpec
                            .getPdbsConfiguration().getPdbConfiguration()));
        }

        return smOracleCloneConfiguration;
    }
    
    private static SmOracleRecoveryScopeParameters createOracleRecoveryScopeParameters(
            SMOCloneParameters smoCloneParameters) {
        SmOracleRecoveryScopeParameters recoveryScopeParams = new SmOracleRecoveryScopeParameters();

        recoveryScopeParams.setSkipRecovery(smoCloneParameters
                .isOracleSkipRecovery());

        recoveryScopeParams.setUntilScn(smoCloneParameters.getOracleUntilScn());

        recoveryScopeParams.setUntilCancel(smoCloneParameters.isOracleUntilCancel());

        if (smoCloneParameters.getOracleUntilTime() != null) {

            XMLGregorianCalendar untilTime = CLIUtils
                    .getXMLGregorianCalendarFromDate(smoCloneParameters
                            .getOracleUntilTime());
            recoveryScopeParams.setUnitlTime(untilTime);

        }

        if (CLIUtils.isNotNullAndNotEmptyCollection(smoCloneParameters
                .getAlternateArchiveLogPaths())) {
            ArrayOfSmPath logPaths = new ArrayOfSmPath();
            for (String logPath : smoCloneParameters
                    .getAlternateArchiveLogPaths()) {
                SmPath alternateArchiveLogsPath = new SmPath();
                alternateArchiveLogsPath.setPath(logPath);
                logPaths.getSmPath().add(alternateArchiveLogsPath);
            }
            recoveryScopeParams.setLogPaths(logPaths);
        }

        return recoveryScopeParams;
    }

    private static ArrayOfSmOracleGenericQueryConfiguration createArrayOfSmOracleGenericQueryConfiguration(
            SqlStatements sqlStatements) {
        ArrayOfSmOracleGenericQueryConfiguration arrayOfSmOracleGenericQueryConfiguration = new ArrayOfSmOracleGenericQueryConfiguration();

        for (String sqlStatement : sqlStatements.getSqlStatement()) {

            if (StringUtils.isBlank(sqlStatement)) {
                continue;
            }

            SmOracleGenericQueryConfiguration smOracleGenericQueryConfiguration = new SmOracleGenericQueryConfiguration();
            smOracleGenericQueryConfiguration.setSQLQuery(StringUtils
                    .removeEnd(sqlStatement.trim(), SEMICOLON));

            arrayOfSmOracleGenericQueryConfiguration
                    .getSmOracleGenericQueryConfiguration().add(
                            smOracleGenericQueryConfiguration);
        }
        return arrayOfSmOracleGenericQueryConfiguration;
    }

    private static ArrayOfSmOracleCustomParameter createArrayOfSmOracleCustomParameter(
            DatabaseParameters databaseParameters) {
        ArrayOfSmOracleCustomParameter arrayOfSmOracleCustomParameter = new ArrayOfSmOracleCustomParameter();
        for (DatabaseParameter param : databaseParameters
                .getDatabaseParameter()) {
            if (StringUtils.isEmpty(param.getValue())) {
                logger.debug("Skipping addition of database parameter "
                        + param.getName() + " since it has no value");
                continue;
            }
            SmOracleCustomParameter smOracleCustomParameter = new SmOracleCustomParameter();
            SmKeyValue smKeyValue = new SmKeyValue();
            smKeyValue.setKey(param.getName().trim());
            smKeyValue.setValue(param.getValue().trim());
            smOracleCustomParameter.setParameter(smKeyValue);
            // User entered parameters are always overridden
            smOracleCustomParameter.setDefault(true);
            arrayOfSmOracleCustomParameter.getSmOracleCustomParameter().add(
                    smOracleCustomParameter);
        }
        return arrayOfSmOracleCustomParameter;
    }

    private static ArrayOfSmOracleDataFileConfiguration createArrayOfSmOracleDataFileConfiguration(
            DataFilesLocations dataFilesLocations) {
        ArrayOfSmOracleDataFileConfiguration arrayOfSmOracleDataFileConfiguration = new ArrayOfSmOracleDataFileConfiguration();

        for (DataFilesLocation dataFilesLocation : dataFilesLocations
                .getDataFilesLocation()) {
            SmOracleDataFileConfiguration smOracleDataFileConfiguration = new SmOracleDataFileConfiguration();
            String sourceLocation = dataFilesLocation.getSourceLocation();
            String destLocation = dataFilesLocation.getCloneLocation();
            if (dataFilesLocation.getLocationType().equals(
                    StorageLocationType.ASM_DISK_GROUP)) {
                sourceLocation = sourceLocation.replace("+", "");
                destLocation = destLocation.replace("+", "");
            }
            smOracleDataFileConfiguration.setSourceLocation(sourceLocation
                    .trim());
            smOracleDataFileConfiguration.setCloneLocation(destLocation.trim());
            LocationType locationType = LocationType.valueOf(dataFilesLocation
                    .getLocationType().name());
            smOracleDataFileConfiguration.setDiskLocationType(locationType);
            arrayOfSmOracleDataFileConfiguration
                    .getSmOracleDataFileConfiguration().add(
                            smOracleDataFileConfiguration);
        }
        return arrayOfSmOracleDataFileConfiguration;
    }

    private static ArrayOfSmOracleRedologFileConfiguration createArrayOfSmOracleRedologFileConfiguration(
            Redologs redologs) {
        ArrayOfSmOracleRedologFileConfiguration arrayOfSmOracleRedologFileConfiguration = new ArrayOfSmOracleRedologFileConfiguration();
        for (Redogroup redogroup : redologs.getRedogroup()) {
            long totalSize = getSizeInMegaBytesFromSizeType(redogroup.getTotalSize());
            for (RedologFile redofile : redogroup.getRedologFiles()
                    .getRedologFile()) {
                SmOracleRedologFileConfiguration smOracleRedologFileConfiguration = new SmOracleRedologFileConfiguration();
                smOracleRedologFileConfiguration.setDirectoryPath(redofile
                        .getPath().trim());
                smOracleRedologFileConfiguration
                        .setThreadNumber((int) redogroup.getNumber());
                smOracleRedologFileConfiguration.setTotalSize(totalSize);
                smOracleRedologFileConfiguration.setBlockSize(REDO_BLOCK_SIZE);
                arrayOfSmOracleRedologFileConfiguration
                        .getSmOracleRedologFileConfiguration().add(
                                smOracleRedologFileConfiguration);
            }
        }
        return arrayOfSmOracleRedologFileConfiguration;
    }

    private static long getSizeInMegaBytesFromSizeType(SizeType size) {
        switch (size.getUnit()) {
            case B:
                return size.getValue() / 1024 / 1024;
            case K:
                return size.getValue() / 1024;
            case M:
                return size.getValue();
            case G:
                return size.getValue() * 1024;
            case T:
                return size.getValue() * 1024 * 1024;
            default:
                return size.getValue();
        }
    }

    private static ArrayOfSmOracleControlFileConfiguration createArrayOfSmOracleControlFileConfiguration(
            ControlFiles controlFiles) {
        ArrayOfSmOracleControlFileConfiguration arrayOfSmOracleControlFileConfiguration = new ArrayOfSmOracleControlFileConfiguration();

        for (ControlFile cf : controlFiles.getControlFile()) {
            SmOracleControlFileConfiguration smOracleControlFileConfiguration = new SmOracleControlFileConfiguration();
            smOracleControlFileConfiguration.setDirectoryPath(cf.getPath()
                    .trim());
            arrayOfSmOracleControlFileConfiguration
                    .getSmOracleControlFileConfiguration().add(
                            smOracleControlFileConfiguration);
        }
        return arrayOfSmOracleControlFileConfiguration;
    }

    private static ArrayOfSmOraclePluggableDatabaseConfiguration createArrayOfSmOraclePluggableDatabaseConfiguration(
            List<PdbConfigurationType> pdbs) {
        ArrayOfSmOraclePluggableDatabaseConfiguration arrayOfSmOraclePluggableDatabaseConfiguration = new ArrayOfSmOraclePluggableDatabaseConfiguration();
        for (PdbConfigurationType pdb : pdbs) {
            if (StringUtils.isBlank(pdb.getNewName())) {
                logger.debug("Skipping addition of pdb renaming for "
                        + pdb.getCurrentName()
                        + " since new name is not specified.");
                continue;
            }
            SmOraclePluggableDatabaseConfiguration smOraclePluggableDatabaseConfiguration = new SmOraclePluggableDatabaseConfiguration();
            smOraclePluggableDatabaseConfiguration.setCurrentName(pdb
                    .getCurrentName().trim());
            smOraclePluggableDatabaseConfiguration.setNewName(pdb.getNewName()
                    .trim());
            arrayOfSmOraclePluggableDatabaseConfiguration
                    .getSmOraclePluggableDatabaseConfiguration().add(
                            smOraclePluggableDatabaseConfiguration);
        }
        return arrayOfSmOraclePluggableDatabaseConfiguration;
    }

    public static ArrayOfSmArchivedLocator createSecondaryLocatorsForDatafilesFromOracleCloneSpecification(
            OracleCloneSpecification oracleCloneSpecification) {
        ArrayOfSmArchivedLocator arrayOfSmArchivedLocator = new ArrayOfSmArchivedLocator();
        if (oracleCloneSpecification != null
                && oracleCloneSpecification.getStorageSpecification() != null) {
            StorageMapping storageMapping = oracleCloneSpecification
                    .getStorageSpecification().getStorageMapping();
            if (storageMapping != null
                    && storageMapping.getSecondaryConfigurationDatafiles() != null) {
                SecondaryLocators secondaryLocators = storageMapping
                        .getSecondaryConfigurationDatafiles().getSecondaryLocators();
                arrayOfSmArchivedLocator
                        .getSmArchivedLocator()
                        .addAll(getListOfArchivedLocatorsFromSecondaryLocators(secondaryLocators));
            }
        }
        return arrayOfSmArchivedLocator;
    }

    public static ArrayOfSmArchivedLocator createSecondaryLocatorsForArchivelogsFromOracleCloneSpecification(
            OracleCloneSpecification oracleCloneSpecification) {
        ArrayOfSmArchivedLocator arrayOfSmArchivedLocator = new ArrayOfSmArchivedLocator();
        if (oracleCloneSpecification != null
                && oracleCloneSpecification.getStorageSpecification() != null) {
            StorageMapping storageMapping = oracleCloneSpecification
                    .getStorageSpecification().getStorageMapping();
            if (storageMapping != null
                    && storageMapping.getSecondaryConfigurationArchivelogs() != null) {
                SecondaryLocators secondaryLocators = storageMapping
                        .getSecondaryConfigurationArchivelogs().getSecondaryLocators();
                arrayOfSmArchivedLocator
                        .getSmArchivedLocator()
                        .addAll(getListOfArchivedLocatorsFromSecondaryLocators(secondaryLocators));
            }
        }
        return arrayOfSmArchivedLocator;
    }

    private static List<SmArchivedLocator> getListOfArchivedLocatorsFromSecondaryLocators(
            SecondaryLocators secondaryLocators) {
        List<SmArchivedLocator> smArchivedLocators = new ArrayList<>();
        if (secondaryLocators != null) {
            for (SecondaryLocator sl : secondaryLocators.getSecondaryLocator()) {
                if (sl.getPrimary() != null && sl.getSecondary() != null) {
                    SmArchivedLocator smArchivedLocator = createSmArchivedLocator(sl);
                    smArchivedLocators.add(smArchivedLocator);
                }
            }
        }
        return smArchivedLocators;
    }

    private static SmArchivedLocator createSmArchivedLocator(SecondaryLocator sl) {
        SmArchivedLocator smArchivedLocator = new SmArchivedLocator();

        SnapshotLocator primary = new SnapshotLocator();
        primary.setStorageSystemName(sl.getPrimary().getSvm());
        primary.setStorageVolumeName(sl.getPrimary().getVolume());
        smArchivedLocator.setPrimaryLocator(primary);

        SnapshotLocator secondary = new SnapshotLocator();
        secondary.setStorageSystemName(sl.getSecondary().getSvm());
        secondary.setStorageVolumeName(sl.getSecondary().getVolume());
        smArchivedLocator.setSecondaryLocator(secondary);

        return smArchivedLocator;
    }
}