/*
 * Copyright (c) 2006-2014 Rogério Liesenfeld
 * This file is subject to the terms of the MIT license (see LICENSE.txt).
 */
package mockit.internal.state;

import org.jetbrains.annotations.*;

import mockit.*;
import mockit.internal.mockups.*;

public final class MockClasses
{
   @NotNull private final MockInstances regularMocks;
   @NotNull private final MockInstances startupMocks;
   @NotNull private final MockStates mockStates;

   MockClasses()
   {
      regularMocks = new MockInstances();
      startupMocks = new MockInstances();
      mockStates = new MockStates();
   }

   @NotNull public MockInstances getRegularMocks() { return regularMocks; }
   @NotNull public MockStates getMockStates() { return mockStates; }

   public int addMock(boolean forStartupMock, @NotNull MockUp<?> mock)
   {
      if (forStartupMock) {
         return -startupMocks.addMock(mock) - 1;
      }

      return regularMocks.addMock(mock);
   }

   @NotNull MockUp<?> getMock(int index)
   {
      return index >= 0 ? regularMocks.getMock(index) : startupMocks.getMock(-index - 1);
   }

   @Nullable public MockUp<?> findMock(Class<?> mockClass)
   {
      MockUp<?> mock = regularMocks.findMock(mockClass);
      if (mock == null) mock = startupMocks.findMock(mockClass);
      return mock;
   }

   public void removeMock(@NotNull MockUp<?> mock)
   {
      regularMocks.removeInstance(mock);
      startupMocks.removeInstance(mock);
   }
}
